// Plot_Table_Columns.java

import ij.*;
import ij.gui.*;
import ij.text.*;
import ij.measure.ResultsTable;
import ij.plugin.*;
import ij.plugin.PlugIn;
import ij.Prefs;
import ij.process.*;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.event.*;
import java.util.Hashtable;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;

import java.awt.event.*;

import java.lang.Math.*;
import java.text.*;

// astroj ------
import astroj.*;
// ------ astroj


/**
 * This plugin plots any number of columns from a Results or MeasurementTable.  This plugin
 * is based on the Plot_Columns plugin by F.V. Hessman.
 *
 * @author F.V. Hessman, Georg-August-Universitaet Goettingen
 * @date 2009-FEB-09
 * @version 1.0
 *
 * @author K.A. Collins, University of Louisville, Kentucky USA
 * @date 2010-JUN-09
 * @version 2.1
 * @changes New GUI interface, support for multiple column plotting, error plotting
 * support, interactive shifting and scaling, titles, subtitles, and legends, custom,
 * semi-custom, and auto scaling
 *
 *
 * 
 */

/**
 * This plugin plots any two columns from a Results or MeasurementTable.
 * @author F.V. Hessman, Georg-August-Universitaet Goettingen
 * @date 2009-FEB-09
 * @version 1.0
 *
 * To create version outside of the Astronomy plugins, comment in the "// plain ..." lines and comment out the "// astroj ..." lines
 */
public class Plot_IJ implements PlugIn, MouseListener, MouseMotionListener, MouseWheelListener {

        String title = "title";
        double titlePosX = .5;
        double titlePosY = 0.06;
        String subtitle ="subtitle";
        double subtitlePosX = 0.5;
        double subtitlePosY = 0.1;
        double legendPosX = 0.8;
        double legendPosY = 0.2;
        boolean button2Drag = false;

        String xlabel = "J.D.-2400000";
        int xcolumn;
        int plotSizeX = 500;
        int plotSizeY = 350;
        double plotMinX;
        double plotMinY;
        double plotMaxX;
        double plotMaxY;
        int xExponent = 0;
        int yExponent = 0;
        int mmagrefs = 10;
        String xLegend = "";
        String yLegend = "";
        boolean legendLeft = false;
        boolean legendRight = false;
        boolean useTitle = false;
        boolean useSubtitle = false;
        boolean useXCustomName = false;
        boolean useYCustomName = false;
        boolean useXColumnName = true;
        boolean useYColumnName = true;
        boolean xTics = true;
        boolean yTics = true;
        boolean xGrid = false;
        boolean yGrid = true;
        boolean xNumbers = true;
        boolean yNumbers = true;
        boolean plotAutoMode = false;
        boolean nextPanel = false;
        boolean canceled = false;
        boolean waitSecs = false;
        boolean openDataSetWindow = false;
        boolean rememberWindowLocations = true;
        boolean autoScaleX = true;
        boolean useFirstX = false;
        boolean autoScaleY = true;
        boolean showToolTips = true;
        boolean showXScaleInfo = true;
        boolean showYScaleInfo = true;
        boolean showYShiftInfo = true;
        boolean showLScaleInfo = true;
        boolean showLShiftInfo = true;
        boolean showYBinInfo = true;
        boolean showLBinInfo = true;
        boolean showYmmagInfo = true;
        boolean showLmmagInfo = true;
        boolean showYSymbolInfo = false;
        boolean showLSymbolInfo = true;
        boolean useTwoLineLegend = false;
        boolean useWideDataPanel = false;
        boolean subframeWasShowing = false;
        boolean newPlotWindow = false;
        boolean useDefaultSettings = false;
        boolean tableHasText = false;
        boolean setSubpanelVisible = false;

        double xMin = 0.0;
        double xMax = 0.0;
        double xWidth = 0.3;
        double yMin = 0.0;
        double yMax = 0.0;
        double xautoscalemin = 0.0;
        double xautoscalemax = 0.0;
        double yautoscalemin = 0.0;
        double yautoscalemax = 0.0;
        double yRange = 0.0;

        int cur;
        int mainFrameLocationX = 40;
        int mainFrameLocationY = 40;
        int subFrameLocationX = 40;
        int subFrameLocationY = 40;
        int plotFrameLocationX = 40;
        int plotFrameLocationY = 40;

        int excludedHeadSamples = 0;
        int excludedTailSamples = 0;
        int excludedHeadSamplesStep = 1;
        int excludedTailSamplesStep = 1;

        double xWidthStep = 0.01;
        double xMaxStep = 0.01;
        double xMinStep = 1.0;
        double yMaxStep = 1.0;
        double yMinStep = 1.0;
        double yMid = 0.0;
        int plotSizeXStep = 10;
        int plotSizeYStep = 10;

        int startDragX, startDragY, endDragX, endDragY;
        int startDragSubImageX, startDragSubImageY;
        int startDragScreenX, startDragScreenY;
        int newPositionX, newPositionY;


        String temptextfield;
        Integer tempIntField;

        JFrame mainFrame = new JFrame ("Plot_IJ Main");
        JFrame subFrame;
        JFrame getMaxCurvesFrame;
        Frame plotFrame;
        Panel plotpanel = new Panel();
        Label coordinateslabel = new Label("This is a test.");
        JPanel mainsubpanel;
        JScrollPane subscrollpane;

        JMenuBar mainmenubar = new JMenuBar();
        JMenu filemenu = new JMenu("File");
        JMenu preferencesmenu = new JMenu("Preferences");
        JCheckBoxMenuItem opendatasetCB;
        JCheckBoxMenuItem usewidedataCB;
        JCheckBoxMenuItem rememberwindowlocationsCB;
        JCheckBoxMenuItem showtooltipsCB;
        JCheckBoxMenuItem usedefaultsettingsCB;

        JTextField titleField, subtitleField, legendField, xlegendField, ylegendField;
        JLabel statusLabel, xcolumnlabel, ycolumnslabel, ycolumnlabel, operatorlabel;
        JLabel xcolumnlabelsub, ycolumnlabelsub, operatorlabelsub;
        JLabel opcolumnlabel, markerlabel, markercolorlabel;
        JLabel opcolumnlabelsub, markerlabelsub, markercolorlabelsub;
        JSlider titlePosYSlider;
        JSlider titlePosXSlider;
        JSlider subtitlePosYSlider;
        JSlider subtitlePosXSlider;
        JSlider legendPosYSlider;
        JSlider legendPosXSlider;        
        JComboBox xdatacolumn;



        SpinnerModel mmagrefsmodel;
        JSpinner mmagrefsspinner;
        SpinnerModel maxcurvesmodel;
        JSpinner maxcurvesspinner;

        JSpinner xwidthstepspinner,xmaxstepspinner, xminstepspinner,ymaxstepspinner,yminstepspinner;
        JSpinner plotwidthstepspinner, plotheightstepspinner;

        JPopupMenu xwidthsteppopup, xmaxsteppopup, xminsteppopup,ymaxsteppopup,yminsteppopup;
        JPopupMenu plotwidthsteppopup, plotheightsteppopup, trimdataheadsteppopup, trimdatatailsteppopup;
        JPopupMenu xaxispopup, yaxispopup, legendpopup;

        SpinnerModel xwidthstepmodel, xmaxstepmodel, xminstepmodel,ymaxstepmodel,yminstepmodel;
        SpinnerModel plotwidthstepmodel, plotheightstepmodel;
        SpinnerModel xwidthspinnermodel, xminspinnermodel, xmaxspinnermodel;
        SpinnerModel yminspinnermodel, ymaxspinnermodel, plotheightspinnermodel, plotwidthspinnermodel;
        SpinnerModel xmultipliermodel, ymultipliermodel;
        SpinnerModel trimdataheadmodel, trimdatatailmodel, trimdataheadstepmodel, trimdatatailstepmodel;


        JSpinner xwidthspinner, xminspinner, xmaxspinner, xmultiplierspinner, ymultiplierspinner;
        JSpinner yminspinner, ymaxspinner, plotheightspinner, plotwidthspinner;
        JSpinner trimdataheadspinner, trimdatatailspinner, trimdataheadstepspinner, trimdatatailstepspinner;


        JCheckBox usextickscheckbox, useytickscheckbox, usexgridcheckbox;
        JCheckBox useygridcheckbox, usexnumberscheckbox, useynumberscheckbox;

        JButton moreybutton, closebutton, grabautoxbutton, grabautoybutton, updateplotbutton;

        String tableName = "Results";
        String[] columns;
        String[] tables;
        MeasurementTable table;
        JLabel dummylabel1, binsizespinnerlabel, mmagrefsspinnerlabel, dummylabel4;

        ImageWindow plotWindow;
        ImagePlus plotImage;
        ImageProcessor ip;
        ImageCanvas plotImageCanvas;
        OverlayCanvas plotOverlayCanvas;
        Panel plotbottompanel;
        Label plotcoordlabel;
        Vector<Roi> list = new Vector<Roi>();
        GeneralPath path = new GeneralPath();
        Roi roi;
        Robot robot;
        

        int maxCurves;
        String[] ylabel;
        int[] ycolumn;
        int[] errcolumn;
        int[] operrcolumn;
        String[] oplabel;
        int[] opcolumn;
        String[] errlabel;
        String[] operrlabel;
        boolean[] showErrors;
        boolean[] showOpErrors;
        boolean[] lines;
        int[] marker;
        Color[] color;
        int[] markerIndex;
        int[] colorIndex;
        int[] operatorIndex;
        String[] cblabels;
        boolean[] moreOptions;
        double[] autoScaleFactor;
        double[] autoScaleStep;
        double[] autoShiftFactor;
        double[] autoShiftStep;
        double[] customScaleFactor;
        double[] customScaleStep;
        double[] customShiftFactor;
        double[] customShiftStep;
        double[] baseline;
        double[] subtotalScaleFactor;
        double[] subtotalShiftFactor;
        double[] totalScaleFactor;
        double[] totalShiftFactor;
        double[] yWidth;
        double[] yMinimum;
        double[] yMaximum;
        double[] yMidpoint;

        boolean[] mmag;
        boolean[] plotY;
        boolean[] useColumnName;
        boolean[] useLegend;
        boolean[] autoScale;
        boolean[] ASInclude;

        int[] binSize;

        String[] legend;


        JCheckBox[] usecurvebox;
        JCheckBox[] errorcolumnbox;
        JCheckBox[] operrorcolumnbox;
        JCheckBox[] usebinbox;
        JCheckBox[] usemmagbox;
        JCheckBox[] uselinesbox;
        JCheckBox[] autoscaleincludebox;

        JComboBox[] ydatacolumn;
        JComboBox[] operatorselection;
        JComboBox[] operatorcolumn;
        JComboBox[] errorcolumn;
        JComboBox[] operrorcolumn;
        JComboBox[] markersymbolselection;
        JComboBox[] markercolorselection;

        SpinnerModel[] binsizespinnermodel;
        JSpinner[] binsizespinner;
        JPanel[] morelegendradiopanelgroup;
        JPanel[] autoscalepanelgroup;
        JPanel[] customscalepanelgroup;
        JRadioButton[] legendnoneButton;
        JRadioButton[] legendcustomNameButton;
        JRadioButton[] legendcolumnNameButton;
        JRadioButton[] autoscaleButton;
        JRadioButton[] customscaleButton;
        ButtonGroup[] morelegendRadioGroup;
        ButtonGroup[] autoscaleRadioGroup;
        JTextField[] morelegendField;
        SpinnerModel[] autoshiftmodel;
        SpinnerModel[] autoshiftstepmodel;
        SpinnerModel[] autoscalemodel;
        SpinnerModel[] autoscalestepmodel;
        SpinnerModel[] customshiftmodel;
        SpinnerModel[] customshiftstepmodel;
        SpinnerModel[] customscalemodel;
        SpinnerModel[] customscalestepmodel;
        JSpinner[] autoshiftspinner;
        JSpinner[] autoshiftstepspinner;
        JSpinner[] autoscalespinner;
        JSpinner[] autoscalestepspinner;
        JSpinner[] customshiftspinner;
        JSpinner[] customshiftstepspinner;
        JSpinner[] customscalespinner;
        JSpinner[] customscalestepspinner;
        JPopupMenu[] autoscalesteppopup;
        JPopupMenu[] autoshiftsteppopup;
        JPopupMenu[] customscalesteppopup;
        JPopupMenu[] customshiftsteppopup;
        JPanel[] autoscalesteppanel;
        JPanel[] customscalesteppanel;
        JPanel[] autoshiftsteppanel;
        JPanel[] customshiftsteppanel;
        JLabel[] autoscalesteplabel;
        JLabel[] autoshiftsteplabel;
        JLabel[] customscalesteplabel;
        JLabel[] customshiftsteplabel;
        JLabel[] curvelabel;
        JLabel[] othercurvelabel;
        JPanel[] grabautopanel;
        JButton[] grabautobutton;

        JSpinner.NumberEditor[] customscaleeditor;
        DecimalFormat[] customscaleformat;
        JSpinner.NumberEditor[] customshifteditor;
        DecimalFormat[] customshiftformat;

        DecimalFormat fourPlaces = new DecimalFormat("0.0000");

        String[] markers = new String[]
                {
                "box",
                "circle",
                "cross",
                "dot",
                "line",
                "triangle",
                "X"
                };
        String[] colors = new String[]
                {
                "black",
                "dark gray",
                "gray",
                "light gray",
                "green",
                "cyan",
                "blue",
                "magenta",
                "pink",
                "red",
                "orange",
                "yellow",
                };

        String[] operators = new String[]
                {
                "none",
                "divide by",
                "multiply by",
                "subtract",
                "add",
                };

        String[] opSymbol = new String[]
                {
                "",
                " / ",
                " * ",
                " - ",
                " + ",
                };

        String[] spinnerscalelist = new String[]
                {
                "      0.0000000001",
                "        0.000000001",
                "          0.00000001",
                "            0.0000001",
                "              0.000001",
                "                0.00001",
                "                  0.0001",
                "                    0.001",
                "                    0.010",
                "                    0.100",
                "                    0.500",
                "                    1.000",
                "                  10.000",
                "                100.000",
                "              1000.000",
                "            10000.000",
                "          100000.000",
                "        1000000.000",
                "      10000000.000",
                "    100000000.000",
                "1000000000.000",
                };

        String[] integerspinnerscalelist = new String[]
                {
                "                    1",
                "                    2",
                "                    3",
                "                    4",
                "                    5",
                "                    6",
                "                    7",
                "                    8",
                "                    9",
                "                  10",
                "                  25",
                "                  50",
                "                100",
                "                250",
                "                500",
                "              1000",
//                "            10000",
//                "          100000",
//                "        1000000",
//                "      10000000",
//                "    100000000",
//                "1000000000",
                };

       
    
	public void run(String arg)
		{
		if (IJ.versionLessThan("1.27t"))
			{
			IJ.showMessage ("Your version of ImageJ is too old!");
			return;
			}
        //SET DEFAULT SYSTEM LOOK AND FEEL
        if (IJ.isWindows() || IJ.isMacOSX())
                {
                try {UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());}
                catch (Exception e) { }
                }
        else
                {
                try {UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");}
                catch (Exception e) { }
                }
        
        setupArrays();

        useDefaultSettings = Prefs.get("plot.useDefaultSettings", useDefaultSettings);
        if (useDefaultSettings)
                {
                useDefaultSettings = false;
                Prefs.set("plot.useDefaultSettings", useDefaultSettings);
                }
        else
                getPreferences();

		// FIND ALL TABLES
		
        tables = MeasurementTable.getMeasurementTableNames();

        if (tables == null || tables.length == 0)
                {
                IJ.showMessage ("No measurement tables to be accessed.");
                return;
                }

        //filter out B&C, Log, Errors windows if they exist
        int j = 0;
        for (int i=0; i < tables.length; i++)
                if (tables[i].equals("B&C") || tables[i].equals("Log") || tables[i].equals("Errors"))
                       j++;
        String[] filteredTables = new String[tables.length-j];
        j = 0;
        for (int i=0; i < tables.length; i++)
                if (!tables[i].equals("B&C") && !tables[i].equals("Log") && !tables[i].equals("Errors"))
                        {
                        filteredTables[j] = tables[i];
                        j++;
                        }
        tables = filteredTables;
        if (!plotAutoMode)
            {
            if (tables.length == 0)
                    {
                    IJ.showMessage ("No measurement tables to be accessed.");
                    return;
                    }
            else if (tables.length == 1)
                    {
                    tableName = tables[0];
                    }
            else                            // IF MORE THAN ONE, ASK WHICH TABLE SHOULD BE USED
                    {
                    GenericDialog gd = new GenericDialog("Plot Table Columns");
                    gd.addMessage ("Select table to be plotted.");
                    gd.addChoice ("Table to be plotted",tables,"Measurements");
                    gd.showDialog();
                    if (gd.wasCanceled()) return;
                    tableName = gd.getNextChoice();
                    }
            }

        Prefs.set("plot.tableName",tableName);

        table = MeasurementTable.getTable(tableName);
                


		if (table == null)
			{
			IJ.showMessage ("Cannot access table called \""+tableName+"\"");
			return ;
			}

		columns = table.getColumnHeadings().split("\t");

		if (columns == null || columns.length == 0)
			{
			IJ.showMessage ("No data columns to plot.");
			return;
			}
        else if (columns.length <= 2)
            {
            table = getTableFromTextPanel(tableName);
            if (table == null)
                return;
            if (tableName.endsWith(".txt"))
                tableName = "Measurements in "+tableName;
            table = MeasurementTable.getTable(tableName);
            }
        else if (!columns[1].equalsIgnoreCase("Label") && !columns[1].equalsIgnoreCase("image"))
            {
            table = getTableFromTextPanel(tableName);
            if (table == null)
                return;
            if (tableName.endsWith(".txt"))
                tableName = "Measurements in "+tableName;
            table = MeasurementTable.getTable(tableName);
            }
        columns = table.getColumnHeadings().split("\t");
        j = 0;
        for (int i=0; i < columns.length; i++)
                if (columns[i].equals("Label") || columns[i].equals("image"))
                       j++;
        String[] filteredColumns = new String[columns.length - j];
        j = 0;
        for (int i=0; i < columns.length; i++)
                if (!columns[i].equals("Label") && !columns[i].equals("image") )
                        {
                        filteredColumns[j] = columns[i];
                        j++;
                        }
        columns = filteredColumns;
 
        int n = table.getCounter();
        if (n < 1)
			{
			IJ.showMessage ("No data in table "+tableName+" to plot.");
			return;
			}
                
		xcolumn = table.getColumnIndex (xlabel);
        if (xcolumn == ResultsTable.COLUMN_NOT_FOUND)
                xlabel = columns[0];

        for (int i=0;i<maxCurves;i++)
            {
            ycolumn[i] = table.getColumnIndex (ylabel[i]);
            errcolumn[i] = table.getColumnIndex (errlabel[i]);
            opcolumn[i] = table.getColumnIndex (oplabel[i]);
            operrcolumn[i] = table.getColumnIndex (operrlabel[i]);
            if (ycolumn[i] == ResultsTable.COLUMN_NOT_FOUND)
                    ylabel[i] = columns[0];
            if (errcolumn[i] == ResultsTable.COLUMN_NOT_FOUND)
                    errlabel[i] = columns[0];
            if (opcolumn[i] == ResultsTable.COLUMN_NOT_FOUND)
                    oplabel[i] = columns[0];
            if (operrcolumn[i] == ResultsTable.COLUMN_NOT_FOUND)
                    operrlabel[i] = columns[0];
            }

        showMainJPanel();

        updatePlot();
        if (plotAutoMode)
            {
            mainFrame.dispose();
            subFrame.dispose();
            }
        return;
		}

    public MeasurementTable getTableFromTextPanel (String tableName)
		{                 //based on MeasurementTable.getTable
		int n;
		String str="";
        String[] lines;
        String newTableName;
        StringTokenizer parser;
        boolean isTable = false;
        MeasurementTable table;

		// GET CONTENTS OF EXISTING TextPanel or TextArea

		TextPanel panel = MeasurementTable.getTextPanel(tableName);

        if (panel == null)
            {                             // no TextPanel Object found, try to open as a TextArea Object
            TextArea textArea = (TextArea)WindowManager.getFrame(tableName).getComponent(0);
            if (textArea == null)
                {
                IJ.showMessage ("Format of table "+tableName+" is not supported.");
                return null;
                }
            lines = textArea.getText().split("\n");
            if (lines.length < 2)
                {
                IJ.showMessage ("Table "+tableName+" has less than two rows.\n"
                        + "A valid table must have at least a heading row and one data row.");
                return null;
                }
            newTableName = "Measurements in "+tableName;
            table = new MeasurementTable(newTableName);

            // GET HEADERS
            String labelHeading = "Label";
            String[] headers = lines[0].split("\t");
            if (headers.length >= 2)
                if (headers[1].equalsIgnoreCase("Label") || headers[1].equalsIgnoreCase("image"))
                    {
                    isTable = true;
                    }

            // GET EXISTING DATA
            n = lines.length;
            int startColumn = 0;
            for (int row=0; row < n - 1; row++)
                {
                parser = new StringTokenizer (lines[row + 1],"\t");
                table.incrementCounter();
                if (isTable)
                    {
                    str = parser.nextToken();   //discard line number
                    table.addLabel(headers[1], parser.nextToken());
                    startColumn = 2;
//                    IJ.log("path 1");
                    }
                else
                    {
                    table.addLabel(labelHeading, tableName);
                    startColumn = 0;
//                    IJ.log("path 2");
                    }

                for (int column = startColumn; column < headers.length; column++)
                    {

                    if (parser.hasMoreTokens() && (str = parser.nextToken()) != null)
                        {
                        try	{
                            double d = Double.parseDouble(str);
                            table.addValue(headers[column],d);
                            }
                        catch (NumberFormatException e)
                            {
                            IJ.showMessage ("Table "+tableName+" contains text.");
                            return null;
                            }
                        }
                    else
                        {
                        table.addValue(headers[column],0.0);

                        }
                    }
                }

            table.show(newTableName);
            return table;
            }
        else
            {
            // CREATE EMPTY TABLE
            newTableName = "Measurements in "+tableName;
            table = new MeasurementTable(newTableName);

            String labelHeading = "Label";
            String[] headers = panel.getColumnHeadings().split("\t");
            if (headers.length >= 2)
                if (headers[1].equalsIgnoreCase("Label") || headers[1].equalsIgnoreCase("image"))
                    {
                    isTable = true;
                    }

            // GET EXISTING DATA
            n = panel.getLineCount();
            int startColumn = 0;
            for (int row=0; row < n - 1; row++)
                {
                parser = new StringTokenizer (panel.getLine(row),"\t");
                table.incrementCounter();
                if (isTable)
                    {
                    str = parser.nextToken();   //discard line number
                    table.addLabel(headers[1], parser.nextToken());
                    startColumn = 2;
//                    IJ.log("path 3");
                    }
                else
                    {
                    table.addLabel(labelHeading, tableName);
                    startColumn = 0;
//                    IJ.log("path 4");
                    }

                for (int column = startColumn; column < headers.length; column++)
                    {

                    if (parser.hasMoreTokens() && (str = parser.nextToken()) != null)
                        {
                        try	{
                            double d = Double.parseDouble(str);
                            table.addValue(headers[column],d);
                            }
                        catch (NumberFormatException e)
                            {
    //                        table.addLabel(str);
                            IJ.showMessage ("Table "+tableName+" contains text.");
                            return null;
                            }
                        }
                    else
                        {
                        table.addValue(headers[column],0.0);

                        }
                    }
                }

            table.show(newTableName);
            return table;
            }
		}

        public void updatePlot()
                {
                // CONVERT INDICES TO ATTRIBUTES
                table = MeasurementTable.getTable (tableName);
                int n = table.getCounter();
                if (n < 1)
                        {
//                        IJ.showMessage ("No data in table "+tableName+" to plot.");
                        return;
                        }
                for (int curve=0; curve<maxCurves; curve++)
                        {
                        if (binSize[curve] < 1) binSize[curve] = 1;
                        else if (binSize[curve] > n) binSize[curve] = n;
                        }
                savePreferences();

                for (int i=0;i<maxCurves;i++)
                        {
                        marker[i] = markerOf(markerIndex[i]);
                        color[i] = colorOf(colorIndex[i]);
                        }

                // INITIALIZE BASELINE FOR RELATIVE MMAG CALCULATIONS
                for (int i=0; i<maxCurves;i++)
                        {
                        baseline[i]=0;
                        }
                // GET DATA

		
                int[] nn = new int[maxCurves];
                double[][] x = new double[maxCurves][];
                double[][] y = new double[maxCurves][];
                double[][] yerr = new double[maxCurves][];
                double[][] yop = new double [maxCurves][];
                double[][] yoperr = new double [maxCurves][];
                double xMultiplierFactor, yMultiplierFactor;

                int excluded = 0;
                if (excludedHeadSamples + excludedTailSamples >= n)  //Handle case for more samples excluded than in dataset
                        {
                        excludedHeadSamples = 0;
                        trimdataheadspinner.setValue(0);
                        excludedTailSamples = 0;
                        trimdatatailspinner.setValue(0);
                        }
                else
                        excluded = excludedHeadSamples + excludedTailSamples;

                xMultiplierFactor = Math.pow(0.1, (double)xExponent);
                yMultiplierFactor = Math.pow(0.1, (double)yExponent);

                for (int curve=0; curve<maxCurves; curve++)
                        {
                        nn[curve] = (n-excluded)/binSize[curve];
                        x[curve] = new double[nn[curve]];
                        y[curve] = new double[nn[curve]];
                        yerr[curve] = new double[nn[curve]];
                        yop[curve] = new double[nn[curve]];
                        yoperr[curve] = new double[nn[curve]];
                        }
                for (int curve=0; curve<maxCurves; curve++)
                        {
                        if (xlabel.trim().length() == 0)
                                {
                                for (int j=0; j < nn[curve]; j++)
                                        x[curve][j] = (double)(j+1);
                                }
                        else	{
                                xcolumn = table.getColumnIndex (xlabel); // STRANGELY, THE IMAGEJ ROUTINES ARE OFF BY 1 INDEX
                                if (xcolumn == ResultsTable.COLUMN_NOT_FOUND)
                                        {
                                        if (xlabel.trim().length() != 0)
                                            IJ.showMessage ("Cannot access X-axis table column "+xlabel+" !");
                                        xlabel = columns[0];
                                        return;
                                        }
                                for (int j=0; j < nn[curve]; j++)
                                        {
                                        x[curve][j] = 0;
                                        for (int k=excludedHeadSamples; k < (binSize[curve]+excludedHeadSamples); k++)
                                            x[curve][j] += table.getValueAsDouble (xcolumn,j*binSize[curve]+k);
                                        x[curve][j] = x[curve][j]*xMultiplierFactor/(double)binSize[curve];
                                        }
                                }
                        }
                for (int curve=0;curve<maxCurves;curve++)
                        {
                        if (plotY[curve])
                                {
                                if (ylabel[curve].trim().length() == 0)
                                        {
                                        for (int j=0; j < nn[curve]; j++)
                                                y[curve][j] = (double)(j+1);
                                        }
                                else	{
                                        ycolumn[curve] = table.getColumnIndex (ylabel[curve]);
                                        if (ycolumn[curve] == ResultsTable.COLUMN_NOT_FOUND)
                                                {
                                                if (ylabel[curve].trim().length() != 0)
                                                    IJ.showMessage ("Cannot access Y"+curve+" table column "+ylabel[curve]+" !");
                                                plotY[curve] = false;
                                                usecurvebox[curve].setSelected(false);
                                                }
                                        if (showErrors[curve])
                                                {
                                                errcolumn[curve] = table.getColumnIndex (errlabel[curve]);	// SEE COMMENT ABOVE
                                                if (errcolumn[curve] == ResultsTable.COLUMN_NOT_FOUND)
                                                        {
                                                        if (errlabel[curve].trim().length() != 0)
                                                            IJ.showMessage ("Cannot access Y"+curve+" error column "+errlabel[curve]+" !");
                                                        showErrors[curve] = false;
                                                        errorcolumnbox[curve].setSelected(false);
                                                        }
                                                }
                                         if (operatorIndex[curve] != 0)
                                                {
                                                opcolumn[curve] = table.getColumnIndex (oplabel[curve]);	// SEE COMMENT ABOVE
                                                if (opcolumn[curve] == ResultsTable.COLUMN_NOT_FOUND)
                                                        {
                                                        if (oplabel[curve].trim().length() != 0)
                                                            IJ.showMessage ("Cannot access Y"+curve+" operator data column "+oplabel[curve]+" !");
                                                        operatorIndex[curve] = 0;
                                                        operatorselection[curve].setSelectedIndex(operatorIndex[curve]);
                                                        }
                                                }
                                        if (showOpErrors[curve])
                                                {
                                                operrcolumn[curve] = table.getColumnIndex (operrlabel[curve]);	// SEE COMMENT ABOVE
                                                if (operrcolumn[curve] == ResultsTable.COLUMN_NOT_FOUND)
                                                        {
                                                        if (operrlabel[curve].trim().length() != 0)
                                                            IJ.showMessage ("Cannot access Y"+curve+" operator error column "+operrlabel[curve]+" !");
                                                        showOpErrors[curve] = false;
                                                        operrorcolumnbox[curve].setSelected(false);
                                                        }
                                                }
                                        for (int j=0; j < nn[curve]; j++)
                                                {
                                                y[curve][j] = 0;
                                                yerr[curve][j] = 0;
                                                yop[curve][j] = 0;
                                                // BIN DATA IF APPLICABLE
                                                for (int k=excludedHeadSamples; k < (binSize[curve]+excludedHeadSamples); k++)
                                                        {
                                                        y[curve][j] += table.getValueAsDouble (ycolumn[curve],j*binSize[curve]+k);
                                                        if (showErrors[curve])
                                                                yerr[curve][j] += Math.pow(table.getValueAsDouble (errcolumn[curve],j*binSize[curve]+k),2);
                                                        if (operatorIndex[curve] != 0)
                                                                yop[curve][j] += table.getValueAsDouble (opcolumn[curve],j*binSize[curve]+k);
                                                        if (showOpErrors[curve])
                                                                yoperr[curve][j] += Math.pow(table.getValueAsDouble (operrcolumn[curve],j*binSize[curve]+k),2);
                                                        }
                                                y[curve][j] = y[curve][j]*yMultiplierFactor/binSize[curve];
                                                if (showErrors[curve])
                                                        yerr[curve][j] = yMultiplierFactor*Math.sqrt(yerr[curve][j])/binSize[curve];
                                                else
                                                        yerr[curve][j] = 0.0;
                                                if (operatorIndex[curve] != 0)
                                                        yop[curve][j] = yop[curve][j]*yMultiplierFactor/binSize[curve];
                                                if (showOpErrors[curve])
                                                        yoperr[curve][j] = yMultiplierFactor*Math.sqrt(yoperr[curve][j])/binSize[curve];
                                                else
                                                        yoperr[curve][j] = 0.0;

                                                //APPLY OPERATOR/OPERROR FUNCTIONS TO YDATA AND YERROR

                                                if (operatorIndex[curve] == 0)  //no operator
                                                        {

                                                        }
                                                else if (operatorIndex[curve] == 1)  //divide by
                                                        {
                                                        if (yop[curve][j]==0)
                                                            {
                                                            yerr[curve][j] = 1.0e+100;
                                                            y[curve][j] = 1.0e+100;
                                                            }
                                                        else
                                                            {
                                                            if (showErrors[curve] || showOpErrors[curve])
                                                                {
                                                                yerr[curve][j] = yMultiplierFactor*Math.sqrt(
                                                                ((yerr[curve][j]*yerr[curve][j])/(yop[curve][j]*yop[curve][j])) +
                                                                ((y[curve][j]*y[curve][j]*yoperr[curve][j]*yoperr[curve][j])/
                                                                 (yop[curve][j]*yop[curve][j]*yop[curve][j]*yop[curve][j])));
                                                                }
                                                            y[curve][j] = y[curve][j]*yMultiplierFactor/yop[curve][j];
                                                            }
                                                        }
                                                else if (operatorIndex[curve] == 2)  //multiply by
                                                        {
                                                        if (showErrors[curve] || showOpErrors[curve])
                                                                yerr[curve][j] = Math.sqrt(yop[curve][j]*yop[curve][j]*yerr[curve][j]*yerr[curve][j]+
                                                                    y[curve][j]*y[curve][j]*yoperr[curve][j]*yoperr[curve][j])/yMultiplierFactor;
                                                        y[curve][j] = y[curve][j]*yop[curve][j]/yMultiplierFactor;
                                                        }
                                                else if (operatorIndex[curve] == 3)  //subtract
                                                        {
                                                        if (showErrors[curve] || showOpErrors[curve])
                                                                yerr[curve][j] = Math.sqrt(yerr[curve][j]*yerr[curve][j]+yoperr[curve][j]*yoperr[curve][j]);
                                                        y[curve][j] = y[curve][j]-yop[curve][j];
                                                        }
                                                else if (operatorIndex[curve] == 4)  //add
                                                        {
                                                         if (showErrors[curve] || showOpErrors[curve])
                                                                yerr[curve][j] = Math.sqrt(yerr[curve][j]*yerr[curve][j]+yoperr[curve][j]*yoperr[curve][j]);
                                                        y[curve][j] = y[curve][j]+yop[curve][j];
                                                        }
                                                }
                                        //PERFORM MAGNITUDE CONVERSION IF APPLICABLE
                                        if (mmag[curve])
                                                {
                                                if (nn[curve] < mmagrefs)    //Prepare mmag calulation baseline magnitude
                                                    {
                                                    for (int j=0; j < nn[curve]; j++)
                                                        {
                                                        baseline[curve] += y[curve][j];
                                                        }
                                                    baseline[curve] /= nn[curve];
                                                    }
                                                else
                                                    {
                                                    for (int j=0; j < mmagrefs; j++)
                                                        {
                                                        baseline[curve] += y[curve][j];
                                                        }
                                                    baseline[curve] /= mmagrefs;
                                                    }

                                                for (int j=0; j < nn[curve]; j++)
                                                        {
                                                        if (showErrors[curve] || showOpErrors[curve])
                                                            {
                                                            yerr[curve][j] = yMultiplierFactor*(2.5*Math.log10(1.0+yerr[curve][j]/y[curve][j]));
                                                            }
                                                        y[curve][j] = yMultiplierFactor*2.5*Math.log10(y[curve][j]/baseline[curve]);
                                                        }
                                                }

                                        }
                                }
                        }
                // PERFORM AUTOSCALING TO ONE OR MORE CURVES
		double dx = 0.0;
		double dy = 0.0;
                double xPlotMin = 0.0;
                double xPlotMax = 0.0;
                double yPlotMin = 0.0;
                double yPlotMax = 0.0;

                xPlotMin = xMin;
                xPlotMax = xMax;
                yPlotMin = yMin;
                yPlotMax = yMax;
                xautoscalemin = minOf(x[0]);
                xautoscalemax = maxOf(x[0]);

                boolean atLeastOne = false;
                int firstCurve = -1;
                for (int curve = 0; curve<maxCurves; curve++)
                        {
                        if (plotY[curve])
                                {
                                yMinimum[curve] = minOf(y[curve]); //FIND MIN AND MAX OF EACH SELECTED DATASET
                                yMaximum[curve] = maxOf(y[curve]);
                                if ((firstCurve == -1))
                                        firstCurve = curve; //FIND THE FIRST CURVE TO DISPLAY - IT IS USED FOR THE Y-AXIS LABEL
                                if (ASInclude[curve])       //INCLUDE ALL DESIRED CURVES IN Y-AXIS AUTOSCALE
                                        if (!atLeastOne)
                                                {
                                                yautoscalemin = yMinimum[curve];
                                                yautoscalemax = yMaximum[curve];
                                                atLeastOne = true;
                                                }
                                        else
                                                {
                                                if (yMinimum[curve] < yautoscalemin) yautoscalemin = yMinimum[curve];
                                                if (yMaximum[curve] > yautoscalemax) yautoscalemax = yMaximum[curve];
                                                }
                                }
                        }

                if (!atLeastOne)  //IF NO CURVES SELECTED FOR Y-AXIS AUTOSCALE, USE THE FIRST CURVE
                        {
                        yautoscalemin = yMinimum[0];
                        yautoscalemax = yMaximum[0];
                        }
                if (firstCurve == -1)   //IF NO CURVES SELECTED FOR DISPLAY, USE THE FIRST CURVE
                        firstCurve = 0;

                if (autoScaleX)
                    {
                    xPlotMin = xautoscalemin;
                    xPlotMax = xautoscalemax;
                                dx = (xPlotMax-xPlotMin)/99.0;
                    }
                        else if (useFirstX)
                    {
                    xPlotMin = x[0][0];
                    xPlotMax = xPlotMin+xWidth;
                    dx = (xWidth)/99.0;
                    }
                        else
                                dx = (xPlotMax-xPlotMin)/99.0;

                if (autoScaleY)
                    {
                    yPlotMin = yautoscalemin;
                    yPlotMax = yautoscalemax;
                    }
                dy = (yPlotMax-yPlotMin)/99.0;

                // IF MJD, SHOW DECIMAL DATE DUE TO float API OF PlotWindow

                double xOffset = 0.0;
                double yOffset = 0.0;
                int jd=0;
                String xlab ="";
                String ylab = "";
                boolean xGoodLabel = false;
                boolean yGoodLabel = false;

                // MAKE X-AXIS LABEL

                if (xlabel.trim().equals("J.D.-2400000") || 
                    xlabel.trim().equals("JD_SOBS") || xlabel.trim().equals("JD_MOBS") ||
                    xlabel.trim().equals("HJD_SOBS") || xlabel.trim().equals("HJD_MOBS") ||
                    xlabel.trim().equals("BJD_MOBS") || xlabel.trim().equals("BJD_SOBS"))
                        {
                        jd = (int)xPlotMin;
                        xOffset = (double)jd;
                        xPlotMin -= xOffset;
                        xPlotMax -= xOffset;
                        for (int curve=0; curve < maxCurves; curve++)
                            for (int j=0; j < nn[curve]; j++)
                                    x[curve][j] -= xOffset;
                        if (useXColumnName)
                                {
                                if (xlabel.trim().equals("J.D.-2400000"))
                                    xlab = "Geocentric Julian Date - "+(2400000+jd);
                                else if (xlabel.trim().equals("BJD_MOBS"))
                                    xlab = "Barycentric Julian Date - "+jd+" (mid-exposure)";
                                else if (xlabel.trim().equals("BJD_SOBS"))
                                    xlab = "Barycentric Julian Date - "+jd+" (exposure start)";
                                else if (xlabel.trim().equals("HJD_MOBS"))
                                    xlab = "Heliocentric Julian Date - "+jd+" (mid-exposure)";
                                else if (xlabel.trim().equals("HJD_SOBS"))
                                    xlab = "Heliocentric Julian Date - "+jd+" (exposure start)";
                                else if (xlabel.trim().equals("JD_MOBS"))
                                    xlab = "Geocentric Julian Date - "+jd+" (mid-exposure)";
                                else if (xlabel.trim().equals("JD_SOBS"))
                                    xlab = "Geocentric Julian Date - "+jd+" (exposure start)";
                                else
                                    xlab = "Label Error";
                                xGoodLabel = true;
                                }
                        else if (useXCustomName)
                                {
                                xlab = xLegend;
                                xGoodLabel = true;
                                }
                        }
                else if (useXColumnName)
                        {
                        xlab = xlabel;
                        xGoodLabel = true;
                        }
                else if (useXCustomName)
                        {
                        xlab = xLegend;
                        xGoodLabel = true;
                        }

                if (xGoodLabel && (xExponent != 0) && showXScaleInfo)
                        xlab += " x 1E"+xExponent;

                // MAKE Y-AXIS LABEL

                if (ylabel[firstCurve].trim().equals("J.D.-2400000") ||
                    ylabel[firstCurve].trim().equals("JD_SOBS") || ylabel[firstCurve].trim().equals("JD_MOBS") ||
                    ylabel[firstCurve].trim().equals("HJD_SOBS") || ylabel[firstCurve].trim().equals("HJD_MOBS") ||
                    ylabel[firstCurve].trim().equals("BJD_SOBS") || ylabel[firstCurve].trim().equals("BJD_MOBS"))
                        {
                        jd = (int)yPlotMin;
                        yOffset = (double)jd;
                        yPlotMin -= yOffset;
                        yPlotMax -= yOffset;
                        for (int j=0; j < nn[firstCurve]; j++)
                                y[firstCurve][j] -= yOffset;
                        if (useYColumnName)
                                {
                                if (ylabel[firstCurve].trim().equals("J.D.-2400000"))
                                    ylab = "Geocentric Julian Date - "+(2400000+jd);
                                else if (ylabel[firstCurve].trim().equals("BJD_MOBS"))
                                    ylab = "Barycentric Julian Date - "+jd+" (mid-exposure)";
                                else if (ylabel[firstCurve].trim().equals("BJD_SOBS"))
                                    ylab = "Barycentric Julian Date - "+jd+" (exposure start)";
                                else if (ylabel[firstCurve].trim().equals("HJD_MOBS"))
                                    ylab = "Heliocentric Julian Date - "+jd+" (mid-exposure)";
                                else if (ylabel[firstCurve].trim().equals("HJD_SOBS"))
                                    ylab = "Heliocentric Julian Date - "+jd+" (exposure start)";
                                else if (ylabel[firstCurve].trim().equals("JD_MOBS"))
                                    ylab = "Geocentric Julian Date - "+jd+" (mid-exposure)";
                                else if (ylabel[firstCurve].trim().equals("JD_SOBS"))
                                    ylab = "Geocentric Julian Date - "+jd+" (exposure start)";
                                else
                                    ylab = "Label Error";
                                yGoodLabel = true;
                                }
                        else if (useYCustomName)
                                {
                                ylab = yLegend;
                                yGoodLabel = true;
                                }
                        }
                else if (useYColumnName)
                        {
                        ylab=ylabel[firstCurve];
                        if (operatorIndex[firstCurve] != 0)
                                ylab += opSymbol[operatorIndex[firstCurve]]+oplabel[firstCurve];
                        yGoodLabel = true;
                        }
                else if (useYCustomName)
                        {
                        ylab = yLegend;
                        yGoodLabel = true;
                        }
                //FINISH Y-LABEL AFTER FINDING SCALING FACTORS BELOW

              
                //SCALE AND SHIFT SECONDARY CURVES TO FIT ON PLOT

                yRange = yPlotMax - yPlotMin;
                yMid = yPlotMin + yRange/2.;
                for (int curve=0; curve<maxCurves; curve++)
                    {
                    if (plotY[curve])
                        {
                        yMidpoint[curve] = (yMaximum[curve] + yMinimum[curve])/2.;
                        yWidth[curve] = yMaximum[curve] - yMinimum[curve];
                        if (yWidth[curve] == 0) yWidth[curve] = 1.0e-10;
                        if (autoScale[curve])
                                {
                                subtotalScaleFactor[curve] = autoScaleFactor[curve]*yRange/yWidth[curve];
                                subtotalShiftFactor[curve] = yMid + yRange*autoShiftFactor[curve] - subtotalScaleFactor[curve]*yMidpoint[curve];
                                totalScaleFactor[curve] = 1.0/(subtotalScaleFactor[curve]*yMultiplierFactor);
                                totalShiftFactor[curve] = subtotalShiftFactor[curve];
                                }
                        else
                                {
                                subtotalScaleFactor[curve] = customScaleFactor[curve];
                                subtotalShiftFactor[curve] = customShiftFactor[curve];
                                totalScaleFactor[curve] = 1.0/(subtotalScaleFactor[curve]*yMultiplierFactor);
                                totalShiftFactor[curve] = subtotalShiftFactor[curve];
                                }
                        for (int j=0; j<nn[curve]; j++)
                            {
                            if (showErrors[curve] || showOpErrors[curve])
                                {
                                yerr[curve][j] = subtotalScaleFactor[curve]*yerr[curve][j];
                                }
                            y[curve][j] = subtotalScaleFactor[curve]*y[curve][j] + subtotalShiftFactor[curve];
                            }
                        }
                    }

                if (yGoodLabel)
                        {
                        if (showYScaleInfo || showYShiftInfo)
                                ylab += scaleShiftText(showYScaleInfo, showYShiftInfo, mmag[firstCurve], showYmmagInfo, totalScaleFactor[firstCurve], totalShiftFactor[firstCurve]);
                        if (mmag[firstCurve] && showYmmagInfo)
                                if (totalScaleFactor[firstCurve] == 0.001)
                                        ylab += " (mmag)";
                                else
                                        ylab += " (mag)";
                        if ((binSize[firstCurve]!=1) && showYBinInfo)
                                ylab += " (bin size = "+binSize[firstCurve]+")";
                        if (showYSymbolInfo)
                                ylab += " ("+colors[colorIndex[firstCurve]]+" "+markers[markerIndex[firstCurve]]+")";
                        }

		// PLOT DATA

                //Set up plot options
                int plotOptions = 0;
                if (xTics) plotOptions += ij.gui.Plot.X_TICKS;
                if (yTics) plotOptions += ij.gui.Plot.Y_TICKS;
                if (xGrid) plotOptions += ij.gui.Plot.X_GRID;
                if (yGrid) plotOptions += ij.gui.Plot.Y_GRID;
                if (xNumbers) plotOptions += ij.gui.Plot.X_NUMBERS;
                if (yNumbers) plotOptions += ij.gui.Plot.Y_NUMBERS;

                double[] nullX = null;
                double[] nullY = null;
                Plot plot = new Plot ("Plot of "+tableName,xlab,ylab,nullX,nullY,plotOptions);
                plot.setSize(plotSizeX, plotSizeY);
                plotMinX = xPlotMin-5.*dx;
                plotMaxX = xPlotMax+5.*dx;
                plotMinY = yPlotMin-5.*dy;
                plotMaxY = yPlotMax+5.*dy;

                plot.setLimits (plotMinX, plotMaxX, plotMinY, plotMaxY);
                double legPosY = legendPosY;
                for (int curve=0; curve<maxCurves; curve++)
                    {
                    if (plotY[curve])
                        {
                        plot.setColor (color[curve]);
                        if (marker[curve] == ij.gui.Plot.DOT)
                            plot.setLineWidth(4);
                        else
                            plot.setLineWidth(1);

                        plot.addPoints (x[curve],y[curve],marker[curve]);

                        plot.setLineWidth(1);

                        if (showErrors[curve] || showOpErrors[curve])     //code to replace plot.addErrorBars
                            {               //since plot.addErrorBars only plots with lines enabled
                            for (int j=0; j < nn[curve]; j++)
                                {
                                plot.drawLine(x[curve][j], y[curve][j]-yerr[curve][j], x[curve][j], y[curve][j]+yerr[curve][j]);
                                }
                            }

                        if (lines[curve] && !(marker[curve] == ij.gui.Plot.LINE))
                            {
                             for (int j=0; j < nn[curve]-1; j++)
                                {
                                plot.drawLine(x[curve][j],y[curve][j],x[curve][j+1],y[curve][j+1]);
                                }
                            }

                        plot.setLineWidth(1);
                        if (legendLeft)
                            plot.setJustification(ij.process.ImageProcessor.LEFT_JUSTIFY);
                        else if (legendRight)
                            plot.setJustification(ij.process.ImageProcessor.RIGHT_JUSTIFY);
                        else
                            plot.setJustification(ij.process.ImageProcessor.CENTER_JUSTIFY);

                        // MAKE FULL LEGEND STRING FOR CURVE
                        String llab = "";

                        if (useColumnName[curve])
                            {
                            llab=ylabel[curve];
                            if (operatorIndex[curve] != 0)
                                llab += opSymbol[operatorIndex[curve]]+oplabel[curve];
                            }
                        else if (useLegend[curve])
                            llab = legend[curve];
                        if (useColumnName[curve] || useLegend[curve])
                            {
                            if (showLScaleInfo || showLShiftInfo)
                                llab += scaleShiftText(showLScaleInfo, showLShiftInfo, mmag[curve], showLmmagInfo, totalScaleFactor[curve], totalShiftFactor[curve]);
                            if (useTwoLineLegend)
                                {
                                plot.addLabel(legendPosX, legPosY, llab);
                                legPosY += 18./plotSizeY;
                                llab = " ";
                                }
                            if (mmag[curve] && showLmmagInfo)
                                if (totalScaleFactor[curve] == 0.001)
                                        llab += " (mmag)";
                                else
                                        llab += " (mag)";
                            if ((binSize[curve] != 1) && showLBinInfo)
                                llab += " (bin size = "+binSize[curve]+")";
                            if (showLSymbolInfo)
                                llab += " ("+markers[markerIndex[curve]]+")";
                            if (!llab.equals(" "))
                                {
                                plot.addLabel(legendPosX, legPosY, llab);
                                legPosY += 18./plotSizeY;
                                }
                            }
                        }
                    }

                plot.setLineWidth(1);
                plot.setJustification(ij.process.ImageProcessor.CENTER_JUSTIFY);
                plot.setColor (java.awt.Color.black);
                if (useTitle)
                        {
                        plot.changeFont(new java.awt.Font("bold16",java.awt.Font.BOLD,16));
                        plot.addLabel(titlePosX, titlePosY,title);
                        }
                if (useSubtitle)
                        {
                        plot.changeFont(new java.awt.Font("plain14",java.awt.Font.PLAIN,14));
                        plot.addLabel(subtitlePosX, subtitlePosY, subtitle);
                        }

                plotImage = WindowManager.getImage("Plot of "+tableName);
                
                if (plotImage == null)
                    {
                    plot.show();
                    plotFrame = WindowManager.getFrame("Plot of "+tableName);
                    if ((plotFrame != null) && rememberWindowLocations)
                            plotFrame.setLocation(plotFrameLocationX, plotFrameLocationY);

                    plotImage = WindowManager.getImage("Plot of "+tableName);
                    plotFrame = WindowManager.getFrame("Plot of "+tableName);

                    plotWindow = plotImage.getWindow();
                    plotImageCanvas = plotImage.getCanvas();
                    plotOverlayCanvas = new OverlayCanvas(plotImage);
                    list.clear();

                    MouseMotionListener[] mml = plotImageCanvas.getMouseMotionListeners();
                    if (mml.length>0)
                            for (int i=0; i<mml.length; i++)
                                    plotImageCanvas.removeMouseMotionListener(mml[i]);

                    MouseWheelListener[] mwl = plotImageCanvas.getMouseWheelListeners();
                    if (mwl.length>0)
                            for (int i=0; i<mwl.length; i++)
                                    plotImageCanvas.removeMouseWheelListener(mwl[i]);

                    MouseWheelListener[] mwl3 = plotWindow.getMouseWheelListeners();
                    if (mwl3.length>0)
                            for (int i=0; i<mwl3.length; i++)
                                    plotWindow.removeMouseWheelListener(mwl3[i]);

                    MouseListener[] ml = plotImageCanvas.getMouseListeners();
                    if (ml.length>0)
                            for (int i=0; i<ml.length; i++)
                                    plotImageCanvas.removeMouseListener(ml[i]);

                    plotImageCanvas.addMouseWheelListener(this);
                    plotImageCanvas.addMouseMotionListener(this);
                    plotImageCanvas.addMouseListener(this);
                    plotImageCanvas.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                    }
                else
                    {
                    ImageProcessor ip = plot.getProcessor();
                    plotImage.setProcessor("Plot of "+tableName,ip);
                    plotImageCanvas = plotImage.getCanvas();
                    plotImageCanvas.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                    }


                //Replot to clean up blank areas

                plotImage = WindowManager.getImage("Plot of "+tableName);
                ip = plot.getProcessor();
                plotImage.setProcessor("Plot of "+tableName,ip);

                }

	void addElement(Vector<Roi> list, Shape shape, Color color, int lineWidth) {
		roi = new ShapeRoi(shape);
		roi.setInstanceColor(color);
		roi.setLineWidth(lineWidth);
		list.addElement(roi);
                }

        public void mouseClicked(MouseEvent e) {

                // mouse clicked code is in mouseReleased() to allow drag/click thresholding
                }

        public void mousePressed(MouseEvent e) {
                        startDragX = plotImageCanvas.offScreenX(e.getX());
                        startDragY = plotImageCanvas.offScreenY(e.getY());
                        startDragScreenX = e.getX();
                        startDragScreenY = e.getY();
                        startDragSubImageX =plotImageCanvas.getSrcRect().x;
                        startDragSubImageY =plotImageCanvas.getSrcRect().y;
                        button2Drag = false;
                }

        public void mouseReleased(MouseEvent e) {
                int screenX = e.getX();
                int screenY = e.getY();
                int imageX = plotImageCanvas.offScreenX(screenX);
                int imageY = plotImageCanvas.offScreenY(screenY);

                double x = plotMinX + (double)(startDragX - 60) * (plotMaxX - plotMinX)/(double)(plotSizeX - 77);
                double y = plotMaxY - (double)(startDragY - 15) * (plotMaxY - plotMinY)/(double)(plotSizeY - 54);
                double dx = (double)(imageX - startDragX) * (plotMaxX - plotMinX)/(double)(plotSizeX - 77);
                double dy = (double)(imageY - startDragY) * (-1.0) * (plotMaxY - plotMinY)/(double)(plotSizeY - 77);


                if (((e.getModifiers() & MouseEvent.BUTTON2_MASK) != 0) && button2Drag)
                        {                                                          // measure distance and report in Results Table
                        button2Drag = false;
                        IJ.write("x="+fourPlaces.format(x)+", y="+fourPlaces.format(y)+
                                 ", dx="+fourPlaces.format(dx)+", dy="+fourPlaces.format(dy));
                        }
                if (Math.sqrt((screenX-startDragScreenX)*(screenX-startDragScreenX) +      //check mouse click/drag threshold
                        (screenY-startDragScreenY)*(screenY-startDragScreenY)) < 4.0)
                        {
                        if (e.getButton() == MouseEvent.BUTTON1)                          //left mouse click
                                {
                                zoomControl(e.getX(), e.getY(), -1);
                                }

                        else if (e.getButton() == MouseEvent.BUTTON2)                     //middle mouse click
                                {
                                IJ.write("x="+fourPlaces.format(x)+", y="+fourPlaces.format(y));
                                }

                        else if (e.getButton() == MouseEvent.BUTTON3)                     //right mouse click
                                {
                                zoomControl(screenX, screenY, 1);
                                }
                        }
                list.clear();
                path.reset();
                plotImageCanvas.repaint();
                }

        public void mouseEntered(MouseEvent e) {}

        public void mouseExited(MouseEvent e) {}
        public void mouseDragged(MouseEvent e) {
                int screenX = e.getX();
                int screenY = e.getY();
                int imageX = plotImageCanvas.offScreenX(e.getX());
                int imageY = plotImageCanvas.offScreenY(e.getY());
                double magnification = plotImageCanvas.getMagnification();
                if (Math.sqrt((screenX-startDragScreenX)*(screenX-startDragScreenX) +  //check mouse click/drag threshold
                        (screenY-startDragScreenY)*(screenY-startDragScreenY)) >= 4.0)
                        {
                        if ((e.getModifiers() & MouseEvent.BUTTON3_MASK) != 0)    // dragging with right mouse button
                                {                                                 // measure distance, show on plot and status bar and save to results window
                                list.clear();
                                path.reset();
                                path.moveTo(startDragX, startDragY);
                                path.lineTo(imageX, imageY);

                                addElement(list, path, Color.red, 2);
                                 plotImageCanvas.setDisplayList(list);
                                double x = plotMinX + (double)(imageX - 60) * (plotMaxX - plotMinX)/(double)(plotSizeX - 77);
                                double y = plotMaxY - (double)(imageY - 15) * (plotMaxY - plotMinY)/(double)(plotSizeY - 54);
                                double dx = (double)(imageX - startDragX) * (plotMaxX - plotMinX)/(double)(plotSizeX - 77);
                                double dy = (double)(imageY - startDragY) * (-1.0) * (plotMaxY - plotMinY)/(double)(plotSizeY - 77);
                                plotbottompanel = (Panel) plotWindow.getComponent(1);
                                plotbottompanel.setSize(600,30);
                                plotcoordlabel = (Label) plotbottompanel.getComponent(3);
                                plotcoordlabel.setSize(400,20);
                                plotcoordlabel.setText("x="+fourPlaces.format(x)+", y="+fourPlaces.format(y)+
                                                       ", dx="+fourPlaces.format(dx)+", dy="+fourPlaces.format(dy));
                                IJ.showStatus("plot coordinates: x="+fourPlaces.format(x)+", y="+fourPlaces.format(y)+
                                              ", dx="+fourPlaces.format(dx)+", dy="+fourPlaces.format(dy));
                                }
                        else if ((e.getModifiers() & MouseEvent.BUTTON2_MASK) != 0)        // dragging with middle mouse button
                                {                                                          // measure distance, show on plot and status bar
                                button2Drag = true;                                        // and save to results window when mouse released
                                list.clear();
                                path.reset();
                                path.moveTo(startDragX, startDragY);
                                path.lineTo(imageX, imageY);

                                addElement(list, path, Color.red, 2);
                                 plotImageCanvas.setDisplayList(list);
                                double x = plotMinX + (double)(imageX - 60) * (plotMaxX - plotMinX)/(double)(plotSizeX - 77);
                                double y = plotMaxY - (double)(imageY - 15) * (plotMaxY - plotMinY)/(double)(plotSizeY - 54);
                                double dx = (double)(imageX - startDragX) * (plotMaxX - plotMinX)/(double)(plotSizeX - 77);
                                double dy = (double)(imageY - startDragY) * (-1.0) * (plotMaxY - plotMinY)/(double)(plotSizeY - 77);
                                plotbottompanel = (Panel) plotWindow.getComponent(1);
                                plotbottompanel.setSize(600,30);
                                plotcoordlabel = (Label) plotbottompanel.getComponent(3);
                                plotcoordlabel.setSize(400,20);
                                plotcoordlabel.setText("x="+fourPlaces.format(x)+", y="+fourPlaces.format(y)+
                                                       ", dx="+fourPlaces.format(dx)+", dy="+fourPlaces.format(dy));
                                IJ.showStatus("plot coordinates: x="+fourPlaces.format(x)+", y="+fourPlaces.format(y)+
                                              ", dx="+fourPlaces.format(dx)+", dy="+fourPlaces.format(dy));
                                }
                        else if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) != 0)            // dragging with left mouse button (pan image)
                                {
                                Rectangle imageRect = plotImageCanvas.getSrcRect();

                                int ox = startDragSubImageX + (int)(e.getX()/magnification);   //convert to offscreen coordinates
                                int oy = startDragSubImageY + (int)(e.getY()/magnification);    //code adapted from ImageCanvas.java
                                //IJ.log("scroll: "+ox+" "+oy+" "+xMouseStart+" "+yMouseStart);
                                int newx = startDragSubImageX + (startDragX-ox);
                                int newy = startDragSubImageY + (startDragY-oy);
                                if (newx<0) newx = 0;
                                if (newy<0) newy = 0;
                                if ((newx+imageRect.width)>plotSizeX) newx = plotSizeX-imageRect.width;
                                if ((newy+imageRect.height)>plotSizeY) newy = plotSizeY-imageRect.height;
                                imageRect.x = newx;
                                imageRect.y = newy;
                                plotImageCanvas.setSourceRect(imageRect);
                                plotImage.updateAndDraw();

                                double x = plotMinX + (double)(imageX - 60) * (plotMaxX - plotMinX)/(double)(plotSizeX - 77);
                                double y = plotMaxY - (double)(imageY - 15) * (plotMaxY - plotMinY)/(double)(plotSizeY - 54);
                                plotbottompanel = (Panel) plotWindow.getComponent(1);
                                plotbottompanel.setSize(450,30);
                                plotcoordlabel = (Label) plotbottompanel.getComponent(3);
                                plotcoordlabel.setSize(300,20);
                                plotcoordlabel.setText("x="+fourPlaces.format(x)+", y="+fourPlaces.format(y));
                                IJ.showStatus("plot coordinates: x="+fourPlaces.format(x)+", y="+fourPlaces.format(y));
                                }
                        }
                }
 
        public void mouseMoved(MouseEvent e) {
                int imageX = plotImageCanvas.offScreenX(e.getX());
                int imageY = plotImageCanvas.offScreenY(e.getY());
                double x = plotMinX + (double)(imageX - 60) * (plotMaxX - plotMinX)/(double)(plotSizeX - 77);
                double y = plotMaxY - (double)(imageY - 15) * (plotMaxY - plotMinY)/(double)(plotSizeY - 54);
                plotbottompanel = (Panel) plotWindow.getComponent(1);
                plotbottompanel.setSize(450,30);
                plotcoordlabel = (Label) plotbottompanel.getComponent(3);
                plotcoordlabel.setSize(300,20);
                plotcoordlabel.setText("x="+fourPlaces.format(x)+", y="+fourPlaces.format(y));
                IJ.showStatus("plot coordinates: x="+fourPlaces.format(x)+", y="+fourPlaces.format(y));
                }
        public void mouseWheelMoved( MouseWheelEvent e )
                {
                int screenX = e.getX();
                int screenY = e.getY();
                int magChangeSteps = e.getWheelRotation();
                zoomControl(screenX, screenY, magChangeSteps);
                }

        public void zoomControl(int screenX, int screenY, int magChangeSteps)
                {
                int imageX = plotImageCanvas.offScreenX(screenX);
                int imageY = plotImageCanvas.offScreenY(screenY);
                double zoom = plotImageCanvas.getMagnification();
                
                if ((magChangeSteps>0) && (zoom>1))     //zoom out
                        plotImageCanvas.zoomOut((int)(screenX), (int)(screenY));
                else if ((magChangeSteps<0) && (zoom<32))   //zoom in
                        zoomIn(screenX, screenY);
                try
                        {
                        Robot robot = new Robot();     //move mouse pointer to new position on screen
                        robot.mouseMove(plotImageCanvas.getLocationOnScreen().x + plotImageCanvas.screenX(imageX),
                                        plotImageCanvas.getLocationOnScreen().y + plotImageCanvas.screenY(imageY));
                        }
                catch (AWTException ee)
                        {
                        ee.printStackTrace();
                        }
                }

        public void zoomIn(int x, int y) {

                double magnification = plotImageCanvas.getMagnification();
		if (magnification>=32) return;
		double newMag = plotImageCanvas.getHigherZoomLevel(magnification);

                int w = (int)Math.round(plotSizeX/newMag);
		if (w*newMag<plotSizeX) w++;
		int h = (int)Math.round(plotSizeY/newMag);
		if (h*newMag<plotSizeY) h++;
		x = plotImageCanvas.offScreenX(x);
		y = plotImageCanvas.offScreenY(y);
		Rectangle r = new Rectangle(x-w/2, y-h/2, w, h);
		if (r.x<0) r.x = 0;
		if (r.y<0) r.y = 0;
		if (r.x+w>plotSizeX) r.x = plotSizeX-w;
		if (r.y+h>plotSizeY) r.y = plotSizeY-h;
		plotImageCanvas.setSourceRect(r);
		plotImageCanvas.setMagnification(newMag);
		plotImageCanvas.repaint();
                }

        public void setupArrays()
                {
                maxCurves = (int)Prefs.get("plot.maxCurves", 8);
                if (maxCurves < 1) maxCurves = 1;
                ylabel = new String[maxCurves];
                ycolumn = new int[maxCurves];
                errcolumn = new int[maxCurves];
                operrcolumn = new int[maxCurves];
                oplabel = new String[maxCurves];
                opcolumn = new int[maxCurves];
                errlabel = new String[maxCurves];
                operrlabel = new String[maxCurves];
                showErrors = new boolean[maxCurves];
                showOpErrors = new boolean[maxCurves];
                lines = new boolean[maxCurves];
                marker = new int[maxCurves];
                color = new Color[maxCurves];
                markerIndex = new int[maxCurves];
                colorIndex = new int[maxCurves];
                operatorIndex = new int[maxCurves];
                cblabels = new String[maxCurves];
                moreOptions = new boolean [maxCurves];
                autoScaleFactor = new double[maxCurves];
                autoScaleStep = new double[maxCurves];
                autoShiftFactor = new double[maxCurves];
                autoShiftStep = new double[maxCurves];
                customScaleFactor = new double[maxCurves];
                customScaleStep = new double[maxCurves];
                customShiftFactor = new double[maxCurves];
                customShiftStep = new double[maxCurves];
                baseline = new double[maxCurves];
                subtotalScaleFactor = new double[maxCurves];
                subtotalShiftFactor = new double[maxCurves];
                totalScaleFactor = new double[maxCurves];
                totalShiftFactor = new double[maxCurves];
                yWidth = new double[maxCurves];
                yMinimum = new double[maxCurves];
                yMaximum = new double[maxCurves];
                yMidpoint = new double[maxCurves];
                mmag = new boolean[maxCurves];
                plotY = new boolean[maxCurves];
                useColumnName = new boolean[maxCurves];
                useLegend = new boolean[maxCurves];
                autoScale = new boolean[maxCurves];
                binSize = new int[maxCurves];
                legend = new String[maxCurves];
                ASInclude = new boolean[maxCurves];
                autoscaleincludebox = new JCheckBox[maxCurves];

                usecurvebox = new JCheckBox[maxCurves];
                errorcolumnbox = new JCheckBox[maxCurves];
                operrorcolumnbox = new JCheckBox[maxCurves];
                usebinbox = new JCheckBox[maxCurves];
                usemmagbox = new JCheckBox[maxCurves];
                uselinesbox = new JCheckBox[maxCurves];
                ydatacolumn = new JComboBox[maxCurves];
                operatorselection = new JComboBox[maxCurves];
                operatorcolumn = new JComboBox[maxCurves];
                errorcolumn = new JComboBox[maxCurves];
                operrorcolumn = new JComboBox[maxCurves];
                markersymbolselection = new JComboBox[maxCurves];
                markercolorselection = new JComboBox[maxCurves];
                binsizespinnermodel = new SpinnerModel[maxCurves];
                binsizespinner = new JSpinner[maxCurves];
                morelegendradiopanelgroup = new JPanel[maxCurves];
                autoscalepanelgroup = new JPanel[maxCurves];
                customscalepanelgroup = new JPanel[maxCurves];
                legendnoneButton = new JRadioButton[maxCurves];
                legendcustomNameButton = new JRadioButton[maxCurves];
                legendcolumnNameButton = new JRadioButton[maxCurves];
                autoscaleButton = new JRadioButton[maxCurves];
                customscaleButton = new JRadioButton[maxCurves];
                morelegendRadioGroup = new ButtonGroup[maxCurves];
                autoscaleRadioGroup = new ButtonGroup[maxCurves];
                morelegendField = new JTextField[maxCurves];
                autoshiftmodel = new SpinnerModel[maxCurves];
                autoshiftstepmodel = new SpinnerModel[maxCurves];
                autoscalemodel  = new SpinnerModel[maxCurves];
                autoscalestepmodel  = new SpinnerModel[maxCurves];
                customshiftmodel = new SpinnerModel[maxCurves];
                customshiftstepmodel = new SpinnerModel[maxCurves];
                customscalemodel = new SpinnerModel[maxCurves];
                customscalestepmodel = new SpinnerModel[maxCurves];
                autoshiftspinner = new JSpinner[maxCurves];
                autoshiftstepspinner = new JSpinner[maxCurves];
                autoscalespinner  = new JSpinner[maxCurves];
                autoscalestepspinner  = new JSpinner[maxCurves];
                customshiftspinner = new JSpinner[maxCurves];
                customshiftstepspinner = new JSpinner[maxCurves];
                customscalespinner = new JSpinner[maxCurves];
                customscalestepspinner = new JSpinner[maxCurves];
                autoscalesteppopup = new JPopupMenu[maxCurves];
                autoshiftsteppopup = new JPopupMenu[maxCurves];
                customscalesteppopup = new JPopupMenu[maxCurves];
                customshiftsteppopup = new JPopupMenu[maxCurves];
                autoscalesteppanel = new JPanel[maxCurves];
                customscalesteppanel = new JPanel[maxCurves];
                autoshiftsteppanel = new JPanel[maxCurves];
                customshiftsteppanel = new JPanel[maxCurves];
                autoscalesteplabel = new JLabel[maxCurves];
                autoshiftsteplabel = new JLabel[maxCurves];
                customscalesteplabel = new JLabel[maxCurves];
                customshiftsteplabel = new JLabel[maxCurves];
                curvelabel = new JLabel[maxCurves];
                othercurvelabel = new JLabel[maxCurves];
                grabautopanel = new JPanel[maxCurves];
                grabautobutton = new JButton[maxCurves];

                customscaleeditor = new JSpinner.NumberEditor[maxCurves];
                customscaleformat = new DecimalFormat[maxCurves];
                customshifteditor = new JSpinner.NumberEditor[maxCurves];
                customshiftformat = new DecimalFormat[maxCurves];

                //INITIALIZE CERTAIN ARRAYS
                for (int i=0; i<maxCurves; i++)
                        {
                        ylabel[i] = "ratio1";
                        errlabel[i] = "ratio1_error";
                        oplabel[i] = "source-sky_#2";
                        operrlabel[i] = "source_error_#2";
                        color[i] = java.awt.Color.black;
                        if (i==0)
                                {
                                autoScale[i] = false;
                                plotY[i] = true;
                                ASInclude[i]= true;
                                }
                        else
                                {
                                autoScale[i] = true;
                                plotY[i] = false;
                                ASInclude[i] = false;
                                }
                        autoScaleFactor[i] = 0.5;
                        autoScaleStep[i] = 1;
                        autoShiftFactor[i] = 0;
                        autoShiftStep[i] = 1;
                        customScaleFactor[i] = 1;
                        customScaleStep[i] = 1;
                        customShiftFactor[i] = 0;
                        customShiftStep[i] = 1;
                        binSize[i] = 1;
                        baseline[i] = 0;
                        legend[i] = "Legend"+(i+1);
                        }
                }




	/**
	 * Returns minimum of double array.
	 */
	double minOf(double[] arr)
		{
		int n=arr.length;
		double mn = arr[0];
		for (int i=1; i < n; i++)
			mn = (arr[i] < mn)? arr[i] : mn;
		return mn;
		}

	/**
	 * Returns maximum of double array.
	 */
	double maxOf(double[] arr)
		{
		int n=arr.length;
		double mx = arr[0];
		for (int i=1; i < n; i++)
			mx = (arr[i] > mx)? arr[i] : mx;
		return mx;
		}
        Color colorOf(int ci)
                {
                Color cmkr = java.awt.Color.black;
                if (ci == 0) cmkr = java.awt.Color.black;
                else if (ci == 1) cmkr = java.awt.Color.darkGray;
                else if (ci == 2) cmkr = java.awt.Color.gray;
                else if (ci == 3) cmkr = java.awt.Color.lightGray;
                else if (ci == 4) cmkr = java.awt.Color.green;
                else if (ci == 5) cmkr = java.awt.Color.cyan;
                else if (ci == 6) cmkr = java.awt.Color.blue;
                else if (ci == 7) cmkr = java.awt.Color.magenta;
                else if (ci == 8) cmkr = java.awt.Color.pink;
                else if (ci == 9) cmkr = java.awt.Color.red;
                else if (ci == 10) cmkr = java.awt.Color.orange;
                else if (ci == 11) cmkr = java.awt.Color.yellow;
                else cmkr = java.awt.Color.black;
                return cmkr;
                }

        int markerOf(int mi)
                {
                int mkr = 3;
                if      (mi == 0) mkr = ij.gui.Plot.BOX;
                else if (mi == 1) mkr = ij.gui.Plot.CIRCLE;
                else if (mi == 2) mkr = ij.gui.Plot.CROSS;
                else if (mi == 3) mkr = ij.gui.Plot.DOT;
                else if (mi == 4) mkr = ij.gui.Plot.LINE;
                else if (mi == 5) mkr = ij.gui.Plot.TRIANGLE;
                else if (mi == 6) mkr = ij.gui.Plot.X;
                else              mkr = ij.gui.Plot.DOT;
                return mkr;
                }


        String scaleShiftText(boolean showScale, boolean showShift, boolean usemmag, boolean showmmaginfo, double scale, double shift)
                {
                String shiftText = "";
                String scaleText = "";
                NumberFormat Eformatter = new DecimalFormat("0.###E0");
                NumberFormat dformatter = new DecimalFormat("##0.###");
                if (showScale && (scale !=1.0))
                        {
                        if (((scale >= 0.01) && (scale < 100.0)) || ((scale <= -0.01) && (scale > -100.0)))
                                scaleText += " x("+dformatter.format(scale)+")";
                        else
                                if (usemmag && showmmaginfo && (scale == 0.001))
                                        scaleText +="";
                                else
                                        scaleText += " x("+Eformatter.format(scale)+")";
                        }
                if (showShift)
                        {
                        if (shift < 0.0)
                                {
                                shift *= -1.0;
                                shiftText += " + ";
                                }
                        else if (shift != 0.0)
                                shiftText += " - ";
                        if ((shift >= 0.01) && (shift < 100.0))
                                shiftText += dformatter.format(shift);
                        else if (shift != 0.0)
                                shiftText += Eformatter.format(shift);
                        }
                return shiftText + scaleText;
                }

        String convertToText(double increment)
                {
                if      (increment == 0.0000000001) return "      0.0000000001";
                else if (increment == 0.000000001)  return "        0.000000001";
                else if (increment == 0.00000001)   return "          0.00000001";
                else if (increment == 0.0000001)    return "            0.0000001";
                else if (increment == 0.000001)     return "              0.000001";
                else if (increment == 0.00001)      return "                0.00001";
                else if (increment == 0.0001)       return "                  0.0001";
                else if (increment == 0.001)        return "                    0.001";
                else if (increment == 0.01)         return "                    0.010";
                else if (increment == 0.1)          return "                    0.100";
                else if (increment == 0.5)          return "                    0.500";
                else if (increment == 1.0)          return "                    1.000";
                else if (increment == 10.0)         return "                  10.000";
                else if (increment == 100.0)        return "                100.000";
                else if (increment == 1000.0)       return "              1000.000";
                else if (increment == 10000.0)      return "            10000.000";
                else if (increment == 100000.0)     return "          100000.000";
                else if (increment == 1000000.0)    return "        1000000.000";
                else if (increment == 10000000.0)   return "      10000000.000";
                else if (increment == 100000000.0)  return "    100000000.000";
                else if (increment == 1000000000.0) return "1000000000.000";
                else return "                    1.000";
                }

        String intConvertToText(int increment)
                {
                if      (increment == 1)          return "                    1";
                else if (increment == 2)          return "                    2";
                else if (increment == 3)          return "                    3";
                else if (increment == 4)          return "                    4";
                else if (increment == 5)          return "                    5";
                else if (increment == 6)          return "                    6";
                else if (increment == 7)          return "                    7";
                else if (increment == 8)          return "                    8";
                else if (increment == 9)          return "                    9";
                else if (increment == 10)         return "                  10";
                else if (increment == 25)         return "                  25";
                else if (increment == 50)         return "                  50";
                else if (increment == 100)        return "                100";
                else if (increment == 250)        return "                250";
                else if (increment == 500)        return "                500";
                else if (increment == 1000)       return "              1000";
//                else if (increment == 10000)      return "            10000";
//                else if (increment == 100000)     return "          100000";
//                else if (increment == 1000000)    return "        1000000";
//                else if (increment == 10000000)   return "      10000000";
//                else if (increment == 100000000)  return "    100000000";
//                else if (increment == 1000000000) return "1000000000";
                else return "                    1";
                }


        void showMainJPanel()
                {
// DISPLAY MAIN PANEL
                mainFrame.setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
                mainFrame.addWindowListener(new WindowAdapter(){
                    @Override
                    public void windowClosing(WindowEvent e){
                    saveAndClose();}});
                    
                JPanel mainpanel = new JPanel (new SpringLayout());

                JScrollPane mainscrollpane = new JScrollPane(mainpanel);
                
                JPanel mainpanelgroupa = new JPanel (new SpringLayout());
                mainpanelgroupa.setBorder(BorderFactory.createTitledBorder("Data"));
                JPanel mainpanelgroupb = new JPanel (new SpringLayout());
                mainpanelgroupb.setBorder(BorderFactory.createTitledBorder("Data Options"));
                JPanel mainpanelgroupc = new JPanel (new SpringLayout());   //TITLE - SUBTITLE - LEGEND GROUP
                JPanel mainpanelgroupd = new JPanel (new SpringLayout());   // X & Y AXIS LEGEND GROUP
                JPanel mainpanelgroupe = new JPanel (new SpringLayout());
                mainpanelgroupe.setBorder(BorderFactory.createTitledBorder("Scaling Options"));
                JPanel mainpanelgroupf = new JPanel (new SpringLayout());

    // BUILD MENUBAR

                JMenuItem exitmenuitem = new JMenuItem("Exit");
                exitmenuitem.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                            saveAndClose(); }});
                filemenu.add(exitmenuitem);
                mainmenubar.add(filemenu);

                opendatasetCB = new JCheckBoxMenuItem("Open Y-data columns window at startup",openDataSetWindow);
                opendatasetCB.addItemListener(new ItemListener(){
                    public void itemStateChanged(ItemEvent e) {
                        if (e.getStateChange() == ItemEvent.DESELECTED)
                                openDataSetWindow = false;
                        else if (e.getStateChange() == ItemEvent.SELECTED)
                                openDataSetWindow = true;
                        Prefs.set("plot.openDataSetWindow",openDataSetWindow); }});
                preferencesmenu.add(opendatasetCB);

                usewidedataCB = new JCheckBoxMenuItem("Use --wide-- Y-data columns window",useWideDataPanel);
                usewidedataCB.addItemListener(new ItemListener(){
                    public void itemStateChanged(ItemEvent e) {
                        if (e.getStateChange() == ItemEvent.DESELECTED)
                                useWideDataPanel = false;
                        else if (e.getStateChange() == ItemEvent.SELECTED)
                                useWideDataPanel = true;
                                Prefs.set("plot.useWideDataPanel",useWideDataPanel);
                                subframeWasShowing = false;
                                if (subFrame.isShowing())
                                    {
                                    subFrameLocationX = subFrame.getLocation().x;
                                    subFrameLocationY = subFrame.getLocation().y;
                                    Prefs.set("plot.subFrameLocationX",subFrameLocationX);
                                    Prefs.set("plot.subFrameLocationY",subFrameLocationY);
                                    subFrame.dispose();
                                    subframeWasShowing = true;
                                    }

                                subFrame = new JFrame ("Plot_IJ Y-datasets");
                                mainsubpanel = new JPanel (new SpringLayout());
                                subscrollpane = new JScrollPane(mainsubpanel);

                                if (subframeWasShowing)
                                        showMoreCurvesJPanel();
                                }});

                preferencesmenu.add(usewidedataCB);  

                rememberwindowlocationsCB = new JCheckBoxMenuItem("Open windows at previous locations",rememberWindowLocations);
                rememberwindowlocationsCB.addItemListener(new ItemListener(){
                    public void itemStateChanged(ItemEvent e) {
                        if (e.getStateChange() == ItemEvent.DESELECTED)
                                rememberWindowLocations = false;
                        else if (e.getStateChange() == ItemEvent.SELECTED)
                                rememberWindowLocations = true;
                        Prefs.set("plot.rememberWindowLocations",rememberWindowLocations); }});
                preferencesmenu.add(rememberwindowlocationsCB);

                showtooltipsCB = new JCheckBoxMenuItem("Show tool tips (restart required)",showToolTips);
                showtooltipsCB.addItemListener(new ItemListener(){
                    public void itemStateChanged(ItemEvent e) {
                        if (e.getStateChange() == ItemEvent.DESELECTED)
                                showToolTips = false;
                        else if (e.getStateChange() == ItemEvent.SELECTED)
                                showToolTips = true;
                        Prefs.set("plot.showToolTips",showToolTips); }});
                preferencesmenu.add(showtooltipsCB);

                usedefaultsettingsCB = new JCheckBoxMenuItem("Reset preferences to default settings (restart required)",useDefaultSettings);
                usedefaultsettingsCB.addItemListener(new ItemListener(){
                    public void itemStateChanged(ItemEvent e) {
                        if (e.getStateChange() == ItemEvent.DESELECTED)
                                useDefaultSettings = false;
                        else if (e.getStateChange() == ItemEvent.SELECTED)
                                useDefaultSettings = true;
                        Prefs.set("plot.useDefaultSettings",useDefaultSettings); }});
                preferencesmenu.add(usedefaultsettingsCB);

                mainmenubar.add(preferencesmenu);



    // MAIN PANEL GROUP A START
                xcolumnlabel = new JLabel ("X-data Column");
                xcolumnlabel.setHorizontalAlignment(JLabel.CENTER);
                mainpanelgroupa.add (xcolumnlabel);

                ycolumnslabel = new JLabel ("Y-data Columns");
                ycolumnslabel.setHorizontalAlignment(JLabel.CENTER);
                mainpanelgroupa.add (ycolumnslabel);

                JLabel dummy50label = new JLabel ("Max Number of Y-datasets");
                dummy50label.setHorizontalAlignment(JLabel.CENTER);
//                dummy50label.setPreferredSize(new Dimension(150, 10));
                mainpanelgroupa.add (dummy50label);

                JLabel mmagsampleslabel = new JLabel ("Relative Magnitude Reference");
                mmagsampleslabel.setHorizontalAlignment(JLabel.CENTER);
//                mmagsampleslabel.setPreferredSize(new Dimension(150, 10));
                if (showToolTips) mmagsampleslabel.setToolTipText("The number of starting samples averaged to calculate relative magnitude reference level");
                mainpanelgroupa.add (mmagsampleslabel);

                JLabel dummy51label = new JLabel ("");
                dummy51label.setHorizontalAlignment(JLabel.CENTER);
                dummy51label.setPreferredSize(new Dimension(150, 10));
                mainpanelgroupa.add (dummy51label);

                JPanel xdatacolumnpanel = new JPanel(new SpringLayout());
                xdatacolumn = new JComboBox(columns);
                xdatacolumn.setSelectedItem(xlabel);
                xdatacolumn.setPrototypeDisplayValue("123456789012345");
                xdatacolumn.addActionListener(new ActionListener(){
                    public void actionPerformed(ActionEvent ae) {
                        xlabel = (String) xdatacolumn.getSelectedItem();
                        updatePlot(); }});
                        
                xdatacolumnpanel.add (xdatacolumn);
                xdatacolumnpanel.setPreferredSize(new Dimension(125, 25));
                SpringUtil.makeCompactGrid (xdatacolumnpanel, 1, 1, 0,0,0,0);
                mainpanelgroupa.add (xdatacolumnpanel);

                JPanel moreybuttonpanel = new JPanel(new SpringLayout());
                moreybutton = new JButton("     Open     ");
                moreybutton.setPreferredSize(new Dimension(150, 20));
                moreybutton.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        if (subFrame.isShowing())
                            subFrame.setVisible(true);
                        else
                            {
                            subFrame = new JFrame ("Plot_IJ Y-datasets");
                            mainsubpanel = new JPanel (new SpringLayout());
                            subscrollpane = new JScrollPane(mainsubpanel);
                            setSubpanelVisible = true;
                            showMoreCurvesJPanel();
                            }
                    }
                });
                subFrame = new JFrame ("Plot_IJ Y-datasets");
                mainsubpanel = new JPanel ();
                subscrollpane = new JScrollPane(mainsubpanel);

                moreybuttonpanel.add (moreybutton);
                moreybuttonpanel.setPreferredSize(new Dimension(125, 25));
                SpringUtil.makeCompactGrid (moreybuttonpanel, 1, 1, 0,0,0,0);
                mainpanelgroupa.add (moreybuttonpanel);

                JPanel maxcurvespanel = new JPanel(new SpringLayout());
                maxcurvespanel.setBorder(BorderFactory.createLineBorder(Color.lightGray,1));
                maxcurvesmodel = new SpinnerNumberModel(new Integer(maxCurves),new Integer(1), null, new Integer(1));
                maxcurvesspinner = new JSpinner(maxcurvesmodel);
                maxcurvesspinner.setPreferredSize(new Dimension(110,25));
                maxcurvesspinner.addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        maxCurves = ((Integer)maxcurvesspinner.getValue()).intValue();
                        Prefs.set("plot.maxCurves", maxCurves);
                        subframeWasShowing = false;
                        if (subFrame.isShowing())
                            {
                            subFrameLocationX = subFrame.getLocation().x;
                            subFrameLocationY = subFrame.getLocation().y;
                            Prefs.set("plot.subFrameLocationX",subFrameLocationX);
                            Prefs.set("plot.subFrameLocationY",subFrameLocationY);
                            subFrame.dispose();
                            subframeWasShowing = true;
                            }
                        setupArrays();
                        getPreferences();
                        subFrame = new JFrame ("Plot_IJ Y-datasets");
                        mainsubpanel = new JPanel (new SpringLayout());
                        subscrollpane = new JScrollPane(mainsubpanel);
                        if (subframeWasShowing)
                                showMoreCurvesJPanel();
                        updatePlot();
                        } });
                maxcurvesspinner.addMouseWheelListener( new MouseWheelListener()
                    {
                    public void mouseWheelMoved( MouseWheelEvent e )
                        {
                        int newValue =  new Integer(((Integer)maxcurvesspinner.getValue()).intValue() - e.getWheelRotation());
                        if (newValue > 0) maxcurvesspinner.setValue(newValue);
                        }
                    });
                maxcurvespanel.add (maxcurvesspinner);

                JLabel maxcurveslabel2 = new JLabel("datasets");
                maxcurveslabel2.setHorizontalAlignment(JLabel.LEFT);
                maxcurvespanel.add (maxcurveslabel2);
                SpringUtil.makeCompactGrid (maxcurvespanel, 1, 2, 0,0,0,0);
                mainpanelgroupa.add(maxcurvespanel);

                JPanel mmagsubpanel = new JPanel (new SpringLayout());
                mmagsubpanel.setBorder(BorderFactory.createLineBorder(Color.lightGray,1));
                if (showToolTips) mmagsubpanel.setToolTipText("The number of starting samples averaged to calculate relative magnitude reference level");
                mmagrefsmodel = new SpinnerNumberModel(new Integer(mmagrefs), new Integer (1), null, new Integer(1));
                mmagrefsspinner = new JSpinner(mmagrefsmodel);
                mmagrefsspinner.setPreferredSize(new Dimension(110, 25));
                if (showToolTips) mmagrefsspinner.setToolTipText("The number of starting samples averaged to calculate relative magnitude reference level");
                mmagrefsspinner.addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        mmagrefs = ((Integer)mmagrefsspinner.getValue()).intValue();
                        updatePlot();
                        }
                    });
                mmagrefsspinner.addMouseWheelListener( new MouseWheelListener()
                    {
                    public void mouseWheelMoved( MouseWheelEvent e )
                        {
                        int newValue =  new Integer(((Integer)mmagrefsspinner.getValue()).intValue() - e.getWheelRotation());
                        if (newValue > 0) mmagrefsspinner.setValue(newValue);
                        }
                    });
                mmagsubpanel.add (mmagrefsspinner);

                mmagrefsspinnerlabel = new JLabel ("samples");
                mmagrefsspinnerlabel.setHorizontalAlignment(JLabel.LEFT);
                mmagsubpanel.add (mmagrefsspinnerlabel);
                SpringUtil.makeCompactGrid (mmagsubpanel, 1, 2, 0,0,0,0);
                mainpanelgroupa.add (mmagsubpanel);

                JLabel dummy61label = new JLabel ("");
                dummy61label.setHorizontalAlignment(JLabel.CENTER);
                dummy61label.setPreferredSize(new Dimension(150, 25));
                mainpanelgroupa.add (dummy61label);


                SpringUtil.makeCompactGrid (mainpanelgroupa, 2, 5, 5,5,5,5);
                mainpanel.add(mainpanelgroupa);

    // MAIN PANEL GROUP C START

                // TITLE GROUP
                        JPanel titlegroup = new JPanel (new SpringLayout());
                        titlegroup.setBorder (BorderFactory.createEmptyBorder(10,10,10,10));
                        titlegroup.setBorder(BorderFactory.createTitledBorder("Title"));

                                // TITLE RADIO BUTTON GROUP
                                JPanel titleradiopanelgroup = new JPanel (new SpringLayout());
                                        JRadioButton noTitleButton = new JRadioButton("None");
                                        if (!useTitle) noTitleButton.setSelected(true);
                                        else noTitleButton.setSelected(false);
                                titleradiopanelgroup.add (noTitleButton);
                                        JRadioButton useTitleButton = new JRadioButton("Custom");
                                        if (useTitle) useTitleButton.setSelected(true);
                                        else useTitleButton.setSelected(false);
                                titleradiopanelgroup.add (useTitleButton);
                                        ButtonGroup titleRadioGroup = new ButtonGroup();
                                        titleRadioGroup.add(noTitleButton);
                                        titleRadioGroup.add(useTitleButton);
                                        noTitleButton.addActionListener(new ActionListener(){
                                            public void actionPerformed(ActionEvent ae) {
                                                useTitle = false;
                                                updatePlot(); }});
                                        useTitleButton.addActionListener(new ActionListener(){
                                            public void actionPerformed(ActionEvent ae) {
                                                useTitle =  true;
                                                updatePlot(); }});
                                SpringUtil.makeCompactGrid (titleradiopanelgroup, 1, 2, 0,0,0,0);
                        titlegroup.add (titleradiopanelgroup);

                                // TITLE TEXT FIELD
                                titleField = new JTextField (title);
                                titleField.setBorder (BorderFactory.createEmptyBorder(0,4,0,4));
                                titleField.getDocument().addDocumentListener (new DocumentListener()
                                    {
                                    public void insertUpdate (DocumentEvent ev)
                                        {
                                        title = titleField.getText();
                                        updatePlot();
                                        }
                                    public void removeUpdate (DocumentEvent ev)
                                        {
                                        title = titleField.getText();
                                        updatePlot();
                                        }
                                    public void changedUpdate (DocumentEvent ev)
                                        {
                                        title = titleField.getText();
                                        updatePlot();
                                        }
                                    });
                                titleField.setPreferredSize (new Dimension(250,20));
                                titleField.setHorizontalAlignment (JTextField.LEFT);
                        titlegroup.add (titleField);

                                //MAKE TITLE Y-POSITION SLIDER
                                
                                titlePosYSlider = new JSlider(JSlider.HORIZONTAL, 0,500, (int) (titlePosY*500.0));
                                titlePosYSlider.setBorder (BorderFactory.createEmptyBorder(10,0,0,0));
                                titlePosYSlider.addChangeListener (new ChangeListener()
                                    {
                                    public void stateChanged(ChangeEvent ev)
                                        {
                                        JSlider slider=(JSlider)ev.getSource();
                                        titlePosY=slider.getValue()/500.0;
                                        updatePlot();

                                        }
                                    });
                                titlePosYSlider.addMouseWheelListener( new MouseWheelListener()
                                    {
                                    public void mouseWheelMoved( MouseWheelEvent e )
                                        {
                                        int newValue =  new Integer(((Integer)titlePosYSlider.getValue()).intValue() - e.getWheelRotation());
                                        if ((newValue >= 0) && (newValue <=500)) titlePosYSlider.setValue(newValue);
                                        }
                                    });
                                titlePosYSlider.setPreferredSize (new Dimension(250,50));
                                titlePosYSlider.setInverted(false);
                                titlePosYSlider.setMajorTickSpacing(250);
                                titlePosYSlider.setMinorTickSpacing(25);
                                titlePosYSlider.setPaintTicks(true);
                                Hashtable<Integer, JLabel> titlePosYLabel = new Hashtable<Integer, JLabel>();
                                titlePosYLabel.put( 0, new JLabel("Top"));
                                titlePosYLabel.put( 250, new JLabel("Middle") );
                                titlePosYLabel.put( 500, new JLabel("Bottom") );
                                titlePosYSlider.setLabelTable( titlePosYLabel );
                                titlePosYSlider.setPaintTrack(true);
                                titlePosYSlider.setPaintLabels(true);
                        titlegroup.add (titlePosYSlider);

                                //MAKE TITLE X-POSITION SLIDER
                                
                                titlePosXSlider = new JSlider(0,500, (int) (titlePosX*500.0));
                                titlePosXSlider.setBorder (BorderFactory.createEmptyBorder(10,5,0,5));
                                titlePosXSlider.addChangeListener (new ChangeListener()
                                    {
                                    public void stateChanged(ChangeEvent ev)
                                        {
                                        JSlider slider=(JSlider)ev.getSource();
                                        titlePosX=slider.getValue()/500.0;
                                        updatePlot();
                                        }
                                    });
                                titlePosXSlider.addMouseWheelListener( new MouseWheelListener()
                                    {
                                    public void mouseWheelMoved( MouseWheelEvent e )
                                        {
                                        int newValue =  new Integer(((Integer)titlePosXSlider.getValue()).intValue() - e.getWheelRotation());
                                        if ((newValue >= 0) && (newValue <=500)) titlePosXSlider.setValue(newValue);
                                        }
                                    });
                                titlePosXSlider.setPreferredSize (new Dimension(250,50));
                                titlePosXSlider.setMajorTickSpacing(250);
                                titlePosXSlider.setMinorTickSpacing(25);
                                titlePosXSlider.setPaintTicks(true);
                                Hashtable<Integer, JLabel> titlePosXLabel = new Hashtable<Integer, JLabel>();
                                titlePosXLabel.put( 0, new JLabel("Left"));
                                titlePosXLabel.put( 250, new JLabel("Center") );
                                titlePosXLabel.put( 500, new JLabel("Right") );
                                titlePosXSlider.setLabelTable( titlePosXLabel );
                                titlePosXSlider.setPaintTrack(true);
                                titlePosXSlider.setPaintLabels(true);
                        titlegroup.add (titlePosXSlider);
                        SpringUtil.makeCompactGrid (titlegroup, 4, 1, 0,0,0,0);
		mainpanelgroupc.add (titlegroup);

                        //SUBTITLE GROUP
                        JPanel subtitlegroup = new JPanel (new SpringLayout());
                        subtitlegroup.setBorder(BorderFactory.createTitledBorder("Subtitle"));

                                // SUBTITLE RADIO BUTTON GROUP
                                JPanel subtitleradiopanelgroup = new JPanel (new SpringLayout());
                                        JRadioButton noSubtitleButton = new JRadioButton("None");
                                        if (!useSubtitle) noSubtitleButton.setSelected(true);
                                        else noSubtitleButton.setSelected(false);
                                subtitleradiopanelgroup.add (noSubtitleButton);
                                        JRadioButton useSubtitleButton = new JRadioButton("Custom");
                                        if (useSubtitle) useSubtitleButton.setSelected(true);
                                        else useSubtitleButton.setSelected(false);
                                subtitleradiopanelgroup.add (useSubtitleButton);
                                        ButtonGroup subtitleRadioGroup = new ButtonGroup();
                                        subtitleRadioGroup.add(noSubtitleButton);
                                        subtitleRadioGroup.add(useSubtitleButton);
                                        noSubtitleButton.addActionListener(new ActionListener(){
                                            public void actionPerformed(ActionEvent ae) {
                                                useSubtitle = false;
                                                updatePlot(); }});
                                        useSubtitleButton.addActionListener(new ActionListener(){
                                            public void actionPerformed(ActionEvent ae) {
                                                useSubtitle =  true;
                                                updatePlot(); }});
                                SpringUtil.makeCompactGrid (subtitleradiopanelgroup, 1, 2, 0,0,0,0);
                        subtitlegroup.add (subtitleradiopanelgroup);

                                // SUBTITLE TEXT FIELD
                                subtitleField = new JTextField (subtitle);
                                subtitleField.setBorder (BorderFactory.createEmptyBorder(0,4,0,4));
                                subtitleField.getDocument().addDocumentListener (new DocumentListener()
                                    {
                                    public void insertUpdate (DocumentEvent ev)
                                        {
                                        subtitle = subtitleField.getText();
                                        updatePlot();
                                        }
                                    public void removeUpdate (DocumentEvent ev)
                                        {
                                        subtitle = subtitleField.getText();
                                        updatePlot();
                                        }
                                    public void changedUpdate (DocumentEvent ev)
                                        {
                                        subtitle = subtitleField.getText();
                                        updatePlot();
                                        }
                                    });
                                subtitleField.setPreferredSize (new Dimension(250,20));
                                subtitleField.setHorizontalAlignment (JTextField.LEFT);

                        subtitlegroup.add (subtitleField);

                                //SUBTITLE Y-POSITION SLIDER
                                
                                subtitlePosYSlider = new JSlider(JSlider.HORIZONTAL, 0,500, (int) (subtitlePosY*500.0));
                                subtitlePosYSlider.setBorder (BorderFactory.createEmptyBorder(10,0,0,0));
                                subtitlePosYSlider.addChangeListener (new ChangeListener()
                                    {
                                    public void stateChanged(ChangeEvent ev)
                                        {
                                        JSlider slider=(JSlider)ev.getSource();
                                        subtitlePosY=slider.getValue()/500.0;
                                        updatePlot();

                                        }
                                    });
                                subtitlePosYSlider.addMouseWheelListener( new MouseWheelListener()
                                    {
                                    public void mouseWheelMoved( MouseWheelEvent e )
                                        {
                                        int newValue =  new Integer(((Integer)subtitlePosYSlider.getValue()).intValue() - e.getWheelRotation());
                                        if ((newValue >= 0) && (newValue <=500)) subtitlePosYSlider.setValue(newValue);
                                        }
                                    });
                                subtitlePosYSlider.setPreferredSize (new Dimension(250,50));
                                subtitlePosYSlider.setInverted(false);
                                subtitlePosYSlider.setMajorTickSpacing(250);
                                subtitlePosYSlider.setMinorTickSpacing(25);
                                subtitlePosYSlider.setPaintTicks(true);
                                Hashtable<Integer, JLabel> subtitlePosYLabel = new Hashtable<Integer, JLabel>();

                                subtitlePosYLabel.put( 0, new JLabel("Top"));
                                subtitlePosYLabel.put( 250, new JLabel("Middle") );
                                subtitlePosYLabel.put( 500, new JLabel("Bottom") );
                                subtitlePosYSlider.setLabelTable( titlePosYLabel );
                                subtitlePosYSlider.setPaintTrack(true);
                                subtitlePosYSlider.setPaintLabels(true);
                        subtitlegroup.add (subtitlePosYSlider);

                                //SUBTITLE X-POSITION SLIDER
                                
                                subtitlePosXSlider = new JSlider(0,500, (int) (subtitlePosX*500.0));
                                subtitlePosXSlider.setBorder (BorderFactory.createEmptyBorder(10,5,0,5));
                                subtitlePosXSlider.addChangeListener (new ChangeListener()
                                    {
                                    public void stateChanged(ChangeEvent ev)
                                        {
                                        JSlider slider=(JSlider)ev.getSource();
                                        subtitlePosX=slider.getValue()/500.0;
                                        updatePlot();

                                        }
                                    });
                                subtitlePosXSlider.addMouseWheelListener( new MouseWheelListener()
                                    {
                                    public void mouseWheelMoved( MouseWheelEvent e )
                                        {
                                        int newValue =  new Integer(((Integer)subtitlePosXSlider.getValue()).intValue() - e.getWheelRotation());
                                        if ((newValue >= 0) && (newValue <=500)) subtitlePosXSlider.setValue(newValue);
                                        }
                                    });
                                subtitlePosXSlider.setPreferredSize (new Dimension(250,50));
                                subtitlePosXSlider.setMajorTickSpacing(250);
                                subtitlePosXSlider.setMinorTickSpacing(25);
                                subtitlePosXSlider.setPaintTicks(true);
                                Hashtable<Integer, JLabel> subtitlePosXLabel = new Hashtable<Integer, JLabel>();
                                subtitlePosXLabel.put( 0, new JLabel("Left"));
                                subtitlePosXLabel.put( 250, new JLabel("Center") );
                                subtitlePosXLabel.put( 500, new JLabel("Right") );
                                subtitlePosXSlider.setLabelTable( titlePosXLabel );
                                subtitlePosXSlider.setPaintTrack(true);
                                subtitlePosXSlider.setPaintLabels(true);
                        subtitlegroup.add (subtitlePosXSlider);
                        SpringUtil.makeCompactGrid (subtitlegroup, 4, 1, 0,0,0,0);
                mainpanelgroupc.add (subtitlegroup);


                        // LEGEND GROUP
                        JPanel legendgroup = new JPanel (new SpringLayout());
                        legendgroup.setBorder(BorderFactory.createTitledBorder("Legend"));

                        // LEGEND LABEL POPUP

                        legendpopup = new JPopupMenu();
                        JMenuItem legendshowscalingCB = new JCheckBoxMenuItem("Append scaling factor to legend",showLScaleInfo);
                        legendshowscalingCB.addItemListener(new ItemListener(){
                            public void itemStateChanged(ItemEvent e) {
                                if (e.getStateChange() == ItemEvent.DESELECTED)
                                        showLScaleInfo = false;
                                else if (e.getStateChange() == ItemEvent.SELECTED)
                                        showLScaleInfo = true;
                                updatePlot(); }});
                        legendpopup.add(legendshowscalingCB);
                        JMenuItem legendshowshiftCB = new JCheckBoxMenuItem("Append shift factor to legend",showLShiftInfo);
                        legendshowshiftCB.addItemListener(new ItemListener(){
                            public void itemStateChanged(ItemEvent e) {
                                if (e.getStateChange() == ItemEvent.DESELECTED)
                                        showLShiftInfo = false;
                                else if (e.getStateChange() == ItemEvent.SELECTED)
                                        showLShiftInfo = true;
                                updatePlot(); }});
                        legendpopup.add(legendshowshiftCB);
                        JMenuItem legendshowbinCB = new JCheckBoxMenuItem("Append bin size to legend",showLBinInfo);
                        legendshowbinCB.addItemListener(new ItemListener(){
                            public void itemStateChanged(ItemEvent e) {
                                if (e.getStateChange() == ItemEvent.DESELECTED)
                                        showLBinInfo = false;
                                else if (e.getStateChange() == ItemEvent.SELECTED)
                                        showLBinInfo = true;
                                updatePlot(); }});
                        legendpopup.add(legendshowbinCB);
                        JMenuItem legendshowmmagCB = new JCheckBoxMenuItem("Append magnitude usage to legend",showLmmagInfo);
                        legendshowmmagCB.addItemListener(new ItemListener(){
                            public void itemStateChanged(ItemEvent e) {
                                if (e.getStateChange() == ItemEvent.DESELECTED)
                                        showLmmagInfo = false;
                                else if (e.getStateChange() == ItemEvent.SELECTED)
                                        showLmmagInfo = true;
                                updatePlot(); }});
                        legendpopup.add(legendshowmmagCB);
                        JMenuItem legendshowsymbolCB = new JCheckBoxMenuItem("Append symbol description to legend",showLSymbolInfo);
                        legendshowsymbolCB.addItemListener(new ItemListener(){
                            public void itemStateChanged(ItemEvent e) {
                                if (e.getStateChange() == ItemEvent.DESELECTED)
                                        showLSymbolInfo = false;
                                else if (e.getStateChange() == ItemEvent.SELECTED)
                                        showLSymbolInfo = true;
                                updatePlot(); }});
                        legendpopup.add(legendshowsymbolCB);
                        JMenuItem twolinelegendCB = new JCheckBoxMenuItem("Use two lines per legend",useTwoLineLegend);
                        twolinelegendCB.addItemListener(new ItemListener(){
                            public void itemStateChanged(ItemEvent e) {
                                if (e.getStateChange() == ItemEvent.DESELECTED)
                                        useTwoLineLegend = false;
                                else if (e.getStateChange() == ItemEvent.SELECTED)
                                        useTwoLineLegend = true;
                                updatePlot(); }});
                        legendpopup.add(twolinelegendCB);


                        legendpopup.addPopupMenuListener (new PopupMenuListener() {
                                public void popupMenuCanceled(PopupMenuEvent popupMenuEvent) { }
                                public void popupMenuWillBecomeInvisible(PopupMenuEvent popupMenuEvent) { }
                                public void popupMenuWillBecomeVisible(PopupMenuEvent popupMenuEvent) { } });


                                // LEGEND RADIO BUTTON GROUP
                                JPanel legendradiopanelgroup = new JPanel (new SpringLayout());
                                JLabel legendradiolabel = new JLabel("Justification:");
                                if (showToolTips) legendradiolabel.setToolTipText("Right click to set legend preferences");
                                legendradiopanelgroup.add (legendradiolabel);
                                        JRadioButton leftButton = new JRadioButton("Left");
                                        if (legendLeft)
                                                leftButton.setSelected(true);
                                        else
                                                leftButton.setSelected(false);
                                legendradiopanelgroup.add (leftButton);
                                        JRadioButton centerButton = new JRadioButton("Center");
                                        if (!legendLeft && !legendRight)
                                                centerButton.setSelected(true);
                                        else
                                                centerButton.setSelected(false);
                                legendradiopanelgroup.add (centerButton);
                                        JRadioButton rightButton = new JRadioButton("Right");
                                        if (legendRight)
                                                rightButton.setSelected(true);
                                        else
                                                rightButton.setSelected(false);
                                legendradiopanelgroup.add (rightButton);

                                        ButtonGroup legendRadioGroup = new ButtonGroup();
                                        legendRadioGroup.add(leftButton);
                                        legendRadioGroup.add(centerButton);
                                        legendRadioGroup.add(rightButton);
                                        leftButton.addActionListener(new ActionListener(){
                                            public void actionPerformed(ActionEvent ae) {
                                                legendLeft = true;
                                                legendRight = false;
                                                updatePlot(); }});
                                        centerButton.addActionListener(new ActionListener(){
                                            public void actionPerformed(ActionEvent ae) {
                                                legendLeft = false;
                                                legendRight = false;
                                                updatePlot(); }});
                                        rightButton.addActionListener(new ActionListener(){
                                            public void actionPerformed(ActionEvent ae) {
                                                legendLeft = false;
                                                legendRight = true;
                                                updatePlot(); }});
                                SpringUtil.makeCompactGrid (legendradiopanelgroup, 1, 4, 0,0,0,0);
                                leftButton.setComponentPopupMenu(legendpopup);
                                if (showToolTips) leftButton.setToolTipText("Right click to set legend preferences");
                                rightButton.setComponentPopupMenu(legendpopup);
                                if (showToolTips) rightButton.setToolTipText("Right click to set legend preferences");
                                centerButton.setComponentPopupMenu(legendpopup);
                                if (showToolTips) centerButton.setToolTipText("Right click to set legend preferences");
                        legendgroup.add (legendradiopanelgroup);

                                // LEGEND SLIDER LABEL
                                JLabel legendsliderlabel = new JLabel("Position");
                                legendsliderlabel.setBorder (BorderFactory.createEmptyBorder(5,0,0,0));
                                if (showToolTips) legendsliderlabel.setToolTipText("Right click to set legend preferences");
                                legendsliderlabel.setHorizontalAlignment (JLabel.CENTER);
                                legendsliderlabel.setComponentPopupMenu(legendpopup);

                        legendgroup.add (legendsliderlabel);

                                //LEGEND Y-POSITION SLIDER
                                
                                legendPosYSlider = new JSlider(JSlider.HORIZONTAL, 0,500, (int) (legendPosY*500.0));
                                legendPosYSlider.setBorder (BorderFactory.createEmptyBorder(10,0,0,0));
                                legendPosYSlider.addChangeListener (new ChangeListener()
                                    {
                                    public void stateChanged(ChangeEvent ev)
                                        {
                                        JSlider slider=(JSlider)ev.getSource();
                                        legendPosY=slider.getValue()/500.0;
                                        updatePlot();
                                        }
                                    });
                                legendPosYSlider.addMouseWheelListener( new MouseWheelListener()
                                    {
                                    public void mouseWheelMoved( MouseWheelEvent e )
                                        {
                                        int newValue =  new Integer(((Integer)legendPosYSlider.getValue()).intValue() - e.getWheelRotation());
                                        if ((newValue >= 0) && (newValue <=500)) legendPosYSlider.setValue(newValue);
                                        }
                                    });
                                legendPosYSlider.setPreferredSize (new Dimension(250,50));
                                legendPosYSlider.setInverted(false);
                                legendPosYSlider.setMajorTickSpacing(250);
                                legendPosYSlider.setMinorTickSpacing(25);
                                legendPosYSlider.setPaintTicks(true);
                                Hashtable<Integer, JLabel> legendPosYLabel = new Hashtable<Integer, JLabel>();
                                legendPosYLabel.put( 0, new JLabel("Top"));
                                legendPosYLabel.put( 250, new JLabel("Middle") );
                                legendPosYLabel.put( 500, new JLabel("Bottom") );
                                legendPosYSlider.setLabelTable( titlePosYLabel );
                                legendPosYSlider.setPaintTrack(true);
                                legendPosYSlider.setPaintLabels(true);
                        legendgroup.add (legendPosYSlider);

                                //LEGEND X-POSITION SLIDER
                                
                                legendPosXSlider = new JSlider(0,500, (int) (legendPosX*500.0));
                                legendPosXSlider.setBorder (BorderFactory.createEmptyBorder(10,5,0,5));
                                legendPosXSlider.addChangeListener (new ChangeListener()
                                    {
                                    public void stateChanged(ChangeEvent ev)
                                        {
                                        JSlider slider=(JSlider)ev.getSource();
                                        legendPosX=slider.getValue()/500.0;
                                        updatePlot();
                                        }
                                    });
                                legendPosXSlider.addMouseWheelListener( new MouseWheelListener()
                                    {
                                    public void mouseWheelMoved( MouseWheelEvent e )
                                        {
                                        int newValue =  new Integer(((Integer)legendPosXSlider.getValue()).intValue() - e.getWheelRotation());
                                        if ((newValue >= 0) && (newValue <=500)) legendPosXSlider.setValue(newValue);
                                        }
                                    });
                                legendPosXSlider.setPreferredSize (new Dimension(250,50));
                                legendPosXSlider.setMajorTickSpacing(250);
                                legendPosXSlider.setMinorTickSpacing(25);
                                legendPosXSlider.setPaintTicks(true);
                                Hashtable<Integer, JLabel> legendPosXLabel = new Hashtable<Integer, JLabel>();
                                legendPosXLabel.put( 0, new JLabel("Left"));
                                legendPosXLabel.put( 250, new JLabel("Center") );
                                legendPosXLabel.put( 500, new JLabel("Right") );
                                legendPosXSlider.setLabelTable( titlePosXLabel );
                                legendPosXSlider.setPaintTrack(true);
                                legendPosXSlider.setPaintLabels(true);
                        legendgroup.add (legendPosXSlider);
                        SpringUtil.makeCompactGrid (legendgroup, 4, 1, 0,0,0,0);

                mainpanelgroupc.add (legendgroup);

                SpringUtil.makeCompactGrid (mainpanelgroupc, 1, 3, 0,0,0,0);
                mainpanel.add(mainpanelgroupc);

    // X & Y AXIS LEGEND GROUP (GROUP D)

            //X-AXIS LEGEND GROUP
                        JPanel xlegendgroup = new JPanel (new SpringLayout());
                        xlegendgroup.setBorder(BorderFactory.createTitledBorder("X-Axis Label"));

                        // X-AXIS LABEL POPUP

                        xaxispopup = new JPopupMenu();
                        JMenuItem showxscalingCB = new JCheckBoxMenuItem("Append scaling factor to X-axis label",showXScaleInfo);

                        showxscalingCB.addItemListener(new ItemListener(){
                            public void itemStateChanged(ItemEvent e) {
                                if (e.getStateChange() == ItemEvent.DESELECTED)
                                        showXScaleInfo = false;
                                else if (e.getStateChange() == ItemEvent.SELECTED)
                                        showXScaleInfo = true;
                                updatePlot(); }});
                        xaxispopup.add(showxscalingCB);
                        xaxispopup.addPopupMenuListener (new PopupMenuListener() {
                                public void popupMenuCanceled(PopupMenuEvent popupMenuEvent) { }
                                public void popupMenuWillBecomeInvisible(PopupMenuEvent popupMenuEvent) { }
                                public void popupMenuWillBecomeVisible(PopupMenuEvent popupMenuEvent) { } });

                        // X-AXIS LEGEND RADIO BUTTON GROUP
                                JPanel xlegendradiopanelgroup = new JPanel (new SpringLayout());
                                        JRadioButton xnoneButton = new JRadioButton("None");
                                        if (!useXCustomName && !useXColumnName)
                                                xnoneButton.setSelected(true);
                                        else
                                                xnoneButton.setSelected(false);
                                xlegendradiopanelgroup.add (xnoneButton);
                                        JRadioButton xcolumnNameButton = new JRadioButton("Column Label");
                                        if (useXColumnName)
                                                xcolumnNameButton.setSelected(true);
                                        else
                                                xcolumnNameButton.setSelected(false);
                                xlegendradiopanelgroup.add (xcolumnNameButton);
                                        JRadioButton xcustomNameButton = new JRadioButton("Custom Label");
                                        if (!useXColumnName && useXCustomName)
                                                xcustomNameButton.setSelected(true);
                                        else
                                                xcustomNameButton.setSelected(false);
                                xlegendradiopanelgroup.add (xcustomNameButton);

                                        ButtonGroup xlegendRadioGroup = new ButtonGroup();
                                        xlegendRadioGroup.add(xnoneButton);
                                        xlegendRadioGroup.add(xcolumnNameButton);
                                        xlegendRadioGroup.add(xcustomNameButton);
                                        xnoneButton.addActionListener(new ActionListener(){
                                            public void actionPerformed(ActionEvent ae) {
                                                useXColumnName = false;
                                                useXCustomName = false;
                                                updatePlot(); }});
                                        xcustomNameButton.addActionListener(new ActionListener(){
                                            public void actionPerformed(ActionEvent ae) {
                                                useXColumnName = false;
                                                useXCustomName = true;
                                                updatePlot(); }});
                                        xcolumnNameButton.addActionListener(new ActionListener(){
                                            public void actionPerformed(ActionEvent ae) {
                                                useXColumnName = true;
                                                useXCustomName = false;
                                                updatePlot(); }});
                                SpringUtil.makeCompactGrid (xlegendradiopanelgroup, 1, 3, 0,0,0,0);
                                xnoneButton.setComponentPopupMenu(xaxispopup);
                                if (showToolTips) xnoneButton.setToolTipText("Right click to set X-axis label preferences");
                                xcustomNameButton.setComponentPopupMenu(xaxispopup);
                                if (showToolTips) xcustomNameButton.setToolTipText("Right click to set X-axis label preferences");
                                xcolumnNameButton.setComponentPopupMenu(xaxispopup);
                                if (showToolTips) xcolumnNameButton.setToolTipText("Right click to set X-axis label preferences");
                        xlegendgroup.add (xlegendradiopanelgroup);

                                // X LEGEND TEXT FIELD
                                xlegendField = new JTextField (xLegend);
                                xlegendField.setBorder (BorderFactory.createEmptyBorder(0,4,0,4));
                                xlegendField.getDocument().addDocumentListener (new DocumentListener()
                                    {
                                    public void insertUpdate (DocumentEvent ev)
                                        {
                                        xLegend = xlegendField.getText();
                                        updatePlot();
                                        }
                                    public void removeUpdate (DocumentEvent ev)
                                        {
                                        xLegend = xlegendField.getText();
                                        updatePlot();
                                        }
                                    public void changedUpdate (DocumentEvent ev)
                                        {
                                        xLegend = xlegendField.getText();
                                        updatePlot();
                                        }
                                    });
                                xlegendField.setPreferredSize (new Dimension(250,20));
                                xlegendField.setHorizontalAlignment (JTextField.LEFT);
                                xlegendField.setComponentPopupMenu(xaxispopup);
                                if (showToolTips) xlegendField.setToolTipText("Right click to set X-axis label preferences");

                        xlegendgroup.add (xlegendField);
                        SpringUtil.makeCompactGrid (xlegendgroup, 2, 1, 0,0,0,0);
                mainpanelgroupd.add (xlegendgroup);

            //Y-AXIS LEGEND GROUP
                        JPanel ylegendgroup = new JPanel (new SpringLayout());
                        ylegendgroup.setBorder(BorderFactory.createTitledBorder("Y-Axis Label"));

                        // Y-AXIS LABEL POPUP

                        yaxispopup = new JPopupMenu();
                        JMenuItem showyscalingCB = new JCheckBoxMenuItem("Append scaling factor to Y-axis label",showYScaleInfo);
                        showyscalingCB.addItemListener(new ItemListener(){
                            public void itemStateChanged(ItemEvent e) {
                                if (e.getStateChange() == ItemEvent.DESELECTED)
                                        showYScaleInfo = false;
                                else if (e.getStateChange() == ItemEvent.SELECTED)
                                        showYScaleInfo = true;
                                updatePlot(); }});
                        yaxispopup.add(showyscalingCB);
                        JMenuItem showyshiftCB = new JCheckBoxMenuItem("Append shift factor to Y-axis label",showYShiftInfo);
                        showyshiftCB.addItemListener(new ItemListener(){
                            public void itemStateChanged(ItemEvent e) {
                                if (e.getStateChange() == ItemEvent.DESELECTED)
                                        showYShiftInfo = false;
                                else if (e.getStateChange() == ItemEvent.SELECTED)
                                        showYShiftInfo = true;
                                updatePlot(); }});
                        yaxispopup.add(showyshiftCB);
                        JMenuItem showybinCB = new JCheckBoxMenuItem("Append bin size to Y-axis label",showYBinInfo);
                        showybinCB.addItemListener(new ItemListener(){
                            public void itemStateChanged(ItemEvent e) {
                                if (e.getStateChange() == ItemEvent.DESELECTED)
                                        showYBinInfo = false;
                                else if (e.getStateChange() == ItemEvent.SELECTED)
                                        showYBinInfo = true;
                                updatePlot(); }});
                        yaxispopup.add(showybinCB);
                        JMenuItem showymmagCB = new JCheckBoxMenuItem("Append magnitude usage to Y-axis label",showYmmagInfo);
                        showymmagCB.addItemListener(new ItemListener(){
                            public void itemStateChanged(ItemEvent e) {
                                if (e.getStateChange() == ItemEvent.DESELECTED)
                                        showYmmagInfo = false;
                                else if (e.getStateChange() == ItemEvent.SELECTED)
                                        showYmmagInfo = true;
                                updatePlot(); }});
                        yaxispopup.add(showymmagCB);
                        JMenuItem showysymbolCB = new JCheckBoxMenuItem("Append symbol description to Y-axis label",showYSymbolInfo);
                        showysymbolCB.addItemListener(new ItemListener(){
                            public void itemStateChanged(ItemEvent e) {
                                if (e.getStateChange() == ItemEvent.DESELECTED)
                                        showYSymbolInfo = false;
                                else if (e.getStateChange() == ItemEvent.SELECTED)
                                        showYSymbolInfo = true;
                                updatePlot(); }});
                        yaxispopup.add(showysymbolCB);
                        yaxispopup.addPopupMenuListener (new PopupMenuListener() {
                                public void popupMenuCanceled(PopupMenuEvent popupMenuEvent) { }
                                public void popupMenuWillBecomeInvisible(PopupMenuEvent popupMenuEvent) { }
                                public void popupMenuWillBecomeVisible(PopupMenuEvent popupMenuEvent) { } });

                        // Y-AXIS LEGEND RADIO BUTTON GROUP
                                JPanel ylegendradiopanelgroup = new JPanel (new SpringLayout());
                                        JRadioButton ynoneButton = new JRadioButton("None");
                                        if (!useYCustomName && !useYColumnName)
                                                ynoneButton.setSelected(true);
                                        else
                                                ynoneButton.setSelected(false);
                                ylegendradiopanelgroup.add (ynoneButton);
                                        JRadioButton ycolumnNameButton = new JRadioButton("Column Label");
                                        if (useYColumnName)
                                                ycolumnNameButton.setSelected(true);
                                        else
                                                ycolumnNameButton.setSelected(false);
                                ylegendradiopanelgroup.add (ycolumnNameButton);
                                        JRadioButton ycustomNameButton = new JRadioButton("Custom Label");
                                        if (!useYColumnName && useYCustomName)
                                                ycustomNameButton.setSelected(true);
                                        else
                                                ycustomNameButton.setSelected(false);
                                ylegendradiopanelgroup.add (ycustomNameButton);

                                        ButtonGroup ylegendRadioGroup = new ButtonGroup();
                                        ylegendRadioGroup.add(ynoneButton);
                                        ylegendRadioGroup.add(ycustomNameButton);
                                        ylegendRadioGroup.add(ycolumnNameButton);
                                        ynoneButton.addActionListener(new ActionListener(){
                                            public void actionPerformed(ActionEvent ae) {
                                                useYColumnName = false;
                                                useYCustomName = false;
                                                updatePlot(); }});
                                        ycustomNameButton.addActionListener(new ActionListener(){
                                            public void actionPerformed(ActionEvent ae) {
                                                useYColumnName = false;
                                                useYCustomName = true;
                                                updatePlot(); }});
                                        ycolumnNameButton.addActionListener(new ActionListener(){
                                            public void actionPerformed(ActionEvent ae) {
                                                useYColumnName = true;
                                                useYCustomName = false;
                                                updatePlot(); }});
                                SpringUtil.makeCompactGrid (ylegendradiopanelgroup, 1, 3, 0,0,0,0);
                                ynoneButton.setComponentPopupMenu(yaxispopup);
                                if (showToolTips) ynoneButton.setToolTipText("Right click to set Y-axis label preferences");
                                ycustomNameButton.setComponentPopupMenu(yaxispopup);
                                if (showToolTips) ycustomNameButton.setToolTipText("Right click to set Y-axis label preferences");
                                ycolumnNameButton.setComponentPopupMenu(yaxispopup);
                                if (showToolTips) ycolumnNameButton.setToolTipText("Right click to set Y-axis label preferences");
                        ylegendgroup.add (ylegendradiopanelgroup);

                                // Y-LEGEND TEXT FIELD
                                ylegendField = new JTextField (yLegend);
                                ylegendField.setBorder (BorderFactory.createEmptyBorder(0,4,0,4));
                                ylegendField.getDocument().addDocumentListener (new DocumentListener()
                                    {
                                    public void insertUpdate (DocumentEvent ev)
                                        {
                                        yLegend = ylegendField.getText();
                                        updatePlot();
                                        }
                                    public void removeUpdate (DocumentEvent ev)
                                        {
                                        yLegend = ylegendField.getText();
                                        updatePlot();
                                        }
                                    public void changedUpdate (DocumentEvent ev)
                                        {
                                        yLegend = ylegendField.getText();
                                        updatePlot();
                                        }
                                    });
                                ylegendField.setPreferredSize (new Dimension(250,20));
                                ylegendField.setHorizontalAlignment (JTextField.LEFT);
                                ylegendField.setComponentPopupMenu(yaxispopup);
                                if (showToolTips) ylegendField.setToolTipText("Right click to set Y-axis label preferences");

                        ylegendgroup.add (ylegendField);
                        SpringUtil.makeCompactGrid (ylegendgroup, 2, 1, 0,0,0,0);
                mainpanelgroupd.add (ylegendgroup);

            //TRIM DATASET SUBPANEL

                JPanel trimdatapanel = new JPanel (new SpringLayout());
                trimdatapanel.setBorder(BorderFactory.createTitledBorder("Trim Data Samples"));

                //TRIM DATA FROM HEAD STEPSIZE POPUP

                trimdataheadsteppopup = new JPopupMenu();
                JPanel trimdataheadsteppanel = new JPanel();
                trimdataheadstepmodel = new SpinnerListModel(integerspinnerscalelist);
                trimdataheadstepspinner = new JSpinner(trimdataheadstepmodel);
                trimdataheadstepspinner.setValue(intConvertToText(excludedHeadSamplesStep));
                trimdataheadstepspinner.addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        excludedHeadSamplesStep = Integer.parseInt(((String)trimdataheadstepspinner.getValue()).trim());
                        trimdataheadspinner.setModel(new SpinnerNumberModel(new Integer(excludedHeadSamples), new Integer(0), null, new Integer(excludedHeadSamplesStep)));
                        Prefs.set("plot.excludedHeadSamplesStep",excludedHeadSamplesStep);
                        }
                    });

                JLabel trimdataheadsteplabel = new JLabel ("Stepsize:");
                trimdataheadsteppanel.add(trimdataheadsteplabel);
                trimdataheadsteppanel.add(trimdataheadstepspinner);
                trimdataheadsteppopup.add(trimdataheadsteppanel);
                trimdataheadsteppopup.setLightWeightPopupEnabled(false);
                trimdataheadsteppopup.addPopupMenuListener (new PopupMenuListener()
                    {
                    public void popupMenuCanceled(PopupMenuEvent popupMenuEvent) {
                            trimdataheadsteppopup.setVisible(false);
                            }

                    public void popupMenuWillBecomeInvisible(PopupMenuEvent popupMenuEvent) {
                            trimdataheadsteppopup.setVisible(true);
                            }

                    public void popupMenuWillBecomeVisible(PopupMenuEvent popupMenuEvent) {
                            trimdataheadsteppopup.setVisible(false);
                            }
                    });

                JLabel trimdataheadlabel = new JLabel ("Head");
                trimdataheadlabel.setHorizontalAlignment(JLabel.RIGHT);
                trimdataheadlabel.setPreferredSize(new Dimension(50, 25));
                if (showToolTips) trimdataheadlabel.setToolTipText("Right click to set spinner stepsize");
                trimdataheadlabel.setComponentPopupMenu(trimdataheadsteppopup);
                trimdatapanel.add (trimdataheadlabel);

                trimdataheadmodel = new SpinnerNumberModel(new Integer(excludedHeadSamples), new Integer(0), null, new Integer(excludedHeadSamplesStep));
                trimdataheadspinner = new JSpinner(trimdataheadmodel);
                trimdataheadspinner.setComponentPopupMenu(trimdataheadsteppopup);
                if (showToolTips) trimdataheadspinner.setToolTipText("Right click to set spinner stepsize");
                trimdataheadspinner.addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        excludedHeadSamples = ((Integer)trimdataheadspinner.getValue()).intValue();
                        updatePlot();
                        }
                    });
                trimdataheadspinner.addMouseWheelListener( new MouseWheelListener()
                    {
                    public void mouseWheelMoved( MouseWheelEvent e )
                        {
                        int newValue =  new Integer(((Integer)trimdataheadspinner.getValue()).intValue() - e.getWheelRotation());
                        if (newValue >= 0) trimdataheadspinner.setValue(newValue);
                        }
                    });
                trimdatapanel.add (trimdataheadspinner);

                //TRIM DATA FROM TAIL STEPSIZE POPUP

                trimdatatailsteppopup = new JPopupMenu();
                JPanel trimdatatailsteppanel = new JPanel();
                trimdatatailstepmodel = new SpinnerListModel(integerspinnerscalelist);
                trimdatatailstepspinner = new JSpinner(trimdatatailstepmodel);
                trimdatatailstepspinner.setValue(intConvertToText(excludedTailSamplesStep));
                trimdatatailstepspinner.addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        excludedTailSamplesStep = Integer.parseInt(((String)trimdatatailstepspinner.getValue()).trim());
                        trimdatatailspinner.setModel(new SpinnerNumberModel(new Integer(excludedTailSamples), new Integer(0), null, new Integer(excludedTailSamplesStep)));
                        Prefs.set("plot.excludedTailSamplesStep",excludedTailSamplesStep);
                        }
                    });

                JLabel trimdatatailsteplabel = new JLabel ("Stepsize:");
                trimdatatailsteppanel.add(trimdatatailsteplabel);
                trimdatatailsteppanel.add(trimdatatailstepspinner);
                trimdatatailsteppopup.add(trimdatatailsteppanel);
                trimdatatailsteppopup.setLightWeightPopupEnabled(false);
                trimdatatailsteppopup.addPopupMenuListener (new PopupMenuListener()
                    {
                    public void popupMenuCanceled(PopupMenuEvent popupMenuEvent) {
                            trimdatatailsteppopup.setVisible(false);
                            }

                    public void popupMenuWillBecomeInvisible(PopupMenuEvent popupMenuEvent) {
                            trimdatatailsteppopup.setVisible(true);
                            }

                    public void popupMenuWillBecomeVisible(PopupMenuEvent popupMenuEvent) {
                            trimdatatailsteppopup.setVisible(false);
                            }
                    });

                JLabel trimdatataillabel = new JLabel ("Tail");
                trimdatataillabel.setHorizontalAlignment(JLabel.RIGHT);
                trimdatataillabel.setPreferredSize(new Dimension(50, 25));
                if (showToolTips) trimdatataillabel.setToolTipText("Right click to set spinner stepsize");
                trimdatataillabel.setComponentPopupMenu(trimdatatailsteppopup);
                trimdatapanel.add (trimdatataillabel);

                trimdatatailmodel = new SpinnerNumberModel(new Integer(excludedTailSamples), new Integer(0), null, new Integer(excludedTailSamplesStep));
                trimdatatailspinner = new JSpinner(trimdatatailmodel);
                trimdatatailspinner.setComponentPopupMenu(trimdatatailsteppopup);
                if (showToolTips) trimdatatailspinner.setToolTipText("Right click to set spinner stepsize");
                trimdatatailspinner.addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        excludedTailSamples = ((Integer)trimdatatailspinner.getValue()).intValue();
                        updatePlot();
                        }
                    });
                trimdatatailspinner.addMouseWheelListener( new MouseWheelListener()
                    {
                    public void mouseWheelMoved( MouseWheelEvent e )
                        {
                        int newValue =  new Integer(((Integer)trimdatatailspinner.getValue()).intValue() - e.getWheelRotation());
                        if (newValue >= 0) trimdatatailspinner.setValue(newValue);
                        }
                    });
                trimdatapanel.add (trimdatatailspinner);
                SpringUtil.makeCompactGrid (trimdatapanel, 2, 2, 0,0,0,0);
                mainpanelgroupd.add (trimdatapanel);

                SpringUtil.makeCompactGrid (mainpanelgroupd, 1, 3, 0,0,0,0);
                mainpanel.add(mainpanelgroupd);


    // MAIN PANEL SCALING GROUP START (GROUP E)

            //X-SCALING SUBPANEL

                JPanel xscalingpanel = new JPanel (new SpringLayout());
                xscalingpanel.setBorder(BorderFactory.createTitledBorder("X-axis"));
                JRadioButton autoxButton = new JRadioButton("Auto X-range");
                autoxButton.setHorizontalAlignment(JLabel.CENTER);
                JRadioButton firstxButton = new JRadioButton("First X-value as min");
                firstxButton.setHorizontalAlignment(JLabel.CENTER);
                JRadioButton customxButton = new JRadioButton("Custom X-range");
                customxButton.setHorizontalAlignment(JLabel.CENTER);
                if (autoScaleX)
                        {
                        autoxButton.setSelected(true);
                        firstxButton.setSelected(false);
                        customxButton.setSelected(false);
                        }
                else if (useFirstX)
                        {
                        autoxButton.setSelected(false);
                        firstxButton.setSelected(true);
                        customxButton.setSelected(false);
                        }
                else
                        {
                        autoxButton.setSelected(false);
                        firstxButton.setSelected(false);
                        customxButton.setSelected(true);
                        }

                xscalingpanel.add (autoxButton);
                xscalingpanel.add (firstxButton);
                xscalingpanel.add (customxButton);

                ButtonGroup xaxisradiogroup = new ButtonGroup();
                        xaxisradiogroup.add(autoxButton);
                        xaxisradiogroup.add(firstxButton);
                        xaxisradiogroup.add(customxButton);
                autoxButton.addActionListener(new ActionListener(){
                    public void actionPerformed(ActionEvent ae) {
                        autoScaleX = true;
                        useFirstX = false;
                        updatePlot(); }});
                firstxButton.addActionListener(new ActionListener(){
                    public void actionPerformed(ActionEvent ae) {
                        autoScaleX = false;
                        useFirstX = true;
                        updatePlot(); }});
                customxButton.addActionListener(new ActionListener(){
                    public void actionPerformed(ActionEvent ae) {
                        autoScaleX = false;
                        useFirstX = false;
                        updatePlot(); }});

                JPanel grabautoxpanel = new JPanel (new SpringLayout());
                grabautoxbutton = new JButton("Grab");
                Dimension d = new Dimension();
                d.setSize(70,15);
                grabautoxbutton.setMaximumSize(d);
                if (showToolTips) grabautoxbutton.setToolTipText("Sends current Auto X-range values to Custom X-range");

                grabautoxbutton.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        xMin = xautoscalemin;
                        xminspinner.setValue(xMin);
                        xMax = xautoscalemax;
                        xmaxspinner.setValue(xMax);
                    }
                });
                grabautoxpanel.add (grabautoxbutton);
                SpringUtil.makeCompactGrid (grabautoxpanel,1,1, 20,0,0,0);
                xscalingpanel.add (grabautoxpanel);

                //XWIDTH STEPSIZE POPUP

                xwidthsteppopup = new JPopupMenu();
                JPanel xwidthsteppanel = new JPanel();
                xwidthstepmodel = new SpinnerListModel(spinnerscalelist);
                xwidthstepspinner = new JSpinner(xwidthstepmodel);
                xwidthstepspinner.setValue(convertToText(xWidthStep));
                xwidthstepspinner.addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        xWidthStep = Double.parseDouble((String)xwidthstepspinner.getValue());
                        xwidthspinner.setModel(new SpinnerNumberModel(new Double(xWidth), new Double(0), null, new Double(xWidthStep)));
                        Prefs.set("plot.xWidthStep",xWidthStep);
                        }
                    });

                JLabel xwidthsteplabel = new JLabel ("Stepsize:");
                xwidthsteppanel.add(xwidthsteplabel);
                xwidthsteppanel.add(xwidthstepspinner);
                xwidthsteppopup.add(xwidthsteppanel);
                xwidthsteppopup.setLightWeightPopupEnabled(false);
                xwidthsteppopup.setLocation(500, 500);
                xwidthsteppopup.addPopupMenuListener (new PopupMenuListener()
                    {
                    public void popupMenuCanceled(PopupMenuEvent popupMenuEvent) {
                            xwidthsteppopup.setVisible(false);
                            }

                    public void popupMenuWillBecomeInvisible(PopupMenuEvent popupMenuEvent) {
                            xwidthsteppopup.setVisible(true);
                            }

                    public void popupMenuWillBecomeVisible(PopupMenuEvent popupMenuEvent) {
                            xwidthsteppopup.setVisible(false);
                            }
                    });

                JPanel xwidthpanel = new JPanel (new SpringLayout());
                JLabel xwidthlabel = new JLabel ("X-width");
                xwidthlabel.setHorizontalAlignment(JLabel.RIGHT);
                xwidthlabel.setPreferredSize(new Dimension(50, 25));
                xwidthlabel.setComponentPopupMenu(xwidthsteppopup);
                if (showToolTips) xwidthlabel.setToolTipText("Right click to set spinner stepsize");
                xwidthpanel.add (xwidthlabel);
                xwidthspinnermodel = new SpinnerNumberModel(new Double(xWidth), new Double(0.000001), null, new Double(xWidthStep));
                xwidthspinner = new JSpinner(xwidthspinnermodel);
                if (showToolTips) xwidthspinner.setToolTipText("Right click to set spinner stepsize");
                xwidthspinner.addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        xWidth = ((Double)xwidthspinner.getValue()).doubleValue();
                        updatePlot();
                        }
                    });
                xwidthspinner.addMouseWheelListener( new MouseWheelListener()
                    {
                    public void mouseWheelMoved( MouseWheelEvent e )
                        {
                        double newvalue = ((Double)xwidthspinner.getValue()).doubleValue() - e.getWheelRotation()*xWidthStep ;
                        if (newvalue > 0)
                            xwidthspinner.setValue(newvalue);
                        else
                            xwidthspinner.setValue(0.000001);
                        }
                    });
                xwidthpanel.add (xwidthspinner);
                xwidthspinner.setComponentPopupMenu(xwidthsteppopup);
                SpringUtil.makeCompactGrid (xwidthpanel, 1, 2, 2,2,0,0);
                xscalingpanel.add (xwidthpanel);

                //XMAX STEPSIZE POPUP

                xmaxsteppopup = new JPopupMenu();
                JPanel xmaxsteppanel = new JPanel();
                xmaxstepmodel = new SpinnerListModel(spinnerscalelist);
                xmaxstepspinner = new JSpinner(xmaxstepmodel);
                xmaxstepspinner.setValue(convertToText(xMaxStep));
                xmaxstepspinner.addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        xMaxStep = Double.parseDouble((String)xmaxstepspinner.getValue());
                        xmaxspinner.setModel(new SpinnerNumberModel(new Double(xMax), null, null, new Double(xMaxStep)));
                        Prefs.set("plot.xMaxStep",xMaxStep);
                        }
                    });

                JLabel xmaxsteplabel = new JLabel ("Stepsize:");
                xmaxsteppanel.add(xmaxsteplabel);
                xmaxsteppanel.add(xmaxstepspinner);
                xmaxsteppopup.add(xmaxsteppanel);
                xmaxsteppopup.setLightWeightPopupEnabled(false);
                xmaxsteppopup.addPopupMenuListener (new PopupMenuListener()
                    {
                    public void popupMenuCanceled(PopupMenuEvent popupMenuEvent) {
                            xmaxsteppopup.setVisible(false);
                            }

                    public void popupMenuWillBecomeInvisible(PopupMenuEvent popupMenuEvent) {
                            xmaxsteppopup.setVisible(true);
                            }

                    public void popupMenuWillBecomeVisible(PopupMenuEvent popupMenuEvent) {
                            xmaxsteppopup.setVisible(false);
                            }
                    });

                JPanel customxmaxpanel = new JPanel(new SpringLayout());
                JLabel xmaxlabel = new JLabel ("X-max");
                xmaxlabel.setHorizontalAlignment(JLabel.RIGHT);
                xmaxlabel.setPreferredSize(new Dimension(50, 25));
                if (showToolTips) xmaxlabel.setToolTipText("Right click to set spinner stepsize");
                xmaxlabel.setComponentPopupMenu(xmaxsteppopup);
                customxmaxpanel.add (xmaxlabel);

                xmaxspinnermodel = new SpinnerNumberModel(new Double(xMax),null, null, new Double(xMaxStep));
                xmaxspinner = new JSpinner(xmaxspinnermodel);
                xmaxspinner.setComponentPopupMenu(xmaxsteppopup);
                if (showToolTips) xmaxspinner.setToolTipText("Right click to set spinner stepsize");
                xmaxspinner.addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        xMax = ((Double)xmaxspinner.getValue()).doubleValue();
                        updatePlot();
                        }
                    });
                xmaxspinner.addMouseWheelListener( new MouseWheelListener()
                    {
                    public void mouseWheelMoved( MouseWheelEvent e )
                        {
                        xmaxspinner.setValue(((Double)xmaxspinner.getValue()).doubleValue()
                                - e.getWheelRotation()*xMaxStep );
                        }
                    });
                customxmaxpanel.add (xmaxspinner);
                SpringUtil.makeCompactGrid (customxmaxpanel, 1, 2, 2,2,0,0);
                xscalingpanel.add (customxmaxpanel);

                JPanel xmultiplierpanel = new JPanel(new SpringLayout());
                xmultiplierpanel.setBorder(BorderFactory.createLineBorder(Color.lightGray));
                JLabel xmultiplierlabel = new JLabel ("X x 1E");
                xmultiplierlabel.setHorizontalAlignment(JLabel.RIGHT);
                xmultiplierpanel.setMaximumSize(new Dimension(250, 30));
                xmultiplierpanel.add (xmultiplierlabel);
                if (showToolTips) xmultiplierlabel.setToolTipText("Sets the X-axis multiplication factor");
                xmultipliermodel = new SpinnerNumberModel(new Integer(xExponent),null, null, new Integer(1));
                xmultiplierspinner = new JSpinner(xmultipliermodel);
                xmultiplierspinner.setMaximumSize(new Dimension (35,25));
                if (showToolTips) xmultiplierspinner.setToolTipText("Sets the X-axis multiplication factor");
                xmultiplierspinner.addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        xExponent = ((Integer)xmultiplierspinner.getValue()).intValue();
                        updatePlot();
                        }
                    });
                xmultiplierspinner.addMouseWheelListener( new MouseWheelListener()
                    {
                    public void mouseWheelMoved( MouseWheelEvent e )
                        {
                        xmultiplierspinner.setValue( new Integer(((Integer)xmultiplierspinner.getValue()).intValue()
                                - e.getWheelRotation()) );
                        }
                    });
                xmultiplierpanel.add (xmultiplierspinner);
                SpringUtil.makeCompactGrid (xmultiplierpanel, 1,2, 2,2,2,0);
                xscalingpanel.add (xmultiplierpanel);

                //XMIN STEPSIZE POPUP

                xminsteppopup = new JPopupMenu();
                JPanel xminsteppanel = new JPanel();
                xminstepmodel = new SpinnerListModel(spinnerscalelist);
                xminstepspinner = new JSpinner(xminstepmodel);
                xminstepspinner.setValue(convertToText(xMinStep));
                xminstepspinner.addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        xMinStep = Double.parseDouble((String)xminstepspinner.getValue());
                        xminspinner.setModel(new SpinnerNumberModel(new Double(xMin), null, null, new Double(xMinStep)));
                        Prefs.set("plot.xMinStep",xMinStep);
                        }
                    });

                JLabel xminsteplabel = new JLabel ("Stepsize:");
                xminsteppanel.add(xminsteplabel);
                xminsteppanel.add(xminstepspinner);
                xminsteppopup.add(xminsteppanel);
                xminsteppopup.setLightWeightPopupEnabled(false);
                xminsteppopup.addPopupMenuListener (new PopupMenuListener()
                    {
                    public void popupMenuCanceled(PopupMenuEvent popupMenuEvent) {
                            xminsteppopup.setVisible(false);
                            }

                    public void popupMenuWillBecomeInvisible(PopupMenuEvent popupMenuEvent) {
                            xminsteppopup.setVisible(true);
                            }

                    public void popupMenuWillBecomeVisible(PopupMenuEvent popupMenuEvent) {
                            xminsteppopup.setVisible(false);
                            }
                    });


                JLabel dummylabel7 = new JLabel ("");
                dummylabel7.setHorizontalAlignment(JLabel.CENTER);
                dummylabel7.setPreferredSize(new Dimension(150, 25));
                dummylabel7.setComponentPopupMenu(xwidthsteppopup);
                xscalingpanel.add (dummylabel7);



                JPanel customxminpanel = new JPanel(new SpringLayout());
                JLabel xminlabel = new JLabel ("X-min");
                xminlabel.setHorizontalAlignment(JLabel.RIGHT);
                xminlabel.setPreferredSize(new Dimension(50, 25));
                if (showToolTips) xminlabel.setToolTipText("Right click to set spinner stepsize");
                xminlabel.setComponentPopupMenu(xminsteppopup);
                customxminpanel.add (xminlabel);

                xminspinnermodel = new SpinnerNumberModel(new Double(xMin),null, null, new Double(xMinStep));
                xminspinner = new JSpinner(xminspinnermodel);
                xminspinner.setComponentPopupMenu(xminsteppopup);
                if (showToolTips) xminspinner.setToolTipText("Right click to set spinner stepsize");
                xminspinner.addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        xMin = ((Double)xminspinner.getValue()).doubleValue();
                        updatePlot();
                        }
                    });
                xminspinner.addMouseWheelListener( new MouseWheelListener()
                    {
                    public void mouseWheelMoved( MouseWheelEvent e )
                        {
                        xminspinner.setValue(((Double)xminspinner.getValue()).doubleValue()
                                - e.getWheelRotation()*xMinStep );
                        }
                    });
                customxminpanel.add (xminspinner);
                SpringUtil.makeCompactGrid (customxminpanel, 1, 2, 2,2,0,0);
                xscalingpanel.add (customxminpanel);

                SpringUtil.makeCompactGrid (xscalingpanel, 3, 3, 0,0,0,0);

                mainpanelgroupe.add (xscalingpanel);

            //Y-SCALING SUBPANEL

                JPanel yscalingpanel = new JPanel (new SpringLayout());
                yscalingpanel.setBorder(BorderFactory.createTitledBorder("Y-axis"));
                JRadioButton autoyButton = new JRadioButton("Auto Y-range");
                autoyButton.setHorizontalAlignment(JLabel.CENTER);
                JRadioButton customyButton = new JRadioButton("Custom Y-range");
                customyButton.setHorizontalAlignment(JLabel.CENTER);
                if (autoScaleY)
                        {
                        autoyButton.setSelected(true);
                        customyButton.setSelected(false);
                        }
                else
                        {
                        autoyButton.setSelected(false);
                        customyButton.setSelected(true);
                        }

                yscalingpanel.add (autoyButton);
                yscalingpanel.add (customyButton);

                ButtonGroup yaxisradiogroup = new ButtonGroup();
                        yaxisradiogroup.add(autoyButton);
                        yaxisradiogroup.add(customyButton);
                autoyButton.addActionListener(new ActionListener(){
                    public void actionPerformed(ActionEvent ae) {
                        autoScaleY = true;
                        updatePlot(); }});
                customyButton.addActionListener(new ActionListener(){
                    public void actionPerformed(ActionEvent ae) {
                        autoScaleY = false;
                        updatePlot(); }});

                JPanel grabautoypanel = new JPanel (new SpringLayout());
                grabautoybutton = new JButton("Grab");
                grabautoybutton.setMaximumSize(d);
                if (showToolTips) grabautoybutton.setToolTipText("Sends current Auto Y-range values to Custom Y-range");
                grabautoybutton.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        yMin = yautoscalemin;
                        yminspinner.setValue(yMin);
                        yMax = yautoscalemax;
                        ymaxspinner.setValue(yMax);
                    }
                    });
                grabautoypanel.add (grabautoybutton);
                SpringUtil.makeCompactGrid (grabautoypanel,1,1, 20,0,0,0);
                yscalingpanel.add (grabautoypanel);

                //YMAX STEPSIZE POPUP

                ymaxsteppopup = new JPopupMenu();
                JPanel ymaxsteppanel = new JPanel();
                ymaxstepmodel = new SpinnerListModel(spinnerscalelist);
                ymaxstepspinner = new JSpinner(ymaxstepmodel);
                ymaxstepspinner.setValue(convertToText(yMaxStep));
                ymaxstepspinner.addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        yMaxStep = Double.parseDouble((String)ymaxstepspinner.getValue());
                        ymaxspinner.setModel(new SpinnerNumberModel(new Double(yMax), null, null, new Double(yMaxStep)));
                        Prefs.set("plot.yMaxStep",yMaxStep);
                        }
                    });

                JLabel ymaxsteplabel = new JLabel ("Stepsize:");
                ymaxsteppanel.add(ymaxsteplabel);
                ymaxsteppanel.add(ymaxstepspinner);
                ymaxsteppopup.add(ymaxsteppanel);
                ymaxsteppopup.setLightWeightPopupEnabled(false);
                ymaxsteppopup.addPopupMenuListener (new PopupMenuListener()
                    {
                    public void popupMenuCanceled(PopupMenuEvent popupMenuEvent) {
                            ymaxsteppopup.setVisible(false);
                            }

                    public void popupMenuWillBecomeInvisible(PopupMenuEvent popupMenuEvent) {
                            ymaxsteppopup.setVisible(true);
                            }

                    public void popupMenuWillBecomeVisible(PopupMenuEvent popupMenuEvent) {
                            ymaxsteppopup.setVisible(false);
                            }
                    });

                JPanel customymaxpanel = new JPanel(new SpringLayout());
                JLabel ymaxlabel = new JLabel ("Y-max");
                ymaxlabel.setHorizontalAlignment(JLabel.RIGHT);
                ymaxlabel.setPreferredSize(new Dimension(50, 25));
                if (showToolTips) ymaxlabel.setToolTipText("Right click to set spinner stepsize");
                ymaxlabel.setComponentPopupMenu(ymaxsteppopup);
                customymaxpanel.add (ymaxlabel);

                ymaxspinnermodel = new SpinnerNumberModel(new Double(yMax), null, null, new Double(yMaxStep));
                ymaxspinner = new JSpinner(ymaxspinnermodel);
                ymaxspinner.setComponentPopupMenu(ymaxsteppopup);
                if (showToolTips) ymaxspinner.setToolTipText("Right click to set spinner stepsize");
                ymaxspinner.addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        yMax = ((Double)ymaxspinner.getValue()).doubleValue();
                        updatePlot();
                        }
                    });
                ymaxspinner.addMouseWheelListener( new MouseWheelListener()
                    {
                    public void mouseWheelMoved( MouseWheelEvent e )
                        {
                        ymaxspinner.setValue(((Double)ymaxspinner.getValue()).doubleValue()
                                - e.getWheelRotation()*yMaxStep );
                        }
                    });
                customymaxpanel.add (ymaxspinner);
                SpringUtil.makeCompactGrid (customymaxpanel, 1, 2, 2,2,0,0);
                yscalingpanel.add (customymaxpanel);

                JPanel ymultiplierpanel = new JPanel(new SpringLayout());
                ymultiplierpanel.setBorder(BorderFactory.createLineBorder(Color.lightGray));
                JLabel ymultiplierlabel = new JLabel ("Y x 1E");
                ymultiplierlabel.setHorizontalAlignment(JLabel.RIGHT);
                ymultiplierpanel.setMaximumSize(new Dimension(250, 30));
                ymultiplierpanel.add (ymultiplierlabel);
                if (showToolTips) ymultiplierlabel.setToolTipText("Sets the Y-axis multiplication factor");
                ymultipliermodel = new SpinnerNumberModel(new Integer(yExponent),null, null, new Integer(1));
                ymultiplierspinner = new JSpinner(ymultipliermodel);
                ymultiplierspinner.setMaximumSize(new Dimension (35,25));
                if (showToolTips) ymultiplierspinner.setToolTipText("Sets the Y-axis multiplication factor");
                ymultiplierspinner.addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        yExponent = ((Integer)ymultiplierspinner.getValue()).intValue();
                        updatePlot();
                        }
                    });
                ymultiplierspinner.addMouseWheelListener( new MouseWheelListener()
                    {
                    public void mouseWheelMoved( MouseWheelEvent e )
                        {
                        ymultiplierspinner.setValue( new Integer(((Integer)ymultiplierspinner.getValue()).intValue()
                                - e.getWheelRotation()) );
                        }
                    });
                ymultiplierpanel.add (ymultiplierspinner);
                SpringUtil.makeCompactGrid (ymultiplierpanel, 1,2, 2,2,2,0);
                yscalingpanel.add (ymultiplierpanel);

                //YMIN STEPSIZE POPUP

                yminsteppopup = new JPopupMenu();
                JPanel yminsteppanel = new JPanel();
                yminstepmodel = new SpinnerListModel(spinnerscalelist);
                yminstepspinner = new JSpinner(yminstepmodel);
                yminstepspinner.setValue(convertToText(yMinStep));
                yminstepspinner.addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        yMinStep = Double.parseDouble((String)yminstepspinner.getValue());
                        yminspinner.setModel(new SpinnerNumberModel(new Double(yMin), null, null, new Double(yMinStep)));
                        Prefs.set("plot.yMinStep",yMinStep);
                        }
                    });

                JLabel yminsteplabel = new JLabel ("Stepsize:");
                yminsteppanel.add(yminsteplabel);
                yminsteppanel.add(yminstepspinner);
                yminsteppopup.add(yminsteppanel);
                yminsteppopup.setLightWeightPopupEnabled(false);
                yminsteppopup.addPopupMenuListener (new PopupMenuListener()
                    {
                    public void popupMenuCanceled(PopupMenuEvent popupMenuEvent) {
                            yminsteppopup.setVisible(false);
                            }

                    public void popupMenuWillBecomeInvisible(PopupMenuEvent popupMenuEvent) {
                            yminsteppopup.setVisible(true);
                            }

                    public void popupMenuWillBecomeVisible(PopupMenuEvent popupMenuEvent) {
                            yminsteppopup.setVisible(false);
                            }
                    });


                JPanel customyminpanel = new JPanel(new SpringLayout());
                JLabel yminlabel = new JLabel ("Y-min");
                yminlabel.setHorizontalAlignment(JLabel.RIGHT);
                yminlabel.setComponentPopupMenu(yminsteppopup);
                yminlabel.setPreferredSize(new Dimension(50, 25));
                if (showToolTips) yminlabel.setToolTipText("Right click to set spinner stepsize");
                customyminpanel.add (yminlabel);

                yminspinnermodel = new SpinnerNumberModel(new Double(yMin),  null, null, new Double(yMinStep));
                yminspinner = new JSpinner(yminspinnermodel);
                yminspinner.setComponentPopupMenu(yminsteppopup);
                if (showToolTips) yminspinner.setToolTipText("Right click to set spinner stepsize");
                yminspinner.addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        yMin = ((Double)yminspinner.getValue()).doubleValue();
                        updatePlot();
                        }
                    });
                yminspinner.addMouseWheelListener( new MouseWheelListener()
                    {
                    public void mouseWheelMoved( MouseWheelEvent e )
                        {
                        yminspinner.setValue(((Double)yminspinner.getValue()).doubleValue()
                                - e.getWheelRotation()*yMinStep );
                        }
                    });
                customyminpanel.add (yminspinner);
                SpringUtil.makeCompactGrid (customyminpanel, 1, 2, 2,2,0,0);
                yscalingpanel.add (customyminpanel);

                SpringUtil.makeCompactGrid (yscalingpanel, 3, 2, 0,0,0,0);
                mainpanelgroupe.add (yscalingpanel);


            //PLOTSIZE SUBPANEL

                JPanel plotsizepanel = new JPanel (new SpringLayout());
                plotsizepanel.setBorder(BorderFactory.createTitledBorder("Plot Size"));


                JLabel dummylabel10 = new JLabel ("");
                dummylabel10.setHorizontalAlignment(JLabel.CENTER);
                dummylabel10.setPreferredSize(new Dimension(50, 25));
                plotsizepanel.add (dummylabel10);

                JLabel dummylabel11 = new JLabel ("");
                dummylabel11.setHorizontalAlignment(JLabel.CENTER);
                dummylabel11.setPreferredSize(new Dimension(50, 25));
                plotsizepanel.add (dummylabel11);

                //PLOT HEIGHT STEPSIZE POPUP

                plotheightsteppopup = new JPopupMenu();
                JPanel plotheightsteppanel = new JPanel();
                plotheightstepmodel = new SpinnerListModel(integerspinnerscalelist);
                plotheightstepspinner = new JSpinner(plotheightstepmodel);
                plotheightstepspinner.setValue(intConvertToText(plotSizeYStep));
                plotheightstepspinner.addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        plotSizeYStep = Integer.parseInt(((String)plotheightstepspinner.getValue()).trim());
                        plotheightspinner.setModel(new SpinnerNumberModel(new Integer(plotSizeY), new Integer(1), null, new Integer(plotSizeYStep)));
                        Prefs.set("plot.plotSizeYStep",plotSizeYStep);
                        }
                    });

                JLabel plotheightsteplabel = new JLabel ("Stepsize:");
                plotheightsteppanel.add(plotheightsteplabel);
                plotheightsteppanel.add(plotheightstepspinner);
                plotheightsteppopup.add(plotheightsteppanel);
                plotheightsteppopup.setLightWeightPopupEnabled(false);
                plotheightsteppopup.addPopupMenuListener (new PopupMenuListener()
                    {
                    public void popupMenuCanceled(PopupMenuEvent popupMenuEvent) {
                            plotheightsteppopup.setVisible(false);
                            }

                    public void popupMenuWillBecomeInvisible(PopupMenuEvent popupMenuEvent) {
                            plotheightsteppopup.setVisible(true);
                            }

                    public void popupMenuWillBecomeVisible(PopupMenuEvent popupMenuEvent) {
                            plotheightsteppopup.setVisible(false);
                            }
                    });

                JLabel plotheightlabel = new JLabel ("Height");
                plotheightlabel.setHorizontalAlignment(JLabel.RIGHT);
                plotheightlabel.setPreferredSize(new Dimension(50, 25));
                if (showToolTips) plotheightlabel.setToolTipText("Right click to set spinner stepsize");
                plotheightlabel.setComponentPopupMenu(plotheightsteppopup);
                plotsizepanel.add (plotheightlabel);

                plotheightspinnermodel = new SpinnerNumberModel(new Integer(plotSizeY), new Integer(1), null, new Integer(plotSizeYStep));
                plotheightspinner = new JSpinner(plotheightspinnermodel);
                plotheightspinner.setComponentPopupMenu(plotheightsteppopup);
                if (showToolTips) plotheightspinner.setToolTipText("Right click to set spinner stepsize");
                plotheightspinner.addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        plotSizeY = ((Integer)plotheightspinner.getValue()).intValue();
                        updatePlot();
                        }
                    });
                plotheightspinner.addMouseWheelListener( new MouseWheelListener()
                    {
                    public void mouseWheelMoved( MouseWheelEvent e )
                        {
                        int newValue =  new Integer(((Integer)plotheightspinner.getValue()).intValue() - e.getWheelRotation()*plotSizeYStep);
                        if (newValue > 0) plotheightspinner.setValue(newValue);
                        }
                    });
                plotsizepanel.add (plotheightspinner);

                //PLOT WIDTH STEPSIZE POPUP

                plotwidthsteppopup = new JPopupMenu();
                JPanel plotwidthsteppanel = new JPanel();
                plotwidthstepmodel = new SpinnerListModel(integerspinnerscalelist);
                plotwidthstepspinner = new JSpinner(plotwidthstepmodel);
                plotwidthstepspinner.setValue(intConvertToText(plotSizeXStep));
                plotwidthstepspinner.addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        plotSizeXStep = Integer.parseInt(((String)plotwidthstepspinner.getValue()).trim());
                        plotwidthspinner.setModel(new SpinnerNumberModel(new Integer(plotSizeX), new Integer(1), null, new Integer(plotSizeXStep)));
                        Prefs.set("plot.plotSizeXStep",plotSizeXStep);
                        }
                    });
                JLabel plotwidthsteplabel = new JLabel ("Stepsize:");
                plotwidthsteppanel.add(plotwidthsteplabel);
                plotwidthsteppanel.add(plotwidthstepspinner);
                plotwidthsteppopup.add(plotwidthsteppanel);
                plotwidthsteppopup.setLightWeightPopupEnabled(false);
                plotwidthsteppopup.addPopupMenuListener (new PopupMenuListener()
                    {
                    public void popupMenuCanceled(PopupMenuEvent popupMenuEvent) {
                            plotwidthsteppopup.setVisible(false);
                            }

                    public void popupMenuWillBecomeInvisible(PopupMenuEvent popupMenuEvent) {
                            plotwidthsteppopup.setVisible(true);
                            }

                    public void popupMenuWillBecomeVisible(PopupMenuEvent popupMenuEvent) {
                            plotwidthsteppopup.setVisible(false);
                            }
                    });





                JLabel plotwidthlabel = new JLabel ("Width");
                plotwidthlabel.setHorizontalAlignment(JLabel.RIGHT);
                plotwidthlabel.setPreferredSize(new Dimension(50, 25));
                if (showToolTips) plotwidthlabel.setToolTipText("Right click to set spinner stepsize");
                plotwidthlabel.setComponentPopupMenu(plotwidthsteppopup);
                plotsizepanel.add (plotwidthlabel);

                plotwidthspinnermodel = new SpinnerNumberModel(new Integer(plotSizeX), new Integer(1), null, new Integer(plotSizeXStep));
                plotwidthspinner = new JSpinner(plotwidthspinnermodel);
                plotwidthspinner.setComponentPopupMenu(plotwidthsteppopup);
                if (showToolTips) plotwidthspinner.setToolTipText("Right click to set spinner stepsize");
                plotwidthspinner.addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        plotSizeX = ((Integer)plotwidthspinner.getValue()).intValue();
                        updatePlot();
                        }
                    });

                plotwidthspinner.addMouseWheelListener( new MouseWheelListener()
                    {
                    public void mouseWheelMoved( MouseWheelEvent e )
                        {
                        int newValue =  new Integer(((Integer)plotwidthspinner.getValue()).intValue() - e.getWheelRotation()*plotSizeXStep);
                        if (newValue > 0) plotwidthspinner.setValue(newValue);
                        }
                    });
                plotsizepanel.add (plotwidthspinner);
                SpringUtil.makeCompactGrid (plotsizepanel, 3, 2, 0,0,0,0);
                mainpanelgroupe.add (plotsizepanel);


        //FINALIZE GROUP E PANEL (SCALING PANEL)

                SpringUtil.makeCompactGrid (mainpanelgroupe, 1, 3, 0,0,0,0);
                mainpanel.add(mainpanelgroupe);
                
        // MAIN PANEL GROUP F START

                JPanel updateplotbuttonpanel = new JPanel (new SpringLayout());
                updateplotbutton = new JButton(" Update Plot ");
                updateplotbutton.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        updatePlot();                    }
                });

                updateplotbuttonpanel.add (updateplotbutton);
                SpringUtil.makeCompactGrid (updateplotbuttonpanel,1,1, 20,15,6,0);
                mainpanelgroupf.add (updateplotbuttonpanel);

                JPanel formatoptionspanel = new JPanel (new SpringLayout());
                TitledBorder formatoptiontitle = BorderFactory.createTitledBorder("Format Options");
                formatoptiontitle.setTitleJustification(TitledBorder.CENTER);
                formatoptionspanel.setBorder(formatoptiontitle);
                usextickscheckbox = new JCheckBox ("X tick marks", xTics);
                usextickscheckbox.addItemListener(new ItemListener(){
                    public void itemStateChanged(ItemEvent e) {
                        if (e.getStateChange() == ItemEvent.DESELECTED)
                                xTics = false;
                        else if (e.getStateChange() == ItemEvent.SELECTED)
                                xTics = true;
                        updatePlot(); }});
                usextickscheckbox.setHorizontalAlignment(JLabel.CENTER);
                formatoptionspanel.add (usextickscheckbox);

                useytickscheckbox = new JCheckBox ("Y tick marks", yTics);
                useytickscheckbox.addItemListener(new ItemListener(){
                    public void itemStateChanged(ItemEvent e) {
                        if (e.getStateChange() == ItemEvent.DESELECTED)
                                yTics = false;
                        else if (e.getStateChange() == ItemEvent.SELECTED)
                                yTics = true;
                        updatePlot(); }});
                useytickscheckbox.setHorizontalAlignment(JLabel.CENTER);
                formatoptionspanel.add (useytickscheckbox);

                usexgridcheckbox = new JCheckBox ("X grid lines", xGrid);
                usexgridcheckbox.addItemListener(new ItemListener(){
                    public void itemStateChanged(ItemEvent e) {
                        if (e.getStateChange() == ItemEvent.DESELECTED)
                                xGrid = false;
                        else if (e.getStateChange() == ItemEvent.SELECTED)
                                xGrid = true;
                        updatePlot(); }});
                usexgridcheckbox.setHorizontalAlignment(JLabel.CENTER);
                formatoptionspanel.add (usexgridcheckbox);

                useygridcheckbox = new JCheckBox ("Y grid lines", yGrid);
                useygridcheckbox.addItemListener(new ItemListener(){
                    public void itemStateChanged(ItemEvent e) {
                        if (e.getStateChange() == ItemEvent.DESELECTED)
                                yGrid = false;
                        else if (e.getStateChange() == ItemEvent.SELECTED)
                                yGrid = true;
                        updatePlot(); }});
                useygridcheckbox.setHorizontalAlignment(JLabel.CENTER);
                formatoptionspanel.add (useygridcheckbox);

                usexnumberscheckbox = new JCheckBox ("X numbers", xNumbers);
                usexnumberscheckbox.addItemListener(new ItemListener(){
                    public void itemStateChanged(ItemEvent e) {
                        if (e.getStateChange() == ItemEvent.DESELECTED)
                                xNumbers = false;
                        else if (e.getStateChange() == ItemEvent.SELECTED)
                                xNumbers = true;
                        updatePlot(); }});
                usexnumberscheckbox.setHorizontalAlignment(JLabel.CENTER);
                formatoptionspanel.add (usexnumberscheckbox);

                useynumberscheckbox = new JCheckBox ("Y numbers", yNumbers);
                useynumberscheckbox.addItemListener(new ItemListener(){
                    public void itemStateChanged(ItemEvent e) {
                        if (e.getStateChange() == ItemEvent.DESELECTED)
                                yNumbers = false;
                        else if (e.getStateChange() == ItemEvent.SELECTED)
                                yNumbers = true;
                        updatePlot(); }});
                useynumberscheckbox.setHorizontalAlignment(JLabel.CENTER);
                formatoptionspanel.add (useynumberscheckbox);
                SpringUtil.makeCompactGrid (formatoptionspanel, 1,6, 2,2,6,2);
                mainpanelgroupf.add (formatoptionspanel);

                JPanel closebuttonpanel = new JPanel (new SpringLayout());
                closebutton = new JButton("        Exit        ");
                closebutton.setForeground(Color.RED);
                closebutton.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                            saveAndClose(); }});
                closebuttonpanel.add (closebutton);
                SpringUtil.makeCompactGrid (closebuttonpanel, 1, 1, 6,15,0,0);
                mainpanelgroupf.add (closebuttonpanel);

                SpringUtil.makeCompactGrid (mainpanelgroupf, 1, 3, 2,0,6,0);
                mainpanel.add(mainpanelgroupf);

//FINALIZE MAIN PANEL

                SpringUtil.makeCompactGrid (mainpanel, 5, 1, 6,6,6,6);



                mainFrame.setJMenuBar(mainmenubar);
                mainFrame.add (mainscrollpane);
                mainFrame.pack();
                mainFrame.setResizable (true);
                if (rememberWindowLocations)
                        mainFrame.setLocation(mainFrameLocationX, mainFrameLocationY);
                if (!plotAutoMode)
                        mainFrame.setVisible (true);
                
                showMoreCurvesJPanel();

                }





        void showMoreCurvesJPanel()
                {
                subFrame.setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
                subFrame.addWindowListener(new WindowAdapter(){
                    @Override
                    public void windowClosing(WindowEvent e){
                            subFrameLocationX = subFrame.getLocation().x;
                            subFrameLocationY = subFrame.getLocation().y;
                            setSubpanelVisible = false;
                            Prefs.set("plot.subFrameLocationX",subFrameLocationX);
                            Prefs.set("plot.subFrameLocationY",subFrameLocationY);
                            }});

                JPanel mainsubpanelgroupa = new JPanel (new SpringLayout());
                mainsubpanelgroupa.setBorder(BorderFactory.createTitledBorder(""));

                JPanel mainsubpanelgroupb = new JPanel (new SpringLayout());
                mainsubpanelgroupb.setBorder(BorderFactory.createTitledBorder(""));

       // SUBPANEL START

             
                
                if (useWideDataPanel)
                        {
//                        constructTopGroupTopLabels(mainsubpanelgroupa);
//                        constructOtherGroupTopLabels(mainsubpanelgroupa);
                        constructTopGroupBottomLabels(mainsubpanelgroupa);
                        constructOtherGroupBottomLabels(mainsubpanelgroupa);

                        }
                else
                        {
//                        constructTopGroupTopLabels(mainsubpanelgroupa);
                        constructTopGroupBottomLabels(mainsubpanelgroupa);
//                        constructOtherGroupTopLabels(mainsubpanelgroupb);
                        constructOtherGroupBottomLabels(mainsubpanelgroupb);
                        }
                for (cur=0; cur<maxCurves; cur++)
                        {
                        constructTopGroup(mainsubpanelgroupa, cur);
                        if (useWideDataPanel)
                                constructOtherGroup(mainsubpanelgroupa, cur);
                        else
                                constructOtherGroup(mainsubpanelgroupb, cur);
                        }

                if (useWideDataPanel)
                        {
                        SpringUtil.makeCompactGrid (mainsubpanelgroupa, maxCurves+1, 21, 5,5,5,5);
                        mainsubpanel.add(mainsubpanelgroupa);
                        SpringUtil.makeCompactGrid (mainsubpanel, 1, 1, 5,5,5,5);
                        }
                else
                        {
                        SpringUtil.makeCompactGrid (mainsubpanelgroupa, maxCurves+1, 14, 5,5,5,5);
                        mainsubpanel.add(mainsubpanelgroupa);
                        SpringUtil.makeCompactGrid (mainsubpanelgroupb, maxCurves+1, 7, 5,5,5,5);
                        mainsubpanel.add(mainsubpanelgroupb);
                        SpringUtil.makeCompactGrid (mainsubpanel, 2, 1, 5,5,5,5);
                        }


                
                
                subFrame.add (subscrollpane);
                subFrame.pack();
                subFrame.setResizable (true);
                if (rememberWindowLocations)
                        subFrame.setLocation(subFrameLocationX, subFrameLocationY);

                if (!plotAutoMode && (openDataSetWindow || setSubpanelVisible))
                        {
                        subFrame.setVisible (true);
                        }
                
                }

        void constructTopGroupTopLabels(JPanel mainsubpanelgroup)
                {
                JLabel dummylabel120 = new JLabel ("Data");
                dummylabel120.setHorizontalAlignment(JLabel.CENTER);
                dummylabel120.setPreferredSize(new Dimension(10,15));
                mainsubpanelgroup.add (dummylabel120);
                JLabel dummylabel121 = new JLabel ("Plot");
                if (showToolTips) dummylabel121.setToolTipText("Show the dataset on the plot");
                dummylabel121.setHorizontalAlignment(JLabel.CENTER);
                dummylabel121.setPreferredSize(new Dimension(10,15));
                mainsubpanelgroup.add (dummylabel121);
                JLabel autoscaletopincludelabel = new JLabel ("Auto");
                if (showToolTips) autoscaletopincludelabel.setToolTipText("Include this dataset in the Y-axis autoscale range");
                autoscaletopincludelabel.setHorizontalAlignment(JLabel.CENTER);
                autoscaletopincludelabel.setPreferredSize(new Dimension(10,15));
                mainsubpanelgroup.add (autoscaletopincludelabel);
                JLabel mmagtoplabel = new JLabel ("");
                if (showToolTips) mmagtoplabel.setToolTipText("Plot data in relative magnitude");
                mmagtoplabel.setHorizontalAlignment(JLabel.CENTER);
                mmagtoplabel.setPreferredSize(new Dimension(10,15));
                mainsubpanelgroup.add (mmagtoplabel);
                JLabel ycolumntoplabelsub = new JLabel ("");
                ycolumntoplabelsub.setHorizontalAlignment(JLabel.CENTER);
                ycolumntoplabelsub.setPreferredSize(new Dimension(10,15));
                mainsubpanelgroup.add (ycolumntoplabelsub);
                JLabel useerrortoplabel = new JLabel ("Use");
                if (showToolTips) useerrortoplabel.setToolTipText("Plot Y-error bars using Y-error Column values");
                useerrortoplabel.setHorizontalAlignment(JLabel.CENTER);
                useerrortoplabel.setPreferredSize(new Dimension(10,15));
                mainsubpanelgroup.add (useerrortoplabel);
                JLabel errortoplabel = new JLabel ("");
                errortoplabel.setHorizontalAlignment(JLabel.CENTER);
                if (showToolTips) errortoplabel.setToolTipText("Error associated with Y-data");
                errortoplabel.setPreferredSize(new Dimension(10,15));
                mainsubpanelgroup.add (errortoplabel);
                JLabel operatortoplabelsub = new JLabel ("");
                operatortoplabelsub.setHorizontalAlignment(JLabel.CENTER);
                if (showToolTips) operatortoplabelsub.setToolTipText("Performs the selected operation on Y-data Column and Y-operator Column and propagates error if enabled");
                operatortoplabelsub.setPreferredSize(new Dimension(10,15));
                mainsubpanelgroup.add (operatortoplabelsub);
                JLabel opcolumntoplabelsub = new JLabel ("");
                opcolumntoplabelsub.setHorizontalAlignment(JLabel.CENTER);
                opcolumntoplabelsub.setPreferredSize(new Dimension(10,15));
                mainsubpanelgroup.add (opcolumntoplabelsub);
                JLabel useoperrortoplabel = new JLabel ("Use");
                if (showToolTips) useoperrortoplabel.setToolTipText("Use Operator Error as error associated with Y-operator Column");
                useoperrortoplabel.setPreferredSize(new Dimension(10,15));
                useoperrortoplabel.setHorizontalAlignment(JLabel.CENTER);
                mainsubpanelgroup.add (useoperrortoplabel);
                JLabel operrortoplabel = new JLabel ("");
                operrortoplabel.setHorizontalAlignment(JLabel.CENTER);
                operrortoplabel.setPreferredSize(new Dimension(10,15));
                mainsubpanelgroup.add (operrortoplabel);
                JLabel markertoplabelsub = new JLabel ("");
                markertoplabelsub.setHorizontalAlignment(JLabel.CENTER);
                markertoplabelsub.setPreferredSize(new Dimension(10,15));
                mainsubpanelgroup.add (markertoplabelsub);
                JLabel markercolortoplabelsub = new JLabel ("");
                markercolortoplabelsub.setHorizontalAlignment(JLabel.CENTER);
                markercolortoplabelsub.setPreferredSize(new Dimension(10,15));
                mainsubpanelgroup.add (markercolortoplabelsub);
                JLabel linestoplabel = new JLabel ("Use");
                if (showToolTips) linestoplabel.setToolTipText("Connect datapoints with lines");
                linestoplabel.setHorizontalAlignment(JLabel.CENTER);
                linestoplabel.setPreferredSize(new Dimension(10,15));
                mainsubpanelgroup.add (linestoplabel);
                }

        void constructTopGroupBottomLabels(JPanel mainsubpanelgroup)
                {
                JLabel dummylabel20 = new JLabel ("Set");
                dummylabel20.setHorizontalAlignment(JLabel.CENTER);
                mainsubpanelgroup.add (dummylabel20);
                JLabel dummylabel21 = new JLabel ("Plot");
                if (showToolTips) dummylabel21.setToolTipText("Show the dataset on the plot");
                dummylabel21.setHorizontalAlignment(JLabel.CENTER);
                mainsubpanelgroup.add (dummylabel21);
                JLabel autoscaleincludelabel = new JLabel ("A/S");
                if (showToolTips) autoscaleincludelabel.setToolTipText("Include this dataset in the Y-axis autoscale range");
                autoscaleincludelabel.setHorizontalAlignment(JLabel.CENTER);
                mainsubpanelgroup.add (autoscaleincludelabel);
                JLabel mmaglabel = new JLabel ("Mag");
                if (showToolTips) mmaglabel.setToolTipText("Plot data in relative magnitude");
                mmaglabel.setHorizontalAlignment(JLabel.CENTER);
                mainsubpanelgroup.add (mmaglabel);
                ycolumnlabelsub = new JLabel ("Y-data Column");
                ycolumnlabelsub.setHorizontalAlignment(JLabel.CENTER);
                mainsubpanelgroup.add (ycolumnlabelsub);
                JLabel useerrorlabel = new JLabel ("Use");
                if (showToolTips) useerrorlabel.setToolTipText("Plot Y-error bars using Y-error Column values");
                useerrorlabel.setHorizontalAlignment(JLabel.CENTER);
                mainsubpanelgroup.add (useerrorlabel);
                JLabel errorlabel = new JLabel ("Y-error Column");
                errorlabel.setHorizontalAlignment(JLabel.CENTER);
                if (showToolTips) errorlabel.setToolTipText("Error associated with Y-data");
                mainsubpanelgroup.add (errorlabel);
                operatorlabelsub = new JLabel ("Operator");
                operatorlabelsub.setHorizontalAlignment(JLabel.CENTER);
                if (showToolTips) operatorlabelsub.setToolTipText("Performs the selected operation on Y-data Column and Y-operator Column and propagates error if enabled");
                mainsubpanelgroup.add (operatorlabelsub);
                opcolumnlabelsub = new JLabel ("Y-operator Column");
                opcolumnlabelsub.setHorizontalAlignment(JLabel.CENTER);
                mainsubpanelgroup.add (opcolumnlabelsub);
                JLabel useoperrorlabel = new JLabel ("Use");
                if (showToolTips) useoperrorlabel.setToolTipText("Use Operator Error as error associated with Y-operator Column");
                useoperrorlabel.setHorizontalAlignment(JLabel.CENTER);
                mainsubpanelgroup.add (useoperrorlabel);
                JLabel operrorlabel = new JLabel ("Y-operator Error");
                operrorlabel.setHorizontalAlignment(JLabel.CENTER);
                mainsubpanelgroup.add (operrorlabel);
                markerlabelsub = new JLabel ("Marker Symbol");
                markerlabelsub.setHorizontalAlignment(JLabel.CENTER);
                mainsubpanelgroup.add (markerlabelsub);
                markercolorlabelsub = new JLabel ("Marker Color");
                markercolorlabelsub.setHorizontalAlignment(JLabel.CENTER);
                mainsubpanelgroup.add (markercolorlabelsub);
                JLabel lineslabel = new JLabel ("Lines");
                if (showToolTips) lineslabel.setToolTipText("Connect datapoints with lines");
                lineslabel.setHorizontalAlignment(JLabel.CENTER);
                mainsubpanelgroup.add (lineslabel);
                }

        void constructOtherGroupTopLabels(JPanel mainsubpanelgroup)
                {
                JLabel seconddatasettoplabel = new JLabel ("Data");
                seconddatasettoplabel.setHorizontalAlignment(JLabel.CENTER);
                if (!useWideDataPanel)
                        mainsubpanelgroup.add (seconddatasettoplabel);
                JLabel binstoplabel = new JLabel ("Samples");
                binstoplabel.setHorizontalAlignment(JLabel.CENTER);
                if (showToolTips) binstoplabel.setToolTipText("Plot binned data using the number of samples/bin indicated");
                mainsubpanelgroup.add (binstoplabel);

                JLabel autoscaletoplabel1 = new JLabel ("Scale Within Plot Range");
                if (showToolTips) autoscaletoplabel1.setToolTipText("Auto shift dataset to center of plot and scale to 100% of plot range");
                autoscaletoplabel1.setHorizontalAlignment(JLabel.CENTER);
                mainsubpanelgroup.add(autoscaletoplabel1);

                JLabel dummylabel139 = new JLabel ("");
                mainsubpanelgroup.add (dummylabel139);

                JLabel customscaletoplabel1 = new JLabel ("Scale Curve Directly");
                if (showToolTips) customscaletoplabel1.setToolTipText("Directly scale and shift dataset");
                customscaletoplabel1.setHorizontalAlignment(JLabel.CENTER);
                mainsubpanelgroup.add(customscaletoplabel1);

                JLabel morelegendstoplabel = new JLabel ("Legend Type");
                morelegendstoplabel.setHorizontalAlignment(JLabel.CENTER);
                morelegendstoplabel.setComponentPopupMenu(legendpopup);
                if (showToolTips) morelegendstoplabel.setToolTipText("Right click to set legend preferences");
                mainsubpanelgroup.add (morelegendstoplabel);

                JLabel morelegendsnametoplabel = new JLabel ("");
                morelegendsnametoplabel.setHorizontalAlignment(JLabel.CENTER);
                morelegendsnametoplabel.setComponentPopupMenu(legendpopup);
                if (showToolTips) morelegendsnametoplabel.setToolTipText("Right click to set legend preferences");
                mainsubpanelgroup.add (morelegendsnametoplabel);

                if (useWideDataPanel)
                        mainsubpanelgroup.add (seconddatasettoplabel);
                }

        void constructOtherGroupBottomLabels(JPanel mainsubpanelgroup)
                {
                JLabel seconddatasetlabel = new JLabel ("Set");
                seconddatasetlabel.setHorizontalAlignment(JLabel.CENTER);
                if (!useWideDataPanel)
                        mainsubpanelgroup.add (seconddatasetlabel);
                JLabel binslabel = new JLabel ("Samples/Bin");
                binslabel.setHorizontalAlignment(JLabel.CENTER);
                if (showToolTips) binslabel.setToolTipText("Plot binned data using the number of samples/bin indicated");
                mainsubpanelgroup.add (binslabel);
                JPanel autoscalelabelgroup = new JPanel();
                autoscalelabelgroup.setLayout(new BoxLayout(autoscalelabelgroup,BoxLayout.X_AXIS));
                JLabel autoscalelabel1 = new JLabel ("Auto");
                if (showToolTips) autoscalelabel1.setToolTipText("Auto shift dataset to center of plot and scale to 100% of plot range");
                autoscalelabel1.setHorizontalAlignment(JLabel.LEFT);
                autoscalelabelgroup.add(autoscalelabel1);
                autoscalelabelgroup.add(Box.createHorizontalGlue());
                JLabel autoscalelabel2 = new JLabel ("Scale%");
                if (showToolTips) autoscalelabel2.setToolTipText("Scale dataset to specified % of plot range");
                autoscalelabel2.setHorizontalAlignment(JLabel.CENTER);
                autoscalelabelgroup.add(autoscalelabel2);
                autoscalelabelgroup.add(Box.createHorizontalStrut(7));
                autoscalelabelgroup.add(Box.createHorizontalGlue());
                JLabel autoscalelabel3 = new JLabel ("Shift%");
                if (showToolTips) autoscalelabel3.setToolTipText("Shift dataset from plot center by specified % of plot range");
                autoscalelabel3.setHorizontalAlignment(JLabel.CENTER);
                autoscalelabelgroup.add(autoscalelabel3);
                autoscalelabelgroup.add(Box.createHorizontalStrut(7));
                mainsubpanelgroup.add (autoscalelabelgroup);

                JLabel dummylabel39 = new JLabel ("");
                mainsubpanelgroup.add (dummylabel39);

                JPanel customscalelabelgroup = new JPanel();
                customscalelabelgroup.setLayout(new BoxLayout(customscalelabelgroup,BoxLayout.X_AXIS));
                JLabel customscalelabel1 = new JLabel ("Custom");
                if (showToolTips) customscalelabel1.setToolTipText("Directly scale and shift dataset");
                customscalelabel1.setHorizontalAlignment(JLabel.LEFT);
                customscalelabelgroup.add(customscalelabel1);
                customscalelabelgroup.add(Box.createHorizontalGlue());
                JLabel customscalelabel2 = new JLabel ("scale by");
                if (showToolTips) customscalelabel2.setToolTipText("Scale dataset by specified value");
                customscalelabel2.setHorizontalAlignment(JLabel.CENTER);
                customscalelabelgroup.add(customscalelabel2);
                customscalelabelgroup.add(Box.createHorizontalStrut(17));
                customscalelabelgroup.add(Box.createHorizontalGlue());
                JLabel customscalelabel3 = new JLabel ("then add");
                if (showToolTips) customscalelabel3.setToolTipText("Add specified constant value to scaled dataset");
                customscalelabel3.setHorizontalAlignment(JLabel.CENTER);
                customscalelabelgroup.add(customscalelabel3);
                customscalelabelgroup.add(Box.createHorizontalStrut(17));
                mainsubpanelgroup.add (customscalelabelgroup);
                JLabel morelegendslabel = new JLabel ("Legend Type");
                morelegendslabel.setHorizontalAlignment(JLabel.CENTER);
                morelegendslabel.setComponentPopupMenu(legendpopup);
                if (showToolTips) morelegendslabel.setToolTipText("Right click to set legend preferences");
                mainsubpanelgroup.add (morelegendslabel);
                JLabel morelegendsnamelabel = new JLabel ("Custom Legend");
                morelegendsnamelabel.setHorizontalAlignment(JLabel.CENTER);
                morelegendsnamelabel.setComponentPopupMenu(legendpopup);
                if (showToolTips) morelegendsnamelabel.setToolTipText("Right click to set legend preferences");
                mainsubpanelgroup.add (morelegendsnamelabel);

                if (useWideDataPanel)
                        mainsubpanelgroup.add (seconddatasetlabel);
                }

        void constructTopGroup(JPanel mainsubpanelgroup, final int c)
                {

                        curvelabel[c] = new JLabel (""+(c+1));
                        curvelabel[c].setHorizontalAlignment(JLabel.CENTER);
                        mainsubpanelgroup.add (curvelabel[c]);

                        usecurvebox[c] = new JCheckBox ("", plotY[c]);
                        usecurvebox[c].addItemListener(new ItemListener(){
                            public void itemStateChanged(ItemEvent e) {
                                if (e.getStateChange() == ItemEvent.DESELECTED)
                                        plotY[c] = false;
                                else if (e.getStateChange() == ItemEvent.SELECTED)
                                        plotY[c] = true;
                                updatePlot(); }});
                        usecurvebox[c].setHorizontalAlignment(JLabel.CENTER);
                        mainsubpanelgroup.add (usecurvebox[c]);

                        autoscaleincludebox[c] = new JCheckBox ("", ASInclude[c]);
                        autoscaleincludebox[c].addItemListener(new ItemListener(){
                            public void itemStateChanged(ItemEvent e) {
                                if (e.getStateChange() == ItemEvent.DESELECTED)
                                        ASInclude[c] = false;
                                else if (e.getStateChange() == ItemEvent.SELECTED)
                                        ASInclude[c] = true;
                                updatePlot(); }});
                        autoscaleincludebox[c].setHorizontalAlignment(JLabel.CENTER);
                        mainsubpanelgroup.add (autoscaleincludebox[c]);

                        usemmagbox[c] = new JCheckBox ("", mmag[c]);
                        usemmagbox[c].addItemListener(new ItemListener(){
                            public void itemStateChanged(ItemEvent e) {
                                if (e.getStateChange() == ItemEvent.DESELECTED)
                                        mmag[c] = false;
                                else if (e.getStateChange() == ItemEvent.SELECTED)
                                        mmag[c] = true;
                                updatePlot(); }});
                        usemmagbox[c].setHorizontalAlignment(JLabel.CENTER);
                        mainsubpanelgroup.add (usemmagbox[c]);

                        ydatacolumn[c] = new JComboBox(columns);
                        ydatacolumn[c].setSelectedItem(ylabel[c]);
                        ydatacolumn[c].setPrototypeDisplayValue("123456789012345");
                        ydatacolumn[c].addActionListener(new ActionListener(){
                            public void actionPerformed(ActionEvent ae) {
                                ylabel[c] = (String) ydatacolumn[c].getSelectedItem();
                                updatePlot(); }});
                        mainsubpanelgroup.add (ydatacolumn[c]);

                        errorcolumnbox[c] = new JCheckBox ("", showErrors[c]);
                        errorcolumnbox[c].addItemListener(new ItemListener(){
                            public void itemStateChanged(ItemEvent e) {
                                if (e.getStateChange() == ItemEvent.DESELECTED)
                                        showErrors[c] = false;
                                else if (e.getStateChange() == ItemEvent.SELECTED)
                                        showErrors[c] = true;
                                updatePlot(); }});
                        errorcolumnbox[c].setHorizontalAlignment(JLabel.CENTER);
                        mainsubpanelgroup.add (errorcolumnbox[c]);

                        errorcolumn[c] = new JComboBox(columns);
                        errorcolumn[c].setSelectedItem(errlabel[c]);
                        errorcolumn[c].setPrototypeDisplayValue("123456789012345");
                        errorcolumn[c].addActionListener(new ActionListener(){
                            public void actionPerformed(ActionEvent ae) {
                                errlabel[c] = (String) errorcolumn[c].getSelectedItem();
                                updatePlot(); }});
                        mainsubpanelgroup.add (errorcolumn[c]);

                        operatorselection[c] = new JComboBox(operators);
                        operatorselection[c].setSelectedIndex(operatorIndex[c]);
                        operatorselection[c].setPrototypeDisplayValue("12345678901");
                        operatorselection[c].addActionListener(new ActionListener(){
                            public void actionPerformed(ActionEvent ae) {
                                operatorIndex[c] = operatorselection[c].getSelectedIndex();
                                updatePlot(); }});
                        mainsubpanelgroup.add (operatorselection[c]);

                        operatorcolumn[c] = new JComboBox(columns);
                        operatorcolumn[c].setSelectedItem(oplabel[c]);
                        operatorcolumn[c].setPrototypeDisplayValue("123456789012345");
                        operatorcolumn[c].addActionListener(new ActionListener(){
                            public void actionPerformed(ActionEvent ae) {
                                oplabel[c] = (String) operatorcolumn[c].getSelectedItem();
                                updatePlot(); }});
                        mainsubpanelgroup.add (operatorcolumn[c]);


                        operrorcolumnbox[c] = new JCheckBox ("", showOpErrors[c]);
                        operrorcolumnbox[c].addItemListener(new ItemListener(){
                            public void itemStateChanged(ItemEvent e) {
                                if (e.getStateChange() == ItemEvent.DESELECTED)
                                        showOpErrors[c] = false;
                                else if (e.getStateChange() == ItemEvent.SELECTED)
                                        showOpErrors[c] = true;
                                updatePlot(); }});
                        operrorcolumnbox[c].setHorizontalAlignment(JLabel.CENTER);
                        mainsubpanelgroup.add (operrorcolumnbox[c]);

                        operrorcolumn[c] = new JComboBox(columns);
                        operrorcolumn[c].setSelectedItem(operrlabel[c]);
                        operrorcolumn[c].setPrototypeDisplayValue("123456789012345");
                        operrorcolumn[c].addActionListener(new ActionListener(){
                            public void actionPerformed(ActionEvent ae) {
                                operrlabel[c] = (String) operrorcolumn[c].getSelectedItem();
                                updatePlot(); }});
                        mainsubpanelgroup.add (operrorcolumn[c]);

                        markersymbolselection[c] = new JComboBox(markers);
                        markersymbolselection[c].setPreferredSize(new Dimension(100, 25));
                        markersymbolselection[c].setSelectedIndex(markerIndex[c]);
                        markersymbolselection[c].setPrototypeDisplayValue("1234567890");
                        markersymbolselection[c].addActionListener(new ActionListener(){
                            public void actionPerformed(ActionEvent ae) {
                                markerIndex[c] = markersymbolselection[c].getSelectedIndex();
                                updatePlot(); }});
                        mainsubpanelgroup.add (markersymbolselection[c]);

                        markercolorselection[c] = new JComboBox(colors);
                        markercolorselection[c].setPreferredSize(new Dimension(100, 25));
                        markercolorselection[c].setSelectedIndex(colorIndex[c]);
                        markercolorselection[c].setPrototypeDisplayValue("1234567890");
                        markercolorselection[c].addActionListener(new ActionListener(){
                            public void actionPerformed(ActionEvent ae) {
                                colorIndex[c] = markercolorselection[c].getSelectedIndex();
                                updatePlot(); }});
                        mainsubpanelgroup.add (markercolorselection[c]);



                        uselinesbox[c] = new JCheckBox ("", lines[c]);
                        uselinesbox[c].addItemListener(new ItemListener(){
                            public void itemStateChanged(ItemEvent e) {
                                if (e.getStateChange() == ItemEvent.DESELECTED)
                                        lines[c] = false;
                                else if (e.getStateChange() == ItemEvent.SELECTED)
                                        lines[c] = true;
                                updatePlot(); }});
                        uselinesbox[c].setHorizontalAlignment(JLabel.CENTER);
                        mainsubpanelgroup.add (uselinesbox[c]);

                }

        void constructOtherGroup(JPanel mainsubpanelgroup, final int c)
                {
                if (!useWideDataPanel)
                        {
                        othercurvelabel[c] = new JLabel (""+(c+1));
                        othercurvelabel[c].setHorizontalAlignment(JLabel.CENTER);
                        mainsubpanelgroup.add (othercurvelabel[c]);
                        }

                binsizespinnermodel[c] = new SpinnerNumberModel(new Integer(binSize[c]),new Integer(1), null, new Integer(1));
                binsizespinner[c] = new JSpinner(binsizespinnermodel[c]);
                binsizespinner[c].addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        binSize[c] = ((Integer)binsizespinner[c].getValue()).intValue();
                        updatePlot();
                        }
                    });
                binsizespinner[c].addMouseWheelListener( new MouseWheelListener()
                    {
                    public void mouseWheelMoved( MouseWheelEvent e )
                        {
                        int newValue =  new Integer(((Integer)binsizespinner[c].getValue()).intValue() - e.getWheelRotation());
                        if (newValue > 0) binsizespinner[c].setValue(newValue);
                        }
                    });
                mainsubpanelgroup.add (binsizespinner[c]);

                autoscalepanelgroup[c] = new JPanel (new SpringLayout());
                autoscalepanelgroup[c].setBorder(BorderFactory.createLineBorder(Color.lightGray));
                customscalepanelgroup[c] = new JPanel (new SpringLayout());
                customscalepanelgroup[c].setBorder(BorderFactory.createLineBorder(Color.lightGray));
                autoscaleButton[c] = new JRadioButton("");
                customscaleButton[c] = new JRadioButton("");
                if (autoScale[c])
                        {
                        autoscaleButton[c].setSelected(true);
                        customscaleButton[c].setSelected(false);
                        }
                else
                        {
                        autoscaleButton[c].setSelected(false);
                        customscaleButton[c].setSelected(true);
                        }

                autoscalepanelgroup[c].add (autoscaleButton[c]);
                customscalepanelgroup[c].add (customscaleButton[c]);
                autoscaleRadioGroup[c] = new ButtonGroup();
                autoscaleRadioGroup[c].add(autoscaleButton[c]);
                autoscaleRadioGroup[c].add(customscaleButton[c]);

                autoscaleButton[c].addActionListener(new ActionListener(){
                    public void actionPerformed(ActionEvent ae) {
                        autoScale[c] = true;
                        updatePlot(); }});
                customscaleButton[c].addActionListener(new ActionListener(){
                    public void actionPerformed(ActionEvent ae) {
                        autoScale[c] = false;
                        updatePlot(); }});

        //AUTO SCALE SPINNER

                autoscalemodel[c] = new SpinnerNumberModel(new Double(autoScaleFactor[c]*100.), null, null, new Double(autoScaleStep[c]));
                autoscalespinner[c] = new JSpinner(autoscalemodel[c]);
                if (showToolTips) autoscalespinner[c].setToolTipText("Right click to set spinner stepsize");
                autoscalespinner[c].setPreferredSize(new Dimension(75,25));
                autoscalespinner[c].addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        autoScaleFactor[c]= ((Double)autoscalespinner[c].getValue()).doubleValue()/100. ;
                        updatePlot();
                        }
                    });
                autoscalespinner[c].addMouseWheelListener( new MouseWheelListener()
                    {
                    public void mouseWheelMoved( MouseWheelEvent e )
                        {
                        autoscalespinner[c].setValue(((Double)autoscalespinner[c].getValue()).doubleValue()
                                - e.getWheelRotation()*autoScaleStep[c]);
                        }
                    });


                autoscalepanelgroup[c].add (autoscalespinner[c]);

                //AUTO SCALE SPINNER STEPSIZE POPUP

                autoscalesteppopup[c] = new JPopupMenu();
                autoscalesteppanel[c] = new JPanel();
                autoscalestepmodel[c] = new SpinnerListModel(spinnerscalelist);
                autoscalestepspinner[c] = new JSpinner(autoscalestepmodel[c]);
                autoscalestepspinner[c].setValue(convertToText(autoScaleStep[c]));
                autoscalestepspinner[c].addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        autoScaleStep[c] = Double.parseDouble((String)autoscalestepspinner[c].getValue());
                        autoscalespinner[c].setModel(new SpinnerNumberModel(new Double(autoScaleFactor[c])*100., null, null, new Double(autoScaleStep[c])));
                        Prefs.set("plot.autoScaleStep"+c,autoScaleStep[c]);
                        }
                    });
                autoscalesteplabel[c] = new JLabel ("Stepsize:");
                autoscalesteppanel[c].add(autoscalesteplabel[c]);
                autoscalesteppanel[c].add(autoscalestepspinner[c]);
                autoscalesteppopup[c].add(autoscalesteppanel[c]);
                autoscalesteppopup[c].addPopupMenuListener (new PopupMenuListener() {
                    public void popupMenuCanceled(PopupMenuEvent popupMenuEvent) { }
                    public void popupMenuWillBecomeInvisible(PopupMenuEvent popupMenuEvent) { }
                    public void popupMenuWillBecomeVisible(PopupMenuEvent popupMenuEvent) { } });
                autoscalespinner[c].setComponentPopupMenu(autoscalesteppopup[c]);

        //AUTO SHIFT SPINNER

                autoshiftmodel[c] = new SpinnerNumberModel(new Double(autoShiftFactor[c]*100.), null, null, new Double(autoShiftStep[c]));
                autoshiftspinner[c] = new JSpinner(autoshiftmodel[c]);
                autoshiftspinner[c].setPreferredSize(new Dimension(75,25));
                if (showToolTips) autoshiftspinner[c].setToolTipText("Right click to set spinner stepsize");
                autoshiftspinner[c].addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        autoShiftFactor[c]= ((Double)autoshiftspinner[c].getValue()).doubleValue()/100. ;
                        updatePlot();
                        }
                    });
                autoshiftspinner[c].addMouseWheelListener( new MouseWheelListener()
                    {
                    public void mouseWheelMoved( MouseWheelEvent e )
                        {
                        autoshiftspinner[c].setValue(((Double)autoshiftspinner[c].getValue()).doubleValue()
                                - e.getWheelRotation()*autoShiftStep[c]);
                        }
                    });
                autoscalepanelgroup[c].add (autoshiftspinner[c]);

                //AUTO SHIFT SPINNER STEPSIZE POPUP

                autoshiftsteppopup[c] = new JPopupMenu();
                autoshiftsteppanel[c] = new JPanel();
                autoshiftstepmodel[c] = new SpinnerListModel(spinnerscalelist);
                autoshiftstepspinner[c] = new JSpinner(autoshiftstepmodel[c]);
                autoshiftstepspinner[c].setValue(convertToText(autoShiftStep[c]));
                autoshiftstepspinner[c].addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        autoShiftStep[c] = Double.parseDouble((String)autoshiftstepspinner[c].getValue());
                        autoshiftspinner[c].setModel(new SpinnerNumberModel(new Double(autoShiftFactor[c])*100., null, null, new Double(autoShiftStep[c])));
                        Prefs.set("plot.autoShiftStep"+c,autoShiftStep[c]);
                        }
                    });
                autoshiftsteplabel[c] = new JLabel ("Stepsize:");
                autoshiftsteppanel[c].add(autoshiftsteplabel[c]);
                autoshiftsteppanel[c].add(autoshiftstepspinner[c]);
                autoshiftsteppopup[c].add(autoshiftsteppanel[c]);
                autoshiftsteppopup[c].addPopupMenuListener (new PopupMenuListener() {
                    public void popupMenuCanceled(PopupMenuEvent popupMenuEvent) { }
                    public void popupMenuWillBecomeInvisible(PopupMenuEvent popupMenuEvent) { }
                    public void popupMenuWillBecomeVisible(PopupMenuEvent popupMenuEvent) { } });
                autoshiftspinner[c].setComponentPopupMenu(autoshiftsteppopup[c]);


        // GRAB AUTO SCALE AND SHIFT VALUES
                grabautopanel[c] = new JPanel (new SpringLayout());
                grabautobutton[c] = new JButton("Grab");
                if (showToolTips) grabautobutton[c].setToolTipText("Sends current RAW auto shift and scale values to custom values");

                grabautobutton[c].addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        customScaleFactor[c] = autoScaleFactor[c]*yRange/yWidth[c];
                        customscalespinner[c].setValue(customScaleFactor[c]);
                        customShiftFactor[c] = yMid + yRange*autoShiftFactor[c] - customScaleFactor[c]*yMidpoint[c];
                        customshiftspinner[c].setValue(customShiftFactor[c]);
                    }
                });
                grabautopanel[c].add (grabautobutton[c]);
                SpringUtil.makeCompactGrid (grabautopanel[c],1,1, 0,0,0,0);


        //CUSTOM SCALE SPINNER

                customscalemodel[c] = new SpinnerNumberModel(new Double(customScaleFactor[c]), null, null, new Double(customScaleStep[c]));
                customscalespinner[c] = new JSpinner(customscalemodel[c]);
                if (showToolTips) customscalespinner[c].setToolTipText("Right click to set spinner stepsize");
                customscalespinner[c].setPreferredSize(new Dimension(75,25));

                customscaleeditor[c] = (JSpinner.NumberEditor)customscalespinner[c].getEditor();
                customscaleformat[c] = customscaleeditor[c].getFormat();
                customscaleformat[c].setMaximumFractionDigits(9);

                customscalespinner[c].addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        customScaleFactor[c]= ((Double)customscalespinner[c].getValue()).doubleValue() ;
                        updatePlot();
                        }
                    });
                customscalespinner[c].addMouseWheelListener( new MouseWheelListener()
                    {
                    public void mouseWheelMoved( MouseWheelEvent e )
                        {
                        customscalespinner[c].setValue(((Double)customscalespinner[c].getValue()).doubleValue()
                                - e.getWheelRotation()*customScaleStep[c]);
                        }
                    });
                customscalepanelgroup[c].add (customscalespinner[c]);

                //CUSTOM SCALE SPINNER STEPSIZE POPUP

                customscalesteppopup[c] = new JPopupMenu();
                customscalesteppanel[c] = new JPanel();
                customscalestepmodel[c] = new SpinnerListModel(spinnerscalelist);
                customscalestepspinner[c] = new JSpinner(customscalestepmodel[c]);
                customscalestepspinner[c].setValue(convertToText(customScaleStep[c]));
                customscalestepspinner[c].addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        customScaleStep[c] = Double.parseDouble((String)customscalestepspinner[c].getValue());
                        customscalespinner[c].setModel(new SpinnerNumberModel(new Double(customScaleFactor[c]), null, null, new Double(customScaleStep[c])));
                        customscaleeditor[c] = (JSpinner.NumberEditor)customscalespinner[c].getEditor();
                        customscaleformat[c] = customscaleeditor[c].getFormat();
                        customscaleformat[c].setMaximumFractionDigits(9);
                        Prefs.set("plot.customScaleStep"+c,customScaleStep[c]);
                        }
                    });
                customscalesteplabel[c] = new JLabel ("Stepsize:");
                customscalesteppanel[c].add(customscalesteplabel[c]);
                customscalesteppanel[c].add(customscalestepspinner[c]);
                customscalesteppopup[c].add(customscalesteppanel[c]);
                customscalesteppopup[c].addPopupMenuListener (new PopupMenuListener() {
                    public void popupMenuCanceled(PopupMenuEvent popupMenuEvent) { }
                    public void popupMenuWillBecomeInvisible(PopupMenuEvent popupMenuEvent) { }
                    public void popupMenuWillBecomeVisible(PopupMenuEvent popupMenuEvent) { } });
                customscalespinner[c].setComponentPopupMenu(customscalesteppopup[c]);

        //CUSTOM SHIFT SPINNER

                customshiftmodel[c] = new SpinnerNumberModel(new Double(customShiftFactor[c]), null, null, new Double(customShiftStep[c]));
                customshiftspinner[c] = new JSpinner(customshiftmodel[c]);

                customshifteditor[c] = (JSpinner.NumberEditor)customshiftspinner[c].getEditor();
                customshiftformat[c] = customshifteditor[c].getFormat();
                customshiftformat[c].setMaximumFractionDigits(6);


                customshiftspinner[c].setPreferredSize(new Dimension(75,25));
                if (showToolTips) customshiftspinner[c].setToolTipText("Right click to set spinner stepsize");
                customshiftspinner[c].addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        customShiftFactor[c]= ((Double)customshiftspinner[c].getValue()).doubleValue();
                        updatePlot();
                        }
                    });
                customshiftspinner[c].addMouseWheelListener( new MouseWheelListener()
                    {
                    public void mouseWheelMoved( MouseWheelEvent e )
                        {
                        customshiftspinner[c].setValue(((Double)customshiftspinner[c].getValue()).doubleValue()
                                - e.getWheelRotation()*customShiftStep[c]);
                        }
                    });
                customscalepanelgroup[c].add (customshiftspinner[c]);

                //CUSTOM SCALE SPINNER STEPSIZE POPUP

                customshiftsteppopup[c] = new JPopupMenu();
                customshiftsteppanel[c] = new JPanel();
                customshiftstepmodel[c] = new SpinnerListModel(spinnerscalelist);
                customshiftstepspinner[c] = new JSpinner(customshiftstepmodel[c]);
                customshiftstepspinner[c].setValue(convertToText(customShiftStep[c]));
                customshiftstepspinner[c].addChangeListener (new ChangeListener()
                    {
                    public void stateChanged(ChangeEvent ev)
                        {
                        customShiftStep[c] = Double.parseDouble((String)customshiftstepspinner[c].getValue());
                        customshiftspinner[c].setModel(new SpinnerNumberModel(new Double(customShiftFactor[c]), null, null, new Double(customShiftStep[c])));
                        customshifteditor[c] = (JSpinner.NumberEditor)customshiftspinner[c].getEditor();
                        customshiftformat[c] = customshifteditor[c].getFormat();
                        customshiftformat[c].setMaximumFractionDigits(6);
                        Prefs.set("plot.customShiftStep"+c,customShiftStep[c]);
                        }
                    });
                customshiftsteplabel[c] = new JLabel ("Stepsize:");
                customshiftsteppanel[c].add(customshiftsteplabel[c]);
                customshiftsteppanel[c].add(customshiftstepspinner[c]);
                customshiftsteppopup[c].add(customshiftsteppanel[c]);
                customshiftsteppopup[c].addPopupMenuListener (new PopupMenuListener() {
                    public void popupMenuCanceled(PopupMenuEvent popupMenuEvent) { }
                    public void popupMenuWillBecomeInvisible(PopupMenuEvent popupMenuEvent) { }
                    public void popupMenuWillBecomeVisible(PopupMenuEvent popupMenuEvent) { } });
                customshiftspinner[c].setComponentPopupMenu(customshiftsteppopup[c]);
                SpringUtil.makeCompactGrid (autoscalepanelgroup[c], 1, 3, 0,0,0,0);
                SpringUtil.makeCompactGrid (customscalepanelgroup[c], 1, 3, 0,0,0,0);
                mainsubpanelgroup.add (autoscalepanelgroup[c]);
                mainsubpanelgroup.add (grabautopanel[c]);
                mainsubpanelgroup.add (customscalepanelgroup[c]);

        //LEGEND RADIO GROUP AND TEXT FIELD

                morelegendradiopanelgroup[c] = new JPanel (new SpringLayout());
                morelegendradiopanelgroup[c].setBorder(BorderFactory.createLineBorder(Color.lightGray));
                        legendnoneButton[c] = new JRadioButton("None");
                        if (!useLegend[c] && !useColumnName[c])
                                legendnoneButton[c].setSelected(true);
                        else
                                legendnoneButton[c].setSelected(false);
                morelegendradiopanelgroup[c].add (legendnoneButton[c]);
                        legendcolumnNameButton[c] = new JRadioButton("Column");
                        if (useColumnName[c])
                                legendcolumnNameButton[c].setSelected(true);
                        else
                                legendcolumnNameButton[c].setSelected(false);
                morelegendradiopanelgroup[c].add (legendcolumnNameButton[c]);
                        legendcustomNameButton[c] = new JRadioButton("Custom");
                        if (!useColumnName[c] && useLegend[c])
                                legendcustomNameButton[c].setSelected(true);
                        else
                                legendcustomNameButton[c].setSelected(false);
                morelegendradiopanelgroup[c].add (legendcustomNameButton[c]);

                        morelegendRadioGroup[c] = new ButtonGroup();
                        morelegendRadioGroup[c].add(legendnoneButton[c]);
                        morelegendRadioGroup[c].add(legendcolumnNameButton[c]);
                        morelegendRadioGroup[c].add(legendcustomNameButton[c]);
                        legendnoneButton[c].addActionListener(new ActionListener(){
                            public void actionPerformed(ActionEvent ae) {
                                useColumnName[c] = false;
                                useLegend[c] = false;
                                updatePlot(); }});
                        legendcustomNameButton[c].addActionListener(new ActionListener(){
                            public void actionPerformed(ActionEvent ae) {
                                useColumnName[c] = false;
                                useLegend[c] = true;
                                updatePlot(); }});
                        legendcolumnNameButton[c].addActionListener(new ActionListener(){
                            public void actionPerformed(ActionEvent ae) {
                                useColumnName[c] = true;
                                useLegend[c] = false;
                                updatePlot(); }});
                SpringUtil.makeCompactGrid (morelegendradiopanelgroup[c], 1, 3, 0,0,0,0);
                legendnoneButton[c].setComponentPopupMenu(legendpopup);
                if (showToolTips) legendnoneButton[c].setToolTipText("Right click to set legend preferences");
                legendcustomNameButton[c].setComponentPopupMenu(legendpopup);
                if (showToolTips) legendcustomNameButton[c].setToolTipText("Right click to set legend preferences");
                legendcolumnNameButton[c].setComponentPopupMenu(legendpopup);
                if (showToolTips) legendcolumnNameButton[c].setToolTipText("Right click to set legend preferences");

                mainsubpanelgroup.add (morelegendradiopanelgroup[c]);

                // LEGEND TEXT FIELD
                morelegendField[c] = new JTextField (legend[c]);
                morelegendField[c].setBorder (BorderFactory.createEmptyBorder(0,4,0,4));
                morelegendField[c].getDocument().addDocumentListener (new DocumentListener()
                    {
                    public void insertUpdate (DocumentEvent ev)
                        {
                        legend[c] = morelegendField[c].getText();
                        updatePlot();
                        }
                    public void removeUpdate (DocumentEvent ev)
                        {
                        legend[c] = morelegendField[c].getText();
                        updatePlot();
                        }
                    public void changedUpdate (DocumentEvent ev)
                        {
                        legend[c] = morelegendField[c].getText();
                        updatePlot();
                        }
                    });
                if (useWideDataPanel)
                        morelegendField[c].setPreferredSize (new Dimension(150,20));
                else
                        morelegendField[c].setPreferredSize (new Dimension(250,20));
                morelegendField[c].setHorizontalAlignment (JTextField.LEFT);
                morelegendField[c].setComponentPopupMenu(legendpopup);
                if (showToolTips) morelegendField[c].setToolTipText("Right click to set legend preferences");
                mainsubpanelgroup.add (morelegendField[c]);

                if (useWideDataPanel)
                        {
                        curvelabel[c] = new JLabel (""+(c+1));
                        curvelabel[c].setHorizontalAlignment(JLabel.CENTER);
                        mainsubpanelgroup.add (curvelabel[c]);
                        }
                }



        void saveAndClose()
                {
                mainFrameLocationX = mainFrame.getLocation().x;
                mainFrameLocationY = mainFrame.getLocation().y;
                Prefs.set("plot.mainFrameLocationX",mainFrameLocationX);
                Prefs.set("plot.mainFrameLocationY",mainFrameLocationY);
                if (subFrame.isShowing())
                        {
                        subFrameLocationX = subFrame.getLocation().x;
                        subFrameLocationY = subFrame.getLocation().y;
                        Prefs.set("plot.subFrameLocationX",subFrameLocationX);
                        Prefs.set("plot.subFrameLocationY",subFrameLocationY);
                        }
                plotFrame = WindowManager.getFrame("Plot of "+tableName);
                if (plotFrame != null)
                        {
                        plotFrameLocationX = plotFrame.getLocation().x;
                        plotFrameLocationY = plotFrame.getLocation().y;
                        Prefs.set("plot.plotFrameLocationX",plotFrameLocationX);
                        Prefs.set("plot.plotFrameLocationY",plotFrameLocationY);
                        }

                plotImageCanvas.removeMouseMotionListener(this);
                plotImageCanvas.removeMouseWheelListener(this);
                plotImageCanvas.removeMouseListener(this);

                savePreferences();    
                mainFrame.dispose();
                subFrame.dispose();
                }


        void getPreferences()
                {
                tableName = Prefs.get("plot.tableName",tableName);
                plotAutoMode=Prefs.get("plot.automode",plotAutoMode);
                xlabel=Prefs.get("plot.xlabel",xlabel);
                title=Prefs.get("plot.title", title);
                useTitle=Prefs.get("plot.useTitle", useTitle);
                subtitle=Prefs.get("plot.subtitle", subtitle);
                useSubtitle=Prefs.get("plot.useSubtitle", useSubtitle);
                titlePosX=Prefs.get("plot.titlePosX",titlePosX);
                titlePosY=Prefs.get("plot.titlePosY",titlePosY);
                subtitlePosX=Prefs.get("plot.subtitlePosX",subtitlePosX);
                subtitlePosY=Prefs.get("plot.subtitlePosY",subtitlePosY);
                legendPosX=Prefs.get("plot.legendPosX",legendPosX);
                legendPosY=Prefs.get("plot.legendPosY",legendPosY);
                xTics=Prefs.get("plot.xTics", xTics);
                yTics=Prefs.get("plot.yTics", yTics);
                xGrid=Prefs.get("plot.xGrid", xGrid);
                yGrid=Prefs.get("plot.yGrid", yGrid);
                xNumbers=Prefs.get("plot.xNumbers", xNumbers);
                yNumbers=Prefs.get("plot.yNumbers", yNumbers);
                autoScaleX=Prefs.get("plot.autoScaleX",autoScaleX);
                useFirstX=Prefs.get("plot.useFirstX",useFirstX);
                autoScaleY=Prefs.get("plot.autoScaleY",autoScaleY);
                xMin=Prefs.get("plot.xMin", xMin);
                xMax=Prefs.get("plot.xMax", xMax);
                xWidth=Prefs.get("plot.xWidth", xWidth);
                yMin=Prefs.get("plot.yMin", yMin);
                yMax=Prefs.get("plot.yMax", yMax);
                plotSizeX=(int)Prefs.get("plot.plotSizeX", plotSizeX);
                plotSizeY=(int)Prefs.get("plot.plotSizeY", plotSizeY);
                mainFrameLocationX=(int)Prefs.get("plot.mainFrameLocationX", mainFrameLocationX);
                mainFrameLocationY=(int)Prefs.get("plot.mainFrameLocationY", mainFrameLocationY);
                subFrameLocationX=(int)Prefs.get("plot.subFrameLocationX", subFrameLocationX);
                subFrameLocationY=(int)Prefs.get("plot.subFrameLocationY", subFrameLocationY);
                plotFrameLocationX=(int)Prefs.get("plot.plotFrameLocationX", plotFrameLocationX);
                plotFrameLocationY=(int)Prefs.get("plot.plotFrameLocationY", plotFrameLocationY);
                openDataSetWindow=Prefs.get("plot.openDataSetWindow", openDataSetWindow);
                rememberWindowLocations=Prefs.get("plot.rememberWindowLocations", rememberWindowLocations);
                mmagrefs=(int)Prefs.get("plot.mmagrefs", mmagrefs);
                xExponent=(int)Prefs.get("plot.xExponent", xExponent);
                yExponent=(int)Prefs.get("plot.yExponent", yExponent);
                xWidthStep=Prefs.get("plot.xWidthStep", xWidthStep);
                showToolTips=Prefs.get("plot.showToolTips", showToolTips);
                xMaxStep=Prefs.get("plot.xMaxStep", xMaxStep);
                xMinStep=Prefs.get("plot.xMinStep", xMinStep);
                yMaxStep=Prefs.get("plot.yMaxStep", yMaxStep);
                yMinStep=Prefs.get("plot.yMinStep", yMinStep);
                plotSizeXStep=(int)Prefs.get("plot.plotSizeXStep", plotSizeXStep);
                plotSizeYStep=(int)Prefs.get("plot.plotSizeYStep", plotSizeYStep);
                excludedHeadSamples=(int)Prefs.get("plot.excludedHeadSamples", excludedHeadSamples);
                excludedTailSamples=(int)Prefs.get("plot.excludedTailSamples", excludedTailSamples);
                excludedHeadSamplesStep=(int)Prefs.get("plot.excludedHeadSamplesStep", excludedHeadSamplesStep);
                excludedTailSamplesStep=(int)Prefs.get("plot.excludedTailSamplesStep", excludedTailSamplesStep);
                useXColumnName=Prefs.get("plot.useXColumnName", useXColumnName);
                useYColumnName=Prefs.get("plot.useYColumnName", useYColumnName);
                useXCustomName=Prefs.get("plot.useXCustomName", useXCustomName);
                useYCustomName=Prefs.get("plot.useYCustomName", useYCustomName);
                xLegend=Prefs.get("plot.xLegend", xLegend);
                yLegend=Prefs.get("plot.yLegend", yLegend);
                showXScaleInfo=Prefs.get("plot.showXScaleInfo", showXScaleInfo);
                showYScaleInfo=Prefs.get("plot.showYScaleInfo", showYScaleInfo);
                showYShiftInfo=Prefs.get("plot.showYShiftInfo", showYShiftInfo);
                showLScaleInfo=Prefs.get("plot.showLScaleInfo", showLScaleInfo);
                showLShiftInfo=Prefs.get("plot.showLShiftInfo", showLShiftInfo);
                showYBinInfo=Prefs.get("plot.showYBinInfo", showYBinInfo);
                showLBinInfo=Prefs.get("plot.showLBinInfo", showLBinInfo);
                showYmmagInfo=Prefs.get("plot.showYmmagInfo", showYmmagInfo);
                showLmmagInfo=Prefs.get("plot.showLmmagInfo", showLmmagInfo);
                showYSymbolInfo=Prefs.get("plot.showYSymbolInfo", showYSymbolInfo);
                showLSymbolInfo=Prefs.get("plot.showLSymbolInfo", showLSymbolInfo);
                useTwoLineLegend=Prefs.get("plot.useTwoLineLegend", useTwoLineLegend);
                useWideDataPanel=Prefs.get("plot.useWideDataPanel", useWideDataPanel);
                legendLeft=Prefs.get("plot.legendLeft", legendLeft);
                legendRight=Prefs.get("plot.legendRight", legendRight);

                for (int i=0; i<maxCurves; i++)
                        {
                        moreOptions[i]=Prefs.get("plot.moreOptions"+i,moreOptions[i]);
                        errlabel[i]=Prefs.get("plot.errlabel"+i,errlabel[i]);
                        oplabel[i]=Prefs.get("plot.oplabel"+i, oplabel[i]);
                        operrlabel[i]=Prefs.get("plot.operrlabel"+i,operrlabel[i]);
                        showErrors[i]=Prefs.get("plot.showErrors"+i,showErrors[i]);
                        showOpErrors[i]=Prefs.get("plot.showOpErrors"+i,showOpErrors[i]);
                        ylabel[i]=Prefs.get("plot.ylabel"+i,ylabel[i]);
                        lines[i]=Prefs.get("plot.lines"+i, lines[i]);
                        markerIndex[i]=(int)Prefs.get("plot.markerIndex"+i,markerIndex[i]);
                        colorIndex[i]=(int)Prefs.get("plot.colorIndex"+i, colorIndex[i]);
                        mmag[i]=Prefs.get("plot.mmag"+i, mmag[i]);
                        useColumnName[i]=Prefs.get("plot.useColumnName"+i, useColumnName[i]);
                        useLegend[i]=Prefs.get("plot.useLegend"+i, useLegend[i]);
                        legend[i] = Prefs.get("plot.legend"+i, legend[i]);
                        operatorIndex[i]=(int)Prefs.get("plot.operatorIndex"+i, operatorIndex[i]);
                        binSize[i]=(int)Prefs.get("plot.binSize"+i, binSize[i]);
                        plotY[i] = Prefs.get("plot.plotY"+i,plotY[i]);
                        autoScale[i] = Prefs.get("plot.autoScale"+i,autoScale[i]);
                        autoScaleFactor[i]=Prefs.get("plot.autoScaleFactor"+i,autoScaleFactor[i]);
                        autoScaleStep[i]=Prefs.get("plot.autoScaleStep"+i,autoScaleStep[i]);
                        autoShiftFactor[i]=Prefs.get("plot.autoShiftFactor"+i,autoShiftFactor[i]);
                        autoShiftStep[i]=Prefs.get("plot.autoShiftStep"+i,autoShiftStep[i]);
                        customScaleFactor[i]=Prefs.get("plot.customScaleFactor"+i,customScaleFactor[i]);
                        customScaleStep[i]=Prefs.get("plot.customScaleStep"+i,customScaleStep[i]);
                        customShiftFactor[i]=Prefs.get("plot.customShiftFactor"+i,customShiftFactor[i]);
                        customShiftStep[i]=Prefs.get("plot.customShiftStep"+i,customShiftStep[i]);
                        ASInclude[i]=Prefs.get("plot.ASInclude"+i,ASInclude[i]);
                        }
                }
        
        
        
        void savePreferences()
                {
                Prefs.set("plot.useDefaultSettings", useDefaultSettings);
                Prefs.set("plot.xlabel",xlabel);
                Prefs.set("plot.xTics", xTics);
                Prefs.set("plot.yTics", yTics);
                Prefs.set("plot.xGrid", xGrid);
                Prefs.set("plot.yGrid", yGrid);
                Prefs.set("plot.xNumbers", xNumbers);
                Prefs.set("plot.yNumbers", yNumbers);
                Prefs.set("plot.autoScaleX", autoScaleX);
                Prefs.set("plot.autoScaleY", autoScaleY);
                Prefs.set("plot.useFirstX", useFirstX);
                Prefs.set("plot.xWidth", xWidth);
                Prefs.set("plot.xMin", xMin);
                Prefs.set("plot.xMax", xMax);
                Prefs.set("plot.yMin", yMin);
                Prefs.set("plot.yMax", yMax);
                Prefs.set("plot.plotSizeX",plotSizeX);
                Prefs.set("plot.plotSizeY",plotSizeY);
                Prefs.set("plot.title",title);
                Prefs.set("plot.useTitle",useTitle);
                Prefs.set("plot.subtitle",subtitle);
                Prefs.set("plot.useSubtitle",useSubtitle);
                Prefs.set("plot.titlePosX",titlePosX);
                Prefs.set("plot.titlePosY",titlePosY);
                Prefs.set("plot.subtitlePosX",subtitlePosX);
                Prefs.set("plot.subtitlePosY",subtitlePosY);
                Prefs.set("plot.openDataSetWindow",openDataSetWindow);
                Prefs.set("plot.legendPosX",legendPosX);
                Prefs.set("plot.legendPosY",legendPosY);
                Prefs.set("plot.rememberWindowLocations",rememberWindowLocations);
                Prefs.set("plot.mmagrefs",mmagrefs);
                Prefs.set("plot.xExponent",xExponent);
                Prefs.set("plot.yExponent",yExponent);
                Prefs.set("plot.xWidthStep",xWidthStep);
                Prefs.set("plot.xMaxStep",xMaxStep);
                Prefs.set("plot.xMinStep",xMinStep);
                Prefs.set("plot.yMaxStep",yMaxStep);
                Prefs.set("plot.yMinStep",yMinStep);
                Prefs.set("plot.plotSizeXStep",plotSizeXStep);
                Prefs.set("plot.plotSizeYStep",plotSizeYStep);
                Prefs.set("plot.excludedHeadSamples", excludedHeadSamples);
                Prefs.set("plot.excludedTailSamples", excludedTailSamples);
                Prefs.set("plot.excludedHeadSamplesStep", excludedHeadSamplesStep);
                Prefs.set("plot.excludedTailSamplesStep", excludedTailSamplesStep);
                Prefs.set("plot.useXColumnName", useXColumnName);
                Prefs.set("plot.useYColumnName", useYColumnName);
                Prefs.set("plot.useXCustomName", useXCustomName);
                Prefs.set("plot.useYCustomName", useYCustomName);
                Prefs.set("plot.xLegend", xLegend);
                Prefs.set("plot.yLegend", yLegend);
                Prefs.set("plot.showXScaleInfo", showXScaleInfo);
                Prefs.set("plot.showYScaleInfo", showYScaleInfo);
                Prefs.set("plot.showYShiftInfo", showYShiftInfo);
                Prefs.set("plot.showLScaleInfo", showLScaleInfo);
                Prefs.set("plot.showLShiftInfo", showLShiftInfo);
                Prefs.set("plot.showYBinInfo", showYBinInfo);
                Prefs.set("plot.showLBinInfo", showLBinInfo);
                Prefs.set("plot.showYmmagInfo", showYmmagInfo);
                Prefs.set("plot.showLmmagInfo", showLmmagInfo);
                Prefs.set("plot.showYSymbolInfo", showYSymbolInfo);
                Prefs.set("plot.showLSymbolInfo", showLSymbolInfo);
                Prefs.set("plot.showLSymbolInfo", showLSymbolInfo);
                Prefs.set("plot.useTwoLineLegend", useTwoLineLegend);
                Prefs.set("plot.legendLeft", legendLeft);
                Prefs.set("plot.legendRight", legendRight);

                for (int i=0; i<maxCurves; i++)
                        {
                        Prefs.set("plot.ylabel"+i,ylabel[i]);
                        Prefs.set("plot.errlabel"+i,errlabel[i]);
                        Prefs.set("plot.oplabel"+i,oplabel[i]);
                        Prefs.set("plot.operrlabel"+i,operrlabel[i]);
                        Prefs.set("plot.showErrors"+i,showErrors[i]);
                        Prefs.set("plot.showOpErrors"+i,showOpErrors[i]);
                        Prefs.set("plot.lines"+i, lines[i]);
                        Prefs.set("plot.markerIndex"+i,markerIndex[i]);
                        Prefs.set("plot.colorIndex"+i,colorIndex[i]);
                        Prefs.set("plot.mmag"+i, mmag[i]);
                        Prefs.set("plot.useColumnName"+i,useColumnName[i]);
                        Prefs.set("plot.useLegend"+i,useLegend[i]);
                        Prefs.set("plot.legend"+i,legend[i]);
                        Prefs.set("plot.operatorIndex"+i,operatorIndex[i]);
                        Prefs.set("plot.moreOptions"+i,moreOptions[i]);
                        Prefs.set("plot.binSize"+i,binSize[i]);
                        Prefs.set("plot.plotY"+i,plotY[i]);
                        Prefs.set("plot.autoScale"+i,autoScale[i]);
                        Prefs.set("plot.autoScaleFactor"+i,autoScaleFactor[i]);
                        Prefs.set("plot.autoScaleStep"+i,autoScaleStep[i]);
                        Prefs.set("plot.autoShiftFactor"+i,autoShiftFactor[i]);
                        Prefs.set("plot.autoShiftStep"+i,autoShiftStep[i]);
                        Prefs.set("plot.customScaleFactor"+i,customScaleFactor[i]);
                        Prefs.set("plot.customScaleStep"+i,customScaleStep[i]);
                        Prefs.set("plot.customShiftFactor"+i,customShiftFactor[i]);
                        Prefs.set("plot.customShiftStep"+i,customShiftStep[i]);
                        Prefs.set("plot.ASInclude"+i,ASInclude[i]);
                        }
                }
	}

